@@ -5,48 +5,37 @@
 use crate::command::LogOptions;
 use crate::logging::Level;
 use crate::marionette::MarionetteSettings;
+use base64::prelude::BASE64_STANDARD;
+use base64::Engine;
 use mozdevice::AndroidStorageInput;
 use mozprofile::preferences::Pref;
 use mozprofile::profile::Profile;
 use mozrunner::firefox_args::{get_arg_value, parse_args, Arg};
 use mozrunner::runner::platform::firefox_default_path;
-use mozversion::{self, firefox_binary_version, firefox_version, Version};
+use mozversion::{firefox_binary_version, firefox_version, Version};
 use regex::bytes::Regex;
 use serde_json::{Map, Value};
 use std::collections::BTreeMap;
 use std::default::Default;
 use std::ffi::OsString;
-use std::fmt::{self, Display};
 use std::fs;
 use std::io;
 use std::io::BufWriter;
 use std::io::Cursor;
 use std::path::{Path, PathBuf};
 use std::str::{self, FromStr};
+use thiserror::Error;
 use webdriver::capabilities::{BrowserCapabilities, Capabilities};
 use webdriver::error::{ErrorStatus, WebDriverError, WebDriverResult};
 
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, Error)]
 enum VersionError {
-    VersionError(mozversion::Error),
+    #[error(transparent)]
+    VersionError(#[from] mozversion::Error),
+    #[error("No binary provided")]
     MissingBinary,
 }
 
-impl From<mozversion::Error> for VersionError {
-    fn from(err: mozversion::Error) -> VersionError {
-        VersionError::VersionError(err)
-    }
-}
-
-impl Display for VersionError {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        match *self {
-            VersionError::VersionError(ref x) => x.fmt(f),
-            VersionError::MissingBinary => "No binary provided".fmt(f),
-        }
-    }
-}
-
 impl From<VersionError> for WebDriverError {
     fn from(err: VersionError) -> WebDriverError {
         WebDriverError::new(ErrorStatus::SessionNotCreated, err.to_string())
@@ -123,7 +112,7 @@ impl<'a> FirefoxCapabilities<'a> {
     }
 }
 
-impl<'a> BrowserCapabilities for FirefoxCapabilities<'a> {
+impl BrowserCapabilities for FirefoxCapabilities<'_> {
     fn init(&mut self, capabilities: &Capabilities) {
         self.set_binary(capabilities);
     }
@@ -182,6 +171,26 @@ impl<'a> BrowserCapabilities for FirefoxCapabilities<'a> {
         Ok(true)
     }
 
+    fn webauthn_virtual_authenticators(&mut self, _: &Capabilities) -> WebDriverResult<bool> {
+        Ok(true)
+    }
+
+    fn webauthn_extension_uvm(&mut self, _: &Capabilities) -> WebDriverResult<bool> {
+        Ok(false)
+    }
+
+    fn webauthn_extension_prf(&mut self, _: &Capabilities) -> WebDriverResult<bool> {
+        Ok(false)
+    }
+
+    fn webauthn_extension_large_blob(&mut self, _: &Capabilities) -> WebDriverResult<bool> {
+        Ok(false)
+    }
+
+    fn webauthn_extension_cred_blob(&mut self, _: &Capabilities) -> WebDriverResult<bool> {
+        Ok(false)
+    }
+
     fn validate_custom(&mut self, name: &str, value: &Value) -> WebDriverResult<()> {
         if !name.starts_with("moz:") {
             return Ok(());
@@ -306,14 +315,6 @@ impl<'a> BrowserCapabilities for FirefoxCapabilities<'a> {
                     }
                 }
             }
-            "moz:useNonSpecCompliantPointerOrigin" => {
-                if !value.is_boolean() {
-                    return Err(WebDriverError::new(
-                        ErrorStatus::InvalidArgument,
-                        "moz:useNonSpecCompliantPointerOrigin is not a boolean",
-                    ));
-                }
-            }
             "moz:webdriverClick" => {
                 if !value.is_boolean() {
                     return Err(WebDriverError::new(
@@ -322,6 +323,7 @@ impl<'a> BrowserCapabilities for FirefoxCapabilities<'a> {
                     ));
                 }
             }
+            // Bug 1967916: Remove when Firefox 140 is no longer supported
             "moz:debuggerAddress" => {
                 if !value.is_boolean() {
                     return Err(WebDriverError::new(
@@ -367,19 +369,14 @@ impl AndroidOptions {
     }
 }
 
-#[derive(Debug, PartialEq)]
+#[derive(Debug, Default, PartialEq)]
 pub enum ProfileType {
     Path(Profile),
     Named,
+    #[default]
     Temporary,
 }
 
-impl Default for ProfileType {
-    fn default() -> Self {
-        ProfileType::Temporary
-    }
-}
-
 /// Rust representation of `moz:firefoxOptions`.
 ///
 /// Calling `FirefoxOptions::from_capabilities(binary, capabilities)` causes
@@ -432,10 +429,9 @@ impl FirefoxOptions {
             rv.env = FirefoxOptions::load_env(options)?;
             rv.log = FirefoxOptions::load_log(options)?;
             rv.prefs = FirefoxOptions::load_prefs(options)?;
-            if let Some(profile) = FirefoxOptions::load_profile(
-                settings.profile_root.as_ref().map(|x| x.as_path()),
-                options,
-            )? {
+            if let Some(profile) =
+                FirefoxOptions::load_profile(settings.profile_root.as_deref(), options)?
+            {
                 rv.profile = ProfileType::Path(profile);
             }
         }
@@ -544,16 +540,6 @@ impl FirefoxOptions {
             }
         }
 
-        // Force Fission disabled until the CDP implementation is compatible,
-        // and preference hasn't been already set
-        if has_debugger_address {
-            let has_fission_pref = rv.prefs.iter().find(|&x| x.0 == "fission.autostart");
-            if has_fission_pref.is_none() {
-                rv.prefs
-                    .push(("fission.autostart".to_owned(), Pref::new(false)));
-            }
-        }
-
         Ok(rv)
     }
 
@@ -565,7 +551,7 @@ impl FirefoxOptions {
             let profile_base64 = profile_json.as_str().ok_or_else(|| {
                 WebDriverError::new(ErrorStatus::InvalidArgument, "Profile is not a string")
             })?;
-            let profile_zip = &*base64::decode(profile_base64)?;
+            let profile_zip = &*BASE64_STANDARD.decode(profile_base64)?;
 
             // Create an emtpy profile directory
             let profile = Profile::new(profile_root)?;
@@ -691,7 +677,7 @@ impl FirefoxOptions {
 
             // https://developer.android.com/studio/build/application-id
             let package_regexp =
-                Regex::new(r#"^([a-zA-Z][a-zA-Z0-9_]*\.){1,}([a-zA-Z][a-zA-Z0-9_]*)$"#).unwrap();
+                Regex::new(r"^([a-zA-Z][a-zA-Z0-9_]*\.){1,}([a-zA-Z][a-zA-Z0-9_]*)$").unwrap();
             if !package_regexp.is_match(package.as_bytes()) {
                 return Err(WebDriverError::new(
                     ErrorStatus::InvalidArgument,
@@ -877,7 +863,7 @@ mod tests {
         let mut profile_data = Vec::with_capacity(1024);
         let mut profile = File::open("src/tests/profile.zip").unwrap();
         profile.read_to_end(&mut profile_data).unwrap();
-        Value::String(base64::encode(&profile_data))
+        Value::String(BASE64_STANDARD.encode(&profile_data))
     }
 
     fn make_options(
@@ -1095,11 +1081,6 @@ mod tests {
         } else {
             panic!("CLI arguments for Firefox not found");
         }
-
-        assert!(opts
-            .prefs
-            .iter()
-            .any(|pref| pref == &("fission.autostart".to_owned(), Pref::new(false))));
     }
 
     #[test]
