@@ -4,6 +4,12 @@ function getLastEventDetails(browser) {
   });
 }
 
+add_setup(async function () {
+  await SpecialPowers.pushPrefEnv({
+    set: [["test.wait300msAfterTabSwitch", true]],
+  });
+});
+
 add_task(async function () {
   let onClickEvt =
     'document.getElementById("out").textContent = event.target.localName + "," + event.clientX + "," + event.clientY;';
