@@ -1,8 +1,8 @@
 setExpectedFailuresForSelfTest(1);
 
 // The test will fail because an expected uncaught rejection is actually caught.
-const { PromiseTestUtils } = ChromeUtils.import(
-  "resource://testing-common/PromiseTestUtils.jsm"
+const { PromiseTestUtils } = ChromeUtils.importESModule(
+  "resource://testing-common/PromiseTestUtils.sys.mjs"
 );
 PromiseTestUtils.expectUncaughtRejection(/Promise rejection./);
 
