@@ -8,8 +8,6 @@
 user_pref("browser.newtabpage.enabled", false);
 // Don't restore the last open set of tabs if the browser has crashed
 user_pref("browser.sessionstore.resume_from_crash", false);
-//  Disable session restore infobar.
-user_pref("browser.startup.couldRestoreSession.count", -1);
 // Don't show the Bookmarks Toolbar on any tab (the above pref that
 // disables the New Tab Page ends up showing the toolbar on about:blank).
 user_pref("browser.toolbars.bookmarks.visibility", "never");
@@ -21,14 +19,16 @@ user_pref("dom.testing.testutils.enabled", true);
 user_pref("extensions.autoDisableScopes", 10);
 // Don't open a dialog to show available add-on updates
 user_pref("extensions.update.notifyUser", false);
+// For cross-browser tests, don't fail on manifest warnings like unknown keys.
+user_pref("extensions.webextensions.warnings-as-errors", false);
+// Adjust behavior of browser.test API to be compatible across engines.
+user_pref("extensions.wpt.enabled", true);
 // Enable test mode to run multiple tests in parallel
 user_pref("focusmanager.testmode", true);
 // Enable fake media streams for getUserMedia
 user_pref("media.navigator.streams.fake", true);
 // Disable permission prompt for getUserMedia
 user_pref("media.navigator.permission.disabled", true);
-// Enable pre-fetching of resources
-user_pref("network.preload", true);
 // Enable direct connection
 user_pref("network.proxy.type", 0);
 // Web-platform-tests load a lot of URLs very quickly. This puts avoidable and
@@ -46,47 +46,70 @@ user_pref("network.http.phishy-userpass-length", 255);
 // Disable safebrowsing components
 user_pref("browser.safebrowsing.blockedURIs.enabled", false);
 user_pref("browser.safebrowsing.downloads.enabled", false);
-user_pref("browser.safebrowsing.passwords.enabled", false);
 user_pref("browser.safebrowsing.malware.enabled", false);
 user_pref("browser.safebrowsing.phishing.enabled", false);
 user_pref("browser.safebrowsing.update.enabled", false);
-// Automatically unload beforeunload alerts
-user_pref("dom.disable_beforeunload", true);
-// Enable implicit keyframes since the common animation interpolation test
-// function assumes this is available.
-user_pref("dom.animations-api.implicit-keyframes.enabled", true);
 // Disable high DPI
-user_pref("layout.css.devPixelsPerPx", "1.0")
+user_pref("layout.css.devPixelsPerPx", "1.0");
 // Enable the parallel styling code.
-user_pref("layout.css.stylo-threads", 4)
+user_pref("layout.css.stylo-threads", 4);
 // sometime wpt runs test even before the document becomes visible, which would
 // delay video.play() and cause play() running in wrong order.
 user_pref("media.block-autoplay-until-in-foreground", false);
 // Disable dark scrollbars as it can be semi-transparent that many reftests
 // don't expect.
 user_pref("widget.disable-dark-scrollbar", true);
+// The Ubuntu Yaru theme has semi-transparent scrollbar track, which some tests
+// assume doesn't exist.
+user_pref("widget.gtk.theme-scrollbar-colors.enabled", false);
+// Disable scrollbar animations. Otherwise reftests that use overlay scrollbars
+// (only Android right now), might get a snapshot at different times during the
+// animation.
+user_pref("ui.scrollbarFadeDuration", 0);
 // Don't enable paint suppression when the background is unknown. While paint
 // is suppressed, synthetic click events and co. go to the old page, which can
 // be confusing for tests that send click events before the first paint.
 user_pref("nglayout.initialpaint.unsuppress_with_no_background", true);
 user_pref("media.block-autoplay-until-in-foreground", false);
-// Force a light color scheme unless explicitly overriden by pref.
+// Force a light color scheme unless explicitly overridden by pref.
 user_pref("layout.css.prefers-color-scheme.content-override", 1);
-// Force OffscreenCanvas support
-user_pref("gfx.offscreencanvas.enabled", true);
-user_pref("dom.workers.requestAnimationFrame", true);
 // A lot of tests use the Reporting API for observing things
 user_pref("dom.reporting.enabled", true);
-user_pref("layout.css.font-loading-api.workers.enabled", true);
 // Enable WebDriver BiDi experimental commands and events during tests.
 user_pref("remote.experimental.enabled", true);
-// Disable always partitioning storage with the Storage Access API
-user_pref("privacy.partition.always_partition_third_party_non_cookie_storage", false);
 // Disable OCSP checks in WPT (webtransport triggers these occasionally)
 user_pref("security.OCSP.enabled", 0);
 // Disable download of intermediate certificates.
 user_pref("security.remote_settings.intermediates.enabled", false);
 // Disable prefers-reduced-motion to ensure that smooth scrolls can be tested.
 user_pref("general.smoothScroll", true);
+// Prevent default handlers being added, since these can cause network fetches
+user_pref("gecko.handlerService.defaultHandlersVersion", 100);
+// Enable virtual WebAuthn authenticators.
+user_pref("security.webauth.webauthn_enable_softtoken", true);
+// Disable hardware WebAuthn authenticators.
+user_pref("security.webauth.webauthn_enable_usbtoken", false);
+// Disable the WebAuthn direct attestation consent prompt.
+user_pref("security.webauth.webauthn_testing_allow_direct_attestation", true);
+// Enable WebAuthn conditional mediation.
+user_pref("security.webauthn.enable_conditional_mediation", true);
 // Disable captive portal service
 user_pref("network.captive-portal-service.enabled", false);
+// Enable http2 websockets support
+user_pref("network.http.http2.websockets", true);
+// Turn off update
+user_pref("app.update.disabledForTesting", true);
+// Use dummy server for geolocation
+user_pref("geo.provider.network.url", "https://web-platform.test:8444/_mozilla/geolocation-API/dummy.py");
+// If we are on a platform where we can detect that we don't have OS
+// geolocation permission, and we can open it and wait for the user to give
+// permission, then don't do that.
+user_pref("geo.prompt.open_system_prefs", false);
+
+// prefs to force font, specifically from ubuntu 18.04
+user_pref("font.name.serif.x-western", "DejaVu Serif");
+user_pref("font.name.sans-serif.x-western", "DejaVu Sans");
+user_pref("font.name.monospace.x-western", "DejaVu Sans Mono");
+user_pref("font.name.serif.x-unicode", "DejaVu Serif");
+user_pref("font.name.sans-serif.x-unicode", "DejaVu Sans");
+user_pref("font.name.monospace.x-unicode", "DejaVu Sans Mono");
