@@ -5,20 +5,27 @@ import json
 import os
 import pathlib
 import re
+from urllib.parse import (
+    parse_qs,
+    unquote,
+    urlencode,
+    urlsplit,
+    urlunsplit,
+)
 
+from cmdline import DESKTOP_APPS, GECKO_PROFILER_APPS, TRACE_APPS
 from constants.raptor_tests_constants import YOUTUBE_PLAYBACK_MEASURE
 from logger.logger import RaptorLogger
 from manifestparser import TestManifest
-from six.moves.urllib.parse import parse_qs, unquote, urlencode, urlsplit, urlunsplit
+from support_class_utils import import_support_class
 from utils import (
     bool_from_str,
-    import_support_class,
     transform_platform,
     transform_subtest,
 )
 
 here = os.path.abspath(os.path.dirname(__file__))
-raptor_ini = os.path.join(here, "raptor.ini")
+raptor_toml = os.path.join(here, "raptor.toml")
 tests_dir = os.path.join(here, "tests")
 LOG = RaptorLogger(component="raptor-manifest")
 
@@ -41,23 +48,36 @@ playback_settings = [
 ]
 
 
-def filter_app(tests, values):
+def filter_app(tests, values, strict=False):
     for test in tests:
-        if values["app"] in test["apps"]:
+        if values["app"] in [app.strip() for app in test["apps"].split(",")]:
             yield test
 
 
 def get_browser_test_list(browser_app, run_local):
-    LOG.info(raptor_ini)
-    test_manifest = TestManifest([raptor_ini], strict=False)
+    LOG.info(raptor_toml)
+    test_manifest = TestManifest([raptor_toml], strict=False)
     info = {"app": browser_app, "run_local": run_local}
     return test_manifest.active_tests(
         exists=False, disabled=False, filters=[filter_app], **info
     )
 
 
-def validate_test_ini(test_details):
-    # validate all required test details were found in the test INI
+def validate_app_playback_settings(test_details):
+    # For Linux desktop playback tests, we need to augment the playback_settings
+    # to ensure that the right recording archive is used. Since Android tests
+    # are run on a Linux host, we should ensure that Android tests are not
+    # affected.
+
+    # This is temporary until Linux machines are upgraded (Bug 1926419)
+    apps = [app.strip() for app in test_details["apps"].split(",")]
+    return any(app in apps for app in DESKTOP_APPS) and test_details.get(
+        "playback_pageset_manifest_backup"
+    )
+
+
+def validate_test_toml(test_details):
+    # validate all required test details were found in the test TOML
     valid_settings = True
 
     for setting in required_settings:
@@ -83,7 +103,11 @@ def validate_test_ini(test_details):
 
     # if playback is specified, we need more playback settings
     if test_details.get("playback") is not None:
-        for setting in playback_settings:
+        # Ensure correct values used for android playback
+        pbs = playback_settings
+        if validate_app_playback_settings(test_details):
+            pbs += ["playback_pageset_manifest_backup"]
+        for setting in pbs:
             if test_details.get(setting) is None:
                 valid_settings = False
                 LOG.error(
@@ -92,14 +116,16 @@ def validate_test_ini(test_details):
                 )
 
     # if 'alert-on' is specified, we need to make sure that the value given is valid
-    # i.e. any 'alert_on' values must be values that exist in the 'measure' ini setting
+    # i.e. any 'alert_on' values must be values that exist in the 'measure' toml setting
     if test_details.get("alert_on") is not None:
-
         # support with or without spaces, i.e. 'measure = fcp, loadtime' or '= fcp,loadtime'
         # convert to a list; and remove any spaces
+        # when switching to .toml, values are \n separated, convert to a ',' and this will
+        # support scenarios where we have older .ini format as well as embedded commas
         # this can also have regexes inside
         test_details["alert_on"] = [
-            _item.strip() for _item in test_details["alert_on"].split(",")
+            _item.strip()
+            for _item in test_details["alert_on"].replace("\n", ",").split(",")
         ]
 
         # this variable will store all the concrete values for alert_on elements
@@ -144,7 +170,9 @@ def validate_test_ini(test_details):
                 "`repository_revision` is required when a `repository` is defined."
             )
             valid_settings = False
-        elif test_details.get("type") not in ("benchmark"):
+        elif test_details.get("type") not in ("benchmark") and not test_details.get(
+            "benchmark_webserver", False
+        ):
             LOG.error("`repository` is only available for benchmark test types.")
             valid_settings = False
 
@@ -202,7 +230,7 @@ def write_test_settings_json(args, test_details, oskey):
         test_settings["raptor-options"]["measure"] = {}
 
         # test_details['measure'] was already converted to a list in get_raptor_test_list below
-        # the 'hero=' line is still a raw string from the test INI
+        # the 'hero=' line is still a raw string from the test TOML
         for m in test_details["measure"]:
             test_settings["raptor-options"]["measure"][m] = True
             if m == "hero":
@@ -257,8 +285,7 @@ def write_test_settings_json(args, test_details, oskey):
 
     # if Gecko profiling is enabled, write profiling settings for webext
     if test_details.get("gecko_profile", False):
-        threads = ["GeckoMain", "Compositor"]
-        threads.extend(["Renderer", "WR"])
+        threads = ["GeckoMain", "Compositor", "Renderer"]
 
         if test_details.get("gecko_profile_threads"):
             # pylint --py3k: W1639
@@ -273,7 +300,7 @@ def write_test_settings_json(args, test_details, oskey):
                 "gecko_profile_entries": int(
                     test_details.get("gecko_profile_entries", 1000000)
                 ),
-                "gecko_profile_interval": int(
+                "gecko_profile_interval": float(
                     test_details.get("gecko_profile_interval", 1)
                 ),
                 "gecko_profile_threads": ",".join(set(threads)),
@@ -311,30 +338,30 @@ def write_test_settings_json(args, test_details, oskey):
         with open(settings_file, "w") as out_file:
             json.dump(test_settings, out_file, indent=4, ensure_ascii=False)
             out_file.close()
-    except IOError:
+    except OSError:
         LOG.info("abort: exception writing test settings json!")
 
 
 def get_raptor_test_list(args, oskey):
     """
-    A test ini (i.e. raptor-firefox-tp6.ini) will have one or more subtests inside,
-    each with it's own name ([the-ini-file-test-section]).
+    A test toml (i.e. raptor-firefox-tp6.toml) will have one or more subtests inside,
+    each with it's own name ([the-toml-file-test-section]).
 
     We want the ability to eiter:
-        - run * all * of the subtests listed inside the test ini; - or -
-        - just run a single one of those subtests that are inside the ini
+        - run * all * of the subtests listed inside the test toml; - or -
+        - just run a single one of those subtests that are inside the toml
 
     A test name is received on the command line. This will either match the name
-    of a single subtest (within an ini) - or - if there's no matching single
+    of a single subtest (within an toml) - or - if there's no matching single
     subtest with that name, then the test name provided might be the name of a
-    test ini itself (i.e. raptor-firefox-tp6) that contains multiple subtests.
+    test toml itself (i.e. raptor-firefox-tp6) that contains multiple subtests.
 
     First look for a single matching subtest name in the list of all availble tests,
     and if it's found we will just run that single subtest.
 
     Then look at the list of all available tests - each available test has a manifest
     name associated to it - and pull out all subtests whose manifest name matches
-    the test name provided on the command line i.e. run all subtests in a specified ini.
+    the test name provided on the command line i.e. run all subtests in a specified toml.
 
     If no tests are found at all then the test name is invalid.
     """
@@ -348,19 +375,30 @@ def get_raptor_test_list(args, oskey):
             tests_to_run.append(next_test)
             break
 
-    # no matches, so now look for all subtests that come from a test ini
+    # Check to make sure that there isn't another test with the same name
+    # and raise an exception if that happens
+    all_tests_available = [
+        next_test for next_test in available_tests if next_test["name"] == args.test
+    ]
+    if len(all_tests_available) > 1:
+        raise Exception(
+            f"Too many tests found with the same test name `{args.test}` for this app. "
+            f"Found in these manifests: "
+            f"{[test['manifest'] for test in all_tests_available]}"
+        )
+
+    # no matches, so now look for all subtests that come from a test toml
     # manifest that matches the test name provided on the commmand line
     if len(tests_to_run) == 0:
-        _ini = args.test + ".ini"
+        _toml = args.test + ".toml"
         for next_test in available_tests:
             head, tail = os.path.split(next_test["manifest"])
-            if tail == _ini:
-                # subtest comes from matching test ini file name, so add it
+            if tail == _toml:
+                # subtest comes from matching test toml file name, so add it
                 tests_to_run.append(next_test)
 
     if args.collect_perfstats and args.app.lower() not in (
         "chrome",
-        "chromium",
         "custom-car",
     ):
         for next_test in tests_to_run:
@@ -375,7 +413,7 @@ def get_raptor_test_list(args, oskey):
             next_test["playback"] = None
 
     # go through each test and set the page-cycles and page-timeout, and some config flags
-    # the page-cycles value in the INI can be overriden when debug-mode enabled, when
+    # the page-cycles value in the TOML can be overriden when debug-mode enabled, when
     # gecko-profiling enabled, or when --page-cycles cmd line arg was used (that overrides all)
     for next_test in tests_to_run:
         LOG.info("configuring settings for test %s" % next_test["name"])
@@ -385,114 +423,139 @@ def get_raptor_test_list(args, oskey):
         # If using playback, the playback recording info may need to be transformed.
         # This transformation needs to happen before the test name is changed
         # below (for cold tests for instance)
-        if next_test.get("playback") is not None:
-            next_test["playback_pageset_manifest"] = transform_subtest(
-                next_test["playback_pageset_manifest"], next_test["name"]
-            )
 
-        if args.gecko_profile is True:
-            next_test["gecko_profile"] = True
-            LOG.info("gecko-profiling enabled")
-            max_page_cycles = 3
-            max_browser_cycles = 3
-
-            if (
-                "gecko_profile_entries" in args
-                and args.gecko_profile_entries is not None
-            ):
-                next_test["gecko_profile_entries"] = str(args.gecko_profile_entries)
-                LOG.info(
-                    "gecko-profiling entries set to %s" % args.gecko_profile_entries
-                )
-
-            if (
-                "gecko_profile_interval" in args
-                and args.gecko_profile_interval is not None
-            ):
-                next_test["gecko_profile_interval"] = str(args.gecko_profile_interval)
-                LOG.info(
-                    "gecko-profiling interval set to %s" % args.gecko_profile_interval
-                )
-
-            if (
-                "gecko_profile_threads" in args
-                and args.gecko_profile_threads is not None
-            ):
-                # pylint --py3k: W1639
-                threads = list(
-                    filter(None, next_test.get("gecko_profile_threads", "").split(","))
-                )
-                threads.extend(args.gecko_profile_threads.split(","))
-                if (
-                    "gecko_profile_extra_threads" in args
-                    and args.gecko_profile_extra_threads is not None
-                ):
-                    threads.extend(getattr(args, "gecko_profile_extra_threads", []))
-                next_test["gecko_profile_threads"] = ",".join(threads)
-                LOG.info("gecko-profiling threads %s" % args.gecko_profile_threads)
-            if (
-                "gecko_profile_features" in args
-                and args.gecko_profile_features is not None
-            ):
-                next_test["gecko_profile_features"] = args.gecko_profile_features
-                LOG.info("gecko-profiling features %s" % args.gecko_profile_features)
+        # Bug 1926419 avoid using mitm11 manifest on linux desktop tests.
+        if oskey == "linux" and args.app in DESKTOP_APPS:
+            if next_test.get("playback") is not None:
+                # Only use the backup manifest if it is set.
+                if next_test.get("playback_pageset_manifest_backup"):
+                    next_test["playback_pageset_manifest_backup"] = transform_subtest(
+                        next_test["playback_pageset_manifest_backup"], next_test["name"]
+                    )
+                else:
+                    next_test["playback_pageset_manifest"] = transform_subtest(
+                        next_test["playback_pageset_manifest"], next_test["name"]
+                    )
 
         else:
-            # if the gecko profiler is not enabled, ignore all of its settings
+            if next_test.get("playback") is not None:
+                next_test["playback_pageset_manifest"] = transform_subtest(
+                    next_test["playback_pageset_manifest"], next_test["name"]
+                )
+
+        # Check if either --gecko-profiler or --extra-profiler-run is enabled.
+        if args.gecko_profile or (
+            args.extra_profiler_run and (args.app in GECKO_PROFILER_APPS + TRACE_APPS)
+        ):
+            if args.gecko_profile:
+                # This is a --gecko-profiler run.
+                next_test["gecko_profile"] = True
+                LOG.info("gecko-profiling enabled")
+                max_page_cycles = 3
+                max_browser_cycles = 3
+            else:
+                # This is an --extra-profiler-run run.
+                next_test["extra_profiler_run"] = True
+                LOG.info("extra-profiler-run enabled")
+                next_test["extra_profiler_run_browser_cycles"] = 1
+                if args.chimera:
+                    next_test["extra_profiler_run_page_cycles"] = 2
+                else:
+                    next_test["extra_profiler_run_page_cycles"] = 1
+
+            # Both --gecko-profiler and --extra-profiler-run shares the same arguments for Firefox.
+            if args.app in GECKO_PROFILER_APPS:
+                if (
+                    "gecko_profile_entries" in args
+                    and args.gecko_profile_entries is not None
+                ):
+                    next_test["gecko_profile_entries"] = str(args.gecko_profile_entries)
+                    LOG.info(
+                        "gecko-profiling entries set to %s" % args.gecko_profile_entries
+                    )
+
+                if (
+                    "gecko_profile_interval" in args
+                    and args.gecko_profile_interval is not None
+                ):
+                    next_test["gecko_profile_interval"] = str(
+                        args.gecko_profile_interval
+                    )
+                    LOG.info(
+                        "gecko-profiling interval set to %s"
+                        % args.gecko_profile_interval
+                    )
+
+                if (
+                    "gecko_profile_threads" in args
+                    and args.gecko_profile_threads is not None
+                ):
+                    # pylint --py3k: W1639
+                    threads = list(
+                        filter(
+                            None, next_test.get("gecko_profile_threads", "").split(",")
+                        )
+                    )
+                    threads.extend(args.gecko_profile_threads.split(","))
+                    if (
+                        "gecko_profile_extra_threads" in args
+                        and args.gecko_profile_extra_threads is not None
+                    ):
+                        threads.extend(getattr(args, "gecko_profile_extra_threads", []))
+                    next_test["gecko_profile_threads"] = ",".join(threads)
+                    LOG.info("gecko-profiling threads %s" % args.gecko_profile_threads)
+                if (
+                    "gecko_profile_features" in args
+                    and args.gecko_profile_features is not None
+                ):
+                    next_test["gecko_profile_features"] = args.gecko_profile_features
+                    LOG.info(
+                        "gecko-profiling features %s" % args.gecko_profile_features
+                    )
+        else:
+            # if the gecko profiler or extra profiler run is not enabled, ignore all of its
+            # settings.
+            args.extra_profiler_run = False
             next_test.pop("gecko_profile_entries", None)
             next_test.pop("gecko_profile_interval", None)
             next_test.pop("gecko_profile_threads", None)
             next_test.pop("gecko_profile_features", None)
 
-        if args.extra_profiler_run is True and args.app == "firefox":
-            next_test["extra_profiler_run"] = True
-            LOG.info("extra-profiler-run enabled")
-            next_test["extra_profiler_run_browser_cycles"] = 1
-            if args.chimera:
-                next_test["extra_profiler_run_page_cycles"] = 2
-            else:
-                next_test["extra_profiler_run_page_cycles"] = 1
-        else:
-            args.extra_profiler_run = False
-            LOG.info("extra-profiler-run disabled")
-
         if args.debug_mode is True:
             next_test["debug_mode"] = True
             LOG.info("debug-mode enabled")
             max_page_cycles = 2
 
-        # if --page-cycles was provided on the command line, use that instead of INI
-        # if just provided in the INI use that but cap at 3 if gecko-profiling is enabled
+        # if --page-cycles was provided on the command line, use that instead of TOML
+        # if just provided in the TOML use that but cap at 3 if gecko-profiling is enabled
         if args.page_cycles is not None:
             next_test["page_cycles"] = args.page_cycles
             LOG.info(
                 "setting page-cycles to %d as specified on cmd line" % args.page_cycles
             )
-        else:
-            if int(next_test.get("page_cycles", 1)) > max_page_cycles:
-                next_test["page_cycles"] = max_page_cycles
-                LOG.info(
-                    "setting page-cycles to %d because gecko-profling is enabled"
-                    % next_test["page_cycles"]
-                )
+        elif int(next_test.get("page_cycles", 1)) > max_page_cycles:
+            next_test["page_cycles"] = max_page_cycles
+            LOG.info(
+                "setting page-cycles to %d because gecko-profling is enabled"
+                % next_test["page_cycles"]
+            )
 
-        # if --browser-cycles was provided on the command line, use that instead of INI
-        # if just provided in the INI use that but cap at 3 if gecko-profiling is enabled
+        # if --browser-cycles was provided on the command line, use that instead of TOML
+        # if just provided in the TOML use that but cap at 3 if gecko-profiling is enabled
         if args.browser_cycles is not None:
             next_test["browser_cycles"] = args.browser_cycles
             LOG.info(
                 "setting browser-cycles to %d as specified on cmd line"
                 % args.browser_cycles
             )
-        else:
-            if int(next_test.get("browser_cycles", 1)) > max_browser_cycles:
-                next_test["browser_cycles"] = max_browser_cycles
-                LOG.info(
-                    "setting browser-cycles to %d because gecko-profilng is enabled"
-                    % next_test["browser_cycles"]
-                )
+        elif int(next_test.get("browser_cycles", 1)) > max_browser_cycles:
+            next_test["browser_cycles"] = max_browser_cycles
+            LOG.info(
+                "setting browser-cycles to %d because gecko-profilng is enabled"
+                % next_test["browser_cycles"]
+            )
 
-        # if --page-timeout was provided on the command line, use that instead of INI
+        # if --page-timeout was provided on the command line, use that instead of TOML
         if args.page_timeout is not None:
             LOG.info(
                 "setting page-timeout to %d as specified on cmd line"
@@ -505,7 +568,7 @@ def get_raptor_test_list(args, oskey):
         # check command line to see if we set cold page load from command line
         if args.cold or next_test.get("cold") == "true":
             # for raptor-webext jobs cold page-load is determined by the 'cold' key
-            # in test manifest INI
+            # in test manifest TOML
             _running_cold = True
         else:
             # if it's a warm load test ignore browser_cycles if set
@@ -534,7 +597,7 @@ def get_raptor_test_list(args, oskey):
         next_test["browser_cycle"] = 1
 
         # if --test-url-params was provided on the command line, add the params to the test_url
-        # provided in the INI
+        # provided in the TOML
         if args.test_url_params is not None:
             initial_test_url = next_test["test_url"]
             next_test["test_url"] = add_test_url_params(
@@ -569,7 +632,7 @@ def get_raptor_test_list(args, oskey):
                 % next_test.get("name", "Unknown")
             )
 
-        # browsertime doesn't use the 'measure' test ini setting; however just for the sake
+        # browsertime doesn't use the 'measure' test toml setting; however just for the sake
         # of supporting both webext and browsertime, just provide a dummy 'measure' setting
         # here to prevent having to check in multiple places; it has no effect on what
         # browsertime actually measures; remove this when eventually we remove webext support
@@ -579,12 +642,13 @@ def get_raptor_test_list(args, oskey):
             and next_test.get("type") == "pageload"
         ):
             next_test["measure"] = (
-                "fnbpaint, fcp, dcf, loadtime,"
-                "ContentfulSpeedIndex, PerceptualSpeedIndex,"
-                "SpeedIndex, FirstVisualChange, LastVisualChange"
+                "fnbpaint, fcp, loadtime, "
+                "ContentfulSpeedIndex, PerceptualSpeedIndex, "
+                "SpeedIndex, FirstVisualChange, LastVisualChange, "
+                "largestContentfulPaint"
             )
 
-        # convert 'measure =' test INI line to list
+        # convert 'measure =' test TOML line to list
         if next_test.get("measure") is not None:
             _measures = []
             for measure in [m.strip() for m in next_test["measure"].split(",")]:
@@ -613,6 +677,7 @@ def get_raptor_test_list(args, oskey):
                 ).resolve()
             )
             next_test["support_class"] = support_class()
+            next_test["support_class"].setup_test(next_test, args)
 
         bool_settings = [
             "lower_is_better",
@@ -620,7 +685,8 @@ def get_raptor_test_list(args, oskey):
             "accept_zero_vismet",
             "interactive",
             "host_from_parent",
-            "expose_gecko_profiler",
+            "expose_browser_profiler",
+            "sparse_checkout",
         ]
         for setting in bool_settings:
             if next_test.get(setting, None) is not None:
@@ -629,7 +695,7 @@ def get_raptor_test_list(args, oskey):
     # write out .json test setting files for the control server to read and send to web ext
     if len(tests_to_run) != 0:
         for test in tests_to_run:
-            if validate_test_ini(test):
+            if validate_test_toml(test):
                 write_test_settings_json(args, test, oskey)
             else:
                 # test doesn't have valid settings, remove it from available list
