@@ -4,18 +4,19 @@
 
 # class to process, format, and report raptor test results
 # received from the raptor control server
-from __future__ import absolute_import
-
+import builtins
 import json
 import os
 import pathlib
 import shutil
+import tarfile
 from abc import ABCMeta, abstractmethod
-from io import open
+from collections.abc import Iterable
+from pathlib import Path
 
-import six
 from logger.logger import RaptorLogger
-from output import RaptorOutput, BrowsertimeOutput
+from output import BrowsertimeOutput, RaptorOutput
+from utils import flatten
 
 LOG = RaptorLogger(component="perftest-results-handler")
 KNOWN_TEST_MODIFIERS = [
@@ -25,30 +26,31 @@ KNOWN_TEST_MODIFIERS = [
     "gecko-profile",
     "cold",
     "webrender",
+    "bytecode-cached",
 ]
+NON_FIREFOX_OPTS = ("webrender", "bytecode-cached", "fission")
+NON_FIREFOX_BROWSERS = ("chrome", "custom-car", "safari", "safari-tp")
+NON_FIREFOX_BROWSERS_MOBILE = ("chrome-m", "cstm-car-m")
 
 
-@six.add_metaclass(ABCMeta)
-class PerftestResultsHandler(object):
+class PerftestResultsHandler(metaclass=ABCMeta):
     """Abstract base class to handle perftest results"""
 
     def __init__(
         self,
         gecko_profile=False,
-        power_test=False,
-        cpu_test=False,
-        memory_test=False,
         live_sites=False,
         app=None,
         conditioned_profile=None,
         cold=False,
         chimera=False,
-        **kwargs
+        fission=True,
+        perfstats=False,
+        test_bytecode_cache=False,
+        extra_summary_methods=[],
+        **kwargs,
     ):
         self.gecko_profile = gecko_profile
-        self.power_test = power_test
-        self.cpu_test = cpu_test
-        self.memory_test = memory_test
         self.live_sites = live_sites
         self.app = app
         self.conditioned_profile = conditioned_profile
@@ -56,16 +58,23 @@ class PerftestResultsHandler(object):
         self.page_timeout_list = []
         self.images = []
         self.supporting_data = None
-        self.fission_enabled = kwargs.get("fission", True)
+        self.fission_enabled = fission
         self.browser_version = None
         self.browser_name = None
         self.cold = cold
         self.chimera = chimera
+        self.perfstats = perfstats
+        self.test_bytecode_cache = test_bytecode_cache
+        self.existing_results = None
+        self.extra_summary_methods = extra_summary_methods
 
     @abstractmethod
     def add(self, new_result_json):
         raise NotImplementedError()
 
+    def result_dir(self):
+        return None
+
     def build_extra_options(self, modifiers=None):
         extra_options = []
 
@@ -88,6 +97,8 @@ class PerftestResultsHandler(object):
                 extra_options.append("gecko-profile")
             if self.cold:
                 extra_options.append("cold")
+            if self.test_bytecode_cache:
+                extra_options.append("bytecode-cached")
             extra_options.append("webrender")
         else:
             for modifier, name in modifiers:
@@ -101,17 +112,18 @@ class PerftestResultsHandler(object):
                         % name
                     )
 
-        if self.app.lower() in (
-            "chrome",
-            "chrome-m",
-            "chromium",
-        ):
-            # Bug 1770225: Make this more dynamic, this will fail us again in the future
-            extra_options.remove("webrender")
-            extra_options.remove("fission")
+        # Bug 1770225: Make this more dynamic, this will fail us again in the future
+        self._clean_up_browser_options(extra_options=extra_options)
 
         return extra_options
 
+    def _clean_up_browser_options(self, extra_options):
+        """Remove certain firefox specific options from different browsers"""
+        if self.app.lower() in NON_FIREFOX_BROWSERS + NON_FIREFOX_BROWSERS_MOBILE:
+            for opts in NON_FIREFOX_OPTS:
+                if opts in extra_options:
+                    extra_options.remove(opts)
+
     def add_browser_meta(self, browser_name, browser_version):
         # sets the browser metadata for the perfherder data
         self.browser_name = browser_name
@@ -166,12 +178,10 @@ class PerftestResultsHandler(object):
             self.supporting_data = []
         self.supporting_data.append(supporting_data)
 
-    def _get_expected_perfherder(self, output):
-        def is_resource_test():
-            if self.power_test or self.cpu_test or self.memory_test:
-                return True
-            return False
+    def use_existing_results(self, directory):
+        self.existing_results = directory
 
+    def _get_expected_perfherder(self, output):
         # if results exists, determine if any test is of type 'scenario'
         is_scenario = False
         if output.summarized_results or output.summarized_supporting_data:
@@ -183,27 +193,10 @@ class PerftestResultsHandler(object):
                 if data_type == "scenario":
                     is_scenario = True
                     break
-
-        if is_scenario and not is_resource_test():
-            # skip perfherder check when a scenario test-type is run without
-            # a resource flag
+        if is_scenario:
+            # skip perfherder check when a scenario test-type is run
             return None
-
-        expected_perfherder = 1
-
-        if is_resource_test():
-            # when resource tests are run, no perfherder data is output
-            # for the regular raptor tests (i.e. speedometer) so we
-            # expect one per resource-type, starting with 0
-            expected_perfherder = 0
-            if self.power_test:
-                expected_perfherder += 1
-            if self.memory_test:
-                expected_perfherder += 1
-            if self.cpu_test:
-                expected_perfherder += 1
-
-        return expected_perfherder
+        return 1
 
     def _validate_treeherder_data(self, output, output_perfdata):
         # late import is required, because install is done in create_virtualenv
@@ -229,7 +222,7 @@ class PerftestResultsHandler(object):
         )
         LOG.info("Validating PERFHERDER_DATA against %s" % schema_path)
         try:
-            with open(schema_path, encoding="utf-8") as f:
+            with builtins.open(schema_path, encoding="utf-8") as f:
                 schema = json.load(f)
             if output.summarized_results:
                 data = output.summarized_results
@@ -274,7 +267,7 @@ class RaptorResultsHandler(PerftestResultsHandler):
         output = RaptorOutput(
             self.results,
             self.supporting_data,
-            test_config["subtest_alert_on"],
+            test_config.get("subtest_alert_on", []),
             self.app,
         )
         output.set_browser_meta(self.browser_name, self.browser_version)
@@ -315,11 +308,187 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
         if not os.path.exists(self._root_results_dir):
             os.mkdir(self._root_results_dir)
 
+        self.browsertime_results_folders = {
+            "browsertime_results": "browsertime-results",
+            "profiler": "browsertime-profiler",
+            "videos_annotated": "browsertime-videos-annotated",
+            "videos_original": "browsertime-videos-original",
+            "results_extra": "browsertime-results-extra",
+        }
+        self.browsertime_temp_folder = "temp-browsertime-results"
+
+    def _determine_target_subfolders(
+        self, result_file, is_profiling_job, has_video_files
+    ):
+        """
+        Determine the target subfolders for a given result file.
+        Args:
+            result_file (Path): The path object for the result file.
+            is_profiling_job (bool): Indicator if the job is a profiling job.
+            has_video_files (bool): Indicator if there are any video recordings present in the results folder,
+            so we can determine if we should group json and mp4 files in the same archive
+
+        Returns:
+            list: A list of target subfolders for the result file.
+        """
+        # Move results files if running a profling job
+        if is_profiling_job:
+            return [self.browsertime_results_folders["profiler"]]
+
+        # Move extra-profiler-run data to separate folder
+        if "profiling" in result_file.parts:
+            return [self.browsertime_results_folders["profiler"]]
+
+        # Check for video files
+        if result_file.suffix == ".mp4":
+            return (
+                [self.browsertime_results_folders["videos_original"]]
+                if "original" in result_file.name
+                else [self.browsertime_results_folders["videos_annotated"]]
+            )
+
+        # JSON files get mapped to multiple folders
+        if result_file.suffix == ".json":
+            target_subfolders = [
+                self.browsertime_results_folders["browsertime_results"]
+            ]
+            if has_video_files:
+                target_subfolders.extend(
+                    [
+                        self.browsertime_results_folders["videos_annotated"],
+                        self.browsertime_results_folders["videos_original"],
+                    ]
+                )
+            return target_subfolders
+
+        # Default folder for unexpected files
+        return [self.browsertime_results_folders["results_extra"]]
+
+    def _cleanup_archive_folders(self, folders_list):
+        """
+        Removes the specified results folders after archiving is complete.
+        """
+        for folder in folders_list:
+            LOG.info(f"Removing {folder}")
+            shutil.rmtree(folder)
+
+    def _archive_results_folder(self, folder_to_archive):
+        """
+        Archives the specified folder into a tar.gz file.
+        """
+        full_archive_path = folder_to_archive.with_suffix(".tgz")
+
+        # Delete previous archives when running locally
+        if full_archive_path.is_file():
+            full_archive_path.unlink(missing_ok=True)
+
+        LOG.info(f"Creating archive: {full_archive_path}")
+        with tarfile.open(str(full_archive_path), "w:gz") as tar:
+            tar.add(folder_to_archive, arcname=folder_to_archive.name)
+
+    def _setup_artifact_folders(self, subfolders, root_path):
+        """
+        Sets up and returns artifact folders based on the provided subfolders and root path.
+        Args:
+            subfolders (List[str]): A list of subfolder names to be set up under the root path.
+            root_path (Path): The root directory under which the artifact subfolders will be created.
+        Returns:
+            Dict[str, Path]: A dictionary mapping each subfolder name to its corresponding Path object.
+        """
+        artifact_folders = {}
+        for subfolder in subfolders:
+            destination_folder = root_path / subfolder
+            destination_folder.mkdir(exist_ok=True, parents=True)
+            artifact_folders[subfolder] = destination_folder
+        return artifact_folders
+
+    def _copy_artifact_to_folders(self, artifact_file, dest_folders_list):
+        """
+        Copies the specified artifact file to multiple destination folders.
+        """
+        for dest_folder in dest_folders_list:
+            dest_folder.mkdir(exist_ok=True, parents=True)
+            shutil.copy(artifact_file, dest_folder)
+
+    def _split_artifact_files(self, artifact_path, artifact_folders, is_profiling_job):
+        """
+        Distributes artifact files from the given artifact path into the appropriate target subfolders.
+        Args:
+            artifact_path (Union[str, Path]): The root path where artifact files are located.
+            artifact_folders (Dict[str, Union[str, Path]]): A dictionary mapping target subfolder names
+            is_profiling_job (bool): A flag indicating whether the current job is a profiling job.
+        """
+        # Need to check if every glob result is a file,
+        # since we can end up with empty folders that contain periods
+        # that get mistaken for files
+        # (such as 'InteractiveRunner.html' for speedometer tests)
+        all_artifact_files = [f for f in artifact_path.glob("**/*.*") if f.is_file()]
+
+        # Check for any mp4 files in the list of unique file extensions
+        has_video_files = ".mp4" in set(f.suffix for f in all_artifact_files)
+
+        for artifact in all_artifact_files:
+            artifact_relpath = artifact.relative_to(artifact_path).parent
+            target_subfolders = self._determine_target_subfolders(
+                artifact, is_profiling_job, has_video_files
+            )
+            destination_folders = [
+                artifact_folders[ts] / artifact_relpath for ts in target_subfolders
+            ]
+            self._copy_artifact_to_folders(artifact, destination_folders)
+
+    def _archive_artifact_folders(self, artifact_folders_list):
+        """
+        Archives all non-empty artifact folders from the given list.
+        Args:
+            artifact_folders_list (List[Union[str, Path]]):
+                A list of artifact folder paths to be checked and archived.
+        """
+        for folder in artifact_folders_list:
+            if self._folder_contains_files(folder):
+                self._archive_results_folder(folder)
+
+    def _folder_contains_files(self, artifact_folder):
+        return os.listdir(artifact_folder)
+
+    def archive_raptor_artifacts(self, is_profiling_job):
+        """
+        Description: Archives browsertime artifacts based on specific criteria.
+        Args:
+        - is_profiling_job (bool): Indicator if a job is running with the gecko_profile flag set to True
+        """
+        test_artifact_path = Path(self.result_dir())
+        destination_root_path = test_artifact_path.parent
+        temp_artifact_path = destination_root_path / self.browsertime_temp_folder
+
+        # Cleanup any temp folders remaining from previous runs, when running locally
+        if temp_artifact_path.is_dir():
+            self._cleanup_archive_folders([temp_artifact_path])
+
+        # Rename browsertime-results to a temp folder so we don't have name conflicts
+        test_artifact_path.rename(temp_artifact_path)
+
+        artifact_folders = self._setup_artifact_folders(
+            self.browsertime_results_folders.values(), destination_root_path
+        )
+
+        self._split_artifact_files(
+            temp_artifact_path, artifact_folders, is_profiling_job
+        )
+        artifact_folders_list = list(artifact_folders.values())
+        self._archive_artifact_folders(artifact_folders_list)
+        artifact_folders_list.append(temp_artifact_path)
+        self._cleanup_archive_folders(artifact_folders_list)
+
     def result_dir(self):
         return self._root_results_dir
 
     def result_dir_for_test(self, test):
-        return os.path.join(self._root_results_dir, test["name"])
+        if self.existing_results is None:
+            results_root = self._root_results_dir
+        else:
+            results_root = self.existing_results
+        return os.path.join(results_root, test["name"])
 
     def remove_result_dir_for_test(self, test):
         test_result_dir = self.result_dir_for_test(test)
@@ -327,6 +496,12 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
             shutil.rmtree(test_result_dir)
         return test_result_dir
 
+    def result_dir_for_test_profiling(self, test):
+        profiling_dir = os.path.join(self.result_dir_for_test(test), "profiling")
+        if not os.path.exists(profiling_dir):
+            os.mkdir(profiling_dir)
+        return profiling_dir
+
     def add(self, new_result_json):
         # not using control server with bt
         pass
@@ -353,6 +528,13 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
         page_count,
         test_name,
         accept_zero_vismet,
+        load_existing,
+        test_summary,
+        subtest_name_filters,
+        handle_custom_data,
+        support_class,
+        submetric_summary_method,
+        **kwargs,
     ):
         """
         Receive a json blob that contains the results direct from the browsertime tool. Parse
@@ -436,7 +618,17 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
                   "max": 864
                 },
               }
-            }
+            },
+            "geckoPerfStats": [
+              {
+                "Compositing": 71,
+                "MajorGC": 144
+              },
+              {
+                "Compositing": 13,
+                "MajorGC": 126
+              }
+            ]
           }
         ]
 
@@ -464,13 +656,13 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
            }
         """
         LOG.info("parsing results from browsertime json")
-
         # bt to raptor names
         conversion = (
             ("fnbpaint", "firstPaint"),
             ("fcp", ["paintTiming", "first-contentful-paint"]),
             ("dcf", "timeToDomContentFlushed"),
             ("loadtime", "loadEventEnd"),
+            ("largestContentfulPaint", ["largestContentfulPaint", "renderTime"]),
         )
 
         def _get_raptor_val(mdict, mname, retval=False):
@@ -485,7 +677,7 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
             # mdict: a dictionary to look through to find the mname
             #        value.
 
-            if type(mname) != list:
+            if type(mname) is not list:
                 if mname in mdict:
                     return mdict[mname]
                 return retval
@@ -505,26 +697,30 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
         # one results entry with all replicates within. When running warm page-load, there will
         # be one results entry for every warm page-load iteration; with one single replicate
         # inside each.
-        if cold:
+        if cold or handle_custom_data:
             if len(raw_btresults) == 0:
                 raise MissingResultsError(
                     "Missing results for all cold browser cycles."
                 )
-        else:
-            if len(raw_btresults) != int(page_cycles):
-                raise MissingResultsError(
-                    "Missing results for at least 1 warm page-cycle."
-                )
+        elif load_existing:
+            pass  # Use whatever is there.
+        elif len(raw_btresults) != int(page_cycles):
+            raise MissingResultsError("Missing results for at least 1 warm page-cycle.")
 
         # now parse out the values
         page_counter = 0
-        for raw_result in raw_btresults:
+        last_result = False
+        for i, raw_result in enumerate(raw_btresults):
             if not raw_result["browserScripts"]:
                 raise MissingResultsError("Browsertime cycle produced no measurements.")
 
             if raw_result["browserScripts"][0].get("timings") is None:
                 raise MissingResultsError("Browsertime cycle is missing all timings")
 
+            if i == (len(raw_btresults) - 1):
+                # Used to tell custom support scripts when the last result
+                # is being parsed. This lets them finalize any overall measurements.
+                last_result = True
             # Desktop chrome doesn't have `browser` scripts data available for now
             bt_browser = raw_result["browserScripts"][0].get("browser", None)
             bt_ver = raw_result["info"]["browsertime"]["version"]
@@ -545,96 +741,91 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
                 "measurements": {},
                 "statistics": {},
             }
+            if submetric_summary_method is not None:
+                bt_result["submetric_summary_method"] = submetric_summary_method
 
-            if self.power_test:
-                power_result = {
-                    "bt_ver": bt_ver,
-                    "browser": bt_browser,
-                    "url": bt_url,
-                    "measurements": {},
-                    "statistics": {},
-                    "power_data": True,
-                }
-                for cycle in raw_result["android"]["power"]:
-                    for metric in cycle:
-                        if "total" in metric:
-                            continue
-                        power_result["measurements"].setdefault(metric, []).append(
-                            cycle[metric]
-                        )
-                power_result["statistics"] = raw_result["statistics"]["android"][
-                    "power"
-                ]
-                results.append(power_result)
+            def _extract_cpu_vals():
+                # Bug 1806402 - Handle chrome cpu data properly
+                cpu_vals = raw_result.get("cpu", None)
+                if (
+                    cpu_vals
+                    and self.app
+                    not in NON_FIREFOX_BROWSERS + NON_FIREFOX_BROWSERS_MOBILE
+                ):
+                    bt_result["measurements"].setdefault("cpuTime", []).extend(cpu_vals)
 
-            custom_types = raw_result["extras"][0]
-            if custom_types:
-                for custom_type in custom_types:
-                    for k, v in custom_types[custom_type].items():
-                        bt_result["measurements"].setdefault(k, []).append(v)
-            else:
-                # extracting values from browserScripts and statistics
-                for bt, raptor in conversion:
-                    if measure is not None and bt not in measure:
-                        continue
-                    # chrome we just measure fcp and loadtime; skip fnbpaint and dcf
-                    if (
-                        self.app
-                        and (
-                            "chrome" in self.app.lower()
-                            or "chromium" in self.app.lower()
-                        )
-                        and bt in ("fnbpaint", "dcf")
-                    ):
-                        continue
-
-                    # FCP uses a different path to get the timing, so we need to do
-                    # some checks here
-                    if bt == "fcp" and not _get_raptor_val(
-                        raw_result["browserScripts"][0]["timings"],
-                        raptor,
-                    ):
-                        continue
-
-                    # XXX looping several times in the list, could do better
-                    for cycle in raw_result["browserScripts"]:
-                        if bt not in bt_result["measurements"]:
-                            bt_result["measurements"][bt] = []
-                        val = _get_raptor_val(cycle["timings"], raptor)
-                        if not val:
-                            raise MissingResultsError(
-                                "Browsertime cycle missing {} measurement".format(
-                                    raptor
-                                )
-                            )
-                        bt_result["measurements"][bt].append(val)
-
-                    # let's add the browsertime statistics; we'll use those for overall values
-                    # instead of calculating our own based on the replicates
-                    bt_result["statistics"][bt] = _get_raptor_val(
-                        raw_result["statistics"]["timings"], raptor, retval={}
+            def _extract_android_power_vals():
+                power_vals = raw_result.get("android").get("power", {})
+                if power_vals:
+                    bt_result["measurements"].setdefault("powerUsage", []).extend(
+                        [
+                            round(vals["powerUsage"] * (1 * 10**-6), 2)
+                            for vals in power_vals
+                        ]
                     )
 
-                if self.browsertime_visualmetrics:
-                    for cycle in raw_result["visualMetrics"]:
-                        for metric in cycle:
-                            if "progress" in metric.lower():
-                                # Bug 1665750 - Determine if we should display progress
-                                continue
+            if support_class:
+                bt_result["custom_data"] = True
+                support_class.save_data(raw_result, bt_result)
+                support_class.handle_result(
+                    bt_result,
+                    raw_result,
+                    conversion=conversion,
+                    last_result=last_result,
+                )
+            elif any(raw_result["extras"]):
+                # Each entry here is a separate cold pageload iteration
+                for custom_types in raw_result["extras"]:
+                    for custom_type in custom_types:
+                        data = custom_types[custom_type]
+                        if handle_custom_data:
+                            if test_summary in ("flatten",):
+                                data = flatten(data, ())
+                            for k, v in data.items():
 
-                            if metric not in measure:
-                                continue
+                                def _ignore_metric(*args):
+                                    if any(
+                                        type(arg) not in (int, float) for arg in args
+                                    ):
+                                        return True
+                                    return False
 
-                            val = cycle[metric]
-                            if not accept_zero_vismet:
-                                if val == 0:
-                                    self.failed_vismets.append(metric)
+                                # Ignore any non-numerical results
+                                if _ignore_metric(v) and _ignore_metric(*v):
                                     continue
 
-                            bt_result["measurements"].setdefault(metric, []).append(val)
-                            bt_result["statistics"][metric] = raw_result["statistics"][
-                                "visualMetrics"
-                            ][metric]
+                                # Clean up the name if requested
+                                filtered_k = k
+                                for name_filter in subtest_name_filters.split(","):
+                                    filtered_k = filtered_k.replace(name_filter, "")
+
+                                if isinstance(v, Iterable):
+                                    bt_result["measurements"].setdefault(
+                                        filtered_k, []
+                                    ).extend(v)
+                                else:
+                                    bt_result["measurements"].setdefault(
+                                        filtered_k, []
+                                    ).append(v)
+                            bt_result["custom_data"] = True
+                        else:
+                            for k, v in data.items():
+                                bt_result["measurements"].setdefault(k, []).append(v)
+                if self.perfstats:
+                    for cycle in raw_result["geckoPerfStats"]:
+                        for metric in cycle:
+                            bt_result["measurements"].setdefault(
+                                "perfstat-" + metric, []
+                            ).append(cycle[metric])
+                if kwargs.get("gather_cpuTime", None):
+                    _extract_cpu_vals()
+            else:
+                raise Exception(
+                    "Test should be using browsertime_pageload.py as the "
+                    "support_class instead of the results.py/output.py parsing."
+                )
+
+            _extract_android_power_vals()
 
             results.append(bt_result)
 
@@ -666,21 +857,22 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
         }
 
     def _label_video_folder(self, result_data, base_dir, kind="warm"):
-        for idx, data in enumerate(result_data["files"]["video"]):
-            parts = list(pathlib.Path(data).parts)
-            lable_idx = parts.index("data")
-            if "query-" in parts[lable_idx - 1]:
-                lable_idx -= 1
+        for filetype in result_data["files"]:
+            for idx, data in enumerate(result_data["files"][filetype]):
+                parts = list(pathlib.Path(data).parts)
+                lable_idx = parts.index("data")
+                if "query-" in parts[lable_idx - 1]:
+                    lable_idx -= 1
 
-            src_dir = pathlib.Path(base_dir).joinpath(*parts[: lable_idx + 1])
-            parts.insert(lable_idx, kind)
-            dst_dir = pathlib.Path(base_dir).joinpath(*parts[: lable_idx + 1])
+                src_dir = pathlib.Path(base_dir).joinpath(*parts[: lable_idx + 1])
+                parts.insert(lable_idx, kind)
+                dst_dir = pathlib.Path(base_dir).joinpath(*parts[: lable_idx + 1])
 
-            if src_dir.exists():
-                pathlib.Path(dst_dir).mkdir(parents=True, exist_ok=True)
-                shutil.move(str(src_dir), str(dst_dir))
+                if src_dir.exists() and not dst_dir.exists():
+                    pathlib.Path(dst_dir).mkdir(parents=True, exist_ok=True)
+                    shutil.move(str(src_dir), str(dst_dir))
 
-            result_data["files"]["video"][idx] = str(pathlib.Path(*parts[:]))
+                result_data["files"][filetype][idx] = str(pathlib.Path(*parts[:]))
 
     def summarize_and_output(self, test_config, tests, test_names):
         """
@@ -727,7 +919,7 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
                 return False
 
             try:
-                with open(bt_res_json, "r", encoding="utf8") as f:
+                with builtins.open(bt_res_json, encoding="utf8") as f:
                     raw_btresults = json.load(f)
             except Exception as e:
                 LOG.error("Exception reading %s" % bt_res_json)
@@ -736,25 +928,67 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
                 raise
 
             # Split the chimera videos here for local testing
-            cold_path = None
-            warm_path = None
-            if self.chimera:
+            def split_browsertime_results(result_json_path, raw_btresults):
                 # First result is cold, second is warm
                 cold_data = raw_btresults[0]
                 warm_data = raw_btresults[1]
 
-                dirpath = os.path.dirname(os.path.abspath(bt_res_json))
-                cold_path = os.path.join(dirpath, "cold-browsertime.json")
-                warm_path = os.path.join(dirpath, "warm-browsertime.json")
+                dirpath = os.path.dirname(os.path.abspath(result_json_path))
+                _cold_path = os.path.join(dirpath, "cold-browsertime.json")
+                _warm_path = os.path.join(dirpath, "warm-browsertime.json")
 
                 self._label_video_folder(cold_data, dirpath, "cold")
                 self._label_video_folder(warm_data, dirpath, "warm")
 
-                with open(cold_path, "w") as f:
+                with builtins.open(_cold_path, "w") as f:
                     json.dump([cold_data], f)
-                with open(warm_path, "w") as f:
+                with builtins.open(_warm_path, "w") as f:
                     json.dump([warm_data], f)
 
+                raw_btresults[0] = cold_data
+                raw_btresults[1] = warm_data
+
+                return _cold_path, _warm_path
+
+            cold_path = None
+            warm_path = None
+            if self.chimera:
+                cold_path, warm_path = split_browsertime_results(
+                    bt_res_json, raw_btresults
+                )
+
+                # Overwrite the contents of the browsertime.json file
+                # to update it with the new file paths
+                try:
+                    with builtins.open(bt_res_json, "w", encoding="utf8") as f:
+                        json.dump(raw_btresults, f)
+                except Exception as e:
+                    LOG.error("Exception reading %s" % bt_res_json)
+                    # XXX this should be replaced by a traceback call
+                    LOG.error("Exception: %s %s" % (type(e).__name__, str(e)))
+                    raise
+
+                # If extra profiler run is enabled, split its browsertime.json
+                # file to cold and warm json files as well.
+                bt_profiling_res_json = os.path.join(
+                    self.result_dir_for_test_profiling(test), "browsertime.json"
+                )
+                has_extra_profiler_run = test_config.get(
+                    "extra_profiler_run", False
+                ) and os.path.exists(bt_profiling_res_json)
+                if has_extra_profiler_run:
+                    try:
+                        with builtins.open(bt_profiling_res_json, encoding="utf8") as f:
+                            raw_profiling_btresults = json.load(f)
+                            split_browsertime_results(
+                                bt_profiling_res_json, raw_profiling_btresults
+                            )
+                    except Exception as e:
+                        LOG.info(
+                            "Exception reading and writing %s" % bt_profiling_res_json
+                        )
+                        LOG.info("Exception: %s %s" % (type(e).__name__, str(e)))
+
             if not run_local:
                 extra_options = self.build_extra_options()
                 tags = self.build_tags(test=test)
@@ -806,6 +1040,13 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
                 test_config.get("page_count", []),
                 test["name"],
                 accept_zero_vismet,
+                self.existing_results is not None,
+                test.get("test_summary", "pageload"),
+                test.get("subtest_name_filters", ""),
+                test.get("custom_data", False) == "true",
+                test.get("support_class", None),
+                test.get("submetric_summary_method", None),
+                gather_cpuTime=test.get("gather_cpuTime", None),
             ):
 
                 def _new_standard_result(new_result, subtest_unit="ms"):
@@ -821,6 +1062,13 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
                             continue
                         new_result[field] = test[field]
 
+                    new_result["min_back_window"] = test.get("min_back_window", None)
+                    new_result["max_back_window"] = test.get("max_back_window", None)
+                    new_result["fore_window"] = test.get("fore_window", None)
+                    new_result["alert_change_type"] = test.get(
+                        "alert_change_type", None
+                    )
+
                     # All Browsertime measurements are elapsed times in milliseconds.
                     new_result["subtest_lower_is_better"] = test.get(
                         "subtest_lower_is_better", True
@@ -835,6 +1083,9 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
                         new_result["extra_options"].remove("cold")
                         new_result["extra_options"].append("warm")
 
+                    # Add the support class to the result
+                    new_result["support_class"] = test.get("support_class", None)
+
                     return new_result
 
                 def _new_powertest_result(new_result):
@@ -848,6 +1099,15 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
                     LOG.info("parsed new power result: %s" % str(new_result))
                     return new_result
 
+                def _new_custom_result(new_result):
+                    new_result["type"] = "pageload"
+                    new_result = _new_standard_result(
+                        new_result, subtest_unit=test.get("subtest_unit", "ms")
+                    )
+
+                    LOG.info("parsed new custom result: %s" % str(new_result))
+                    return new_result
+
                 def _new_pageload_result(new_result):
                     new_result["type"] = "pageload"
                     new_result = _new_standard_result(new_result)
@@ -861,7 +1121,7 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
                     new_result = _new_standard_result(
                         new_result, subtest_unit=test.get("subtest_unit", "ms")
                     )
-
+                    new_result["gather_cpuTime"] = test.get("gather_cpuTime", None)
                     LOG.info("parsed new benchmark result: %s" % str(new_result))
                     return new_result
 
@@ -873,16 +1133,17 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
                 if new_result.get("power_data", False):
                     self.results.append(_new_powertest_result(new_result))
                 elif test["type"] == "pageload":
-                    self.results.append(_new_pageload_result(new_result))
+                    if test.get("custom_data", False) == "true":
+                        self.results.append(_new_custom_result(new_result))
+                    else:
+                        self.results.append(_new_pageload_result(new_result))
                 elif test["type"] == "benchmark":
                     for i, item in enumerate(self.results):
                         if item["name"] == test["name"] and not _is_supporting_data(
                             item
                         ):
                             # add page cycle custom measurements to the existing results
-                            for measurement in six.iteritems(
-                                new_result["measurements"]
-                            ):
+                            for measurement in new_result["measurements"].items():
                                 self.results[i]["measurements"][measurement[0]].extend(
                                     measurement[1]
                                 )
@@ -894,8 +1155,9 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
         output = BrowsertimeOutput(
             self.results,
             self.supporting_data,
-            test_config["subtest_alert_on"],
+            test_config.get("subtest_alert_on", []),
             self.app,
+            self.extra_summary_methods,
         )
         output.set_browser_meta(self.browser_name, self.browser_version)
         output.summarize(test_names)
@@ -925,14 +1187,21 @@ class BrowsertimeResultsHandler(PerftestResultsHandler):
 
             jobs_file = os.path.join(self.result_dir(), "jobs.json")
             LOG.info(
-                "Writing video jobs and application data {} into {}".format(
-                    jobs_json, jobs_file
-                )
+                f"Writing video jobs and application data {jobs_json} into {jobs_file}"
             )
-            with open(jobs_file, "w") as f:
+            with builtins.open(jobs_file, "w") as f:
                 f.write(json.dumps(jobs_json))
 
-        return (success and validate_success) and len(self.failed_vismets) == 0
+        support_class_success = True
+        for test in tests:
+            if test.get("support_class"):
+                support_class_success = test.get("support_class").report_test_success()
+
+        return (
+            (success and validate_success)
+            and len(self.failed_vismets) == 0
+            and support_class_success
+        )
 
 
 class MissingResultsError(Exception):
