@@ -17,13 +17,22 @@ import six.moves.urllib_parse as urlparse
 from mozbuild.base import MachCommandConditions as conditions
 from mozbuild.base import MozbuildObject
 from mozfile import which
+from mozinfo.platforminfo import PlatformInfo
 from moztest.resolve import TestManifestLoader, TestResolver
+from redo import retriable
 
 REFERER = "https://wiki.developer.mozilla.org/en-US/docs/Mozilla/Test-Info"
 MAX_DAYS = 30
 
 
-class TestInfo(object):
+class SetEncoder(json.JSONEncoder):
+    def default(self, obj):
+        if isinstance(obj, set):
+            return list(obj)
+        return super().default(obj)
+
+
+class TestInfo:
     """
     Support 'mach test-info'.
     """
@@ -228,13 +237,28 @@ class TestInfoReport(TestInfo):
         TestInfo.__init__(self, verbose)
         self.threads = []
 
+    @retriable(attempts=3, sleeptime=5, sleepscale=2)
+    def get_url(self, target_url):
+        # if we fail to get valid json (i.e. end point has malformed data), return {}
+        retVal = {}
+        try:
+            self.log_verbose("getting url: %s" % target_url)
+            r = requests.get(target_url, headers={"User-agent": "mach-test-info/1.0"})
+            self.log_verbose("got status: %s" % r.status_code)
+            r.raise_for_status()
+            retVal = r.json()
+        except json.decoder.JSONDecodeError:
+            self.log_verbose("Error retrieving data from %s" % target_url)
+
+        return retVal
+
     def update_report(self, by_component, result, path_mod):
         def update_item(item, label, value):
             # It is important to include any existing item value in case ActiveData
             # returns multiple records for the same test; that can happen if the report
             # sometimes maps more than one ActiveData record to the same path.
             new_value = item.get(label, 0) + value
-            if type(new_value) == int:
+            if type(new_value) is int:
                 item[label] = new_value
             else:
                 item[label] = float(round(new_value, 2))  # pylint: disable=W1633
@@ -293,8 +317,8 @@ class TestInfoReport(TestInfo):
         return os.path.join("js", "src", "jit-test", "tests", path)
 
     def path_mod_xpcshell(self, path):
-        # <manifest>.ini:<path> -> "<path>"
-        path = path.split(".ini:")[-1]
+        # <manifest>.{ini|toml}:<path> -> "<path>"
+        path = path.split(":")[-1]
         return path
 
     def description(
@@ -358,22 +382,34 @@ class TestInfoReport(TestInfo):
             return name_part.split()[-1]  # get just the test name, not extra words
         return None
 
-    def get_runcount_data(self, start, end):
+    def get_runcount_data(self, runcounts_input_file, start, end):
         # TODO: use start/end properly
-        runcounts = self.get_runcounts(days=MAX_DAYS)
+        if runcounts_input_file:
+            try:
+                with open(runcounts_input_file) as f:
+                    runcounts = json.load(f)
+            except:
+                print("Unable to load runcounts from path: %s" % runcounts_input_file)
+                raise
+        else:
+            runcounts = self.get_runcounts(days=MAX_DAYS)
         runcounts = self.squash_runcounts(runcounts, days=MAX_DAYS)
         return runcounts
 
     def get_testinfoall_index_url(self):
         import taskcluster
 
-        queue = taskcluster.Queue()
         index = taskcluster.Index(
             {
                 "rootUrl": "https://firefox-ci-tc.services.mozilla.com",
             }
         )
         route = "gecko.v2.mozilla-central.latest.source.test-info-all"
+        queue = taskcluster.Queue(
+            {
+                "rootUrl": "https://firefox-ci-tc.services.mozilla.com",
+            }
+        )
 
         task_id = index.findTask(route)["taskId"]
         artifacts = queue.listLatestArtifacts(task_id)["artifacts"]
@@ -388,57 +424,76 @@ class TestInfoReport(TestInfo):
     def get_runcounts(self, days=MAX_DAYS):
         testrundata = {}
         # get historical data from test-info job artifact; if missing get fresh
-        try:
-            url = self.get_testinfoall_index_url()
-            print("INFO: requesting runcounts url: %s" % url)
-            r = requests.get(url, headers={"User-agent": "mach-test-info/1.0"})
-            r.raise_for_status()
-            testrundata = r.json()
-        except Exception:
-            pass
+        url = self.get_testinfoall_index_url()
+        print("INFO: requesting runcounts url: %s" % url)
+        olddata = self.get_url(url)
 
         # fill in any holes we have
         endday = datetime.datetime.now(datetime.timezone.utc) - datetime.timedelta(
             days=1
         )
         startday = endday - datetime.timedelta(days=days)
+        urls_to_fetch = []
+        # build list of dates with missing data
         while startday < endday:
             nextday = startday + datetime.timedelta(days=1)
-            retries = 2
-            done = False
-            if (
-                str(nextday) not in testrundata.keys()
-                or testrundata[str(nextday)] == {}
-            ):
-                while not done:
-                    url = "https://treeherder.mozilla.org/api/groupsummary/"
-                    url += "?startdate=%s&enddate=%s" % (
-                        startday.date(),
-                        nextday.date(),
-                    )
-                    try:
-                        print("INFO: requesting groupsummary url: %s" % url)
-                        r = requests.get(
-                            url, headers={"User-agent": "mach-test-info/1.0"}
-                        )
-                        done = True
-                    except requests.exceptions.HTTPError:
-                        retries -= 1
-                        if retries <= 0:
-                            r.raise_for_status()
-                    try:
-                        testrundata[str(nextday.date())] = r.json()
-                    except json.decoder.JSONDecodeError:
-                        print(
-                            "Warning unable to retrieve (from treeherder's groupsummary api) testrun data for date: %s, skipping for now"
-                            % nextday.date()
-                        )
-                        testrundata[str(nextday.date())] = {}
-                        continue
+            if not olddata.get(str(nextday.date()), {}):
+                url = "https://treeherder.mozilla.org/api/groupsummary/"
+                url += "?startdate=%s&enddate=%s" % (
+                    startday.date(),
+                    nextday.date(),
+                )
+                urls_to_fetch.append([str(nextday.date()), url])
+            testrundata[str(nextday.date())] = olddata.get(str(nextday.date()), {})
+
             startday = nextday
 
+        # limit missing data collection to 5 most recent days days to reduce overall runtime
+        for date, url in urls_to_fetch[-5:]:
+            try:
+                testrundata[date] = self.get_url(url)
+            except requests.exceptions.HTTPError:
+                # We want to see other errors, but can accept HTTPError failures
+                print(f"Unable to retrieve results for url: {url}")
+                pass
+
         return testrundata
 
+    def optimize_runcounts_data(self, runcounts, num_days):
+        yesterday = datetime.date.today() - datetime.timedelta(days=1)
+        if num_days > 1:
+            startday = yesterday - datetime.timedelta(days=num_days)
+        else:
+            startday = yesterday
+
+        days = [
+            (startday + datetime.timedelta(days=i)).strftime("%Y-%m-%d")
+            for i in range(num_days)
+        ]
+
+        summary_groups = {key: runcounts[key] for key in days if key in runcounts}
+        tasks_and_count = {"manifests": []}
+        for day in days:
+            if day not in summary_groups or not summary_groups[day]:
+                continue
+            all_task_labels = summary_groups[day]["job_type_names"]
+            for tasks_by_manifest in summary_groups[day]["manifests"]:
+                for manifest in tasks_by_manifest:
+                    tasks_and_count.setdefault(manifest, {})
+                    for task_index, _, _, count in tasks_by_manifest[manifest]:
+                        task_label = all_task_labels[task_index]
+                        if task_label not in tasks_and_count["manifests"]:
+                            tasks_and_count["manifests"].append(task_label)
+                            new_index = len(tasks_and_count["manifests"]) - 1
+                        else:
+                            new_index = tasks_and_count["manifests"].index(task_label)
+
+                        if new_index not in tasks_and_count[manifest]:
+                            tasks_and_count[manifest][new_index] = 0
+                        tasks_and_count[manifest][new_index] += count
+
+        return tasks_and_count
+
     def squash_runcounts(self, runcounts, days=MAX_DAYS):
         # squash all testrundata together into 1 big happy family for the last X days
         endday = datetime.datetime.now(datetime.timezone.utc) - datetime.timedelta(
@@ -459,15 +514,19 @@ class TestInfoReport(TestInfo):
                 print("Warning: Missing job type names from date: %s" % datekey)
                 continue
 
+            # TODO: changed this to include all manifests, not just first
             for m in runcounts[datekey]["manifests"]:
-                man_name = list(m.keys())[0]
+                for man_name in m.keys():
+                    for job_type_id, result, classification, count in m[man_name]:
+                        # HACK: we treat shippable and opt the same for mozinfo and runcounts
+                        job_name = jtn[job_type_id].replace("-shippable", "")
 
-                for job_type_id, result, classification, count in m[man_name]:
-                    # format: job_type_name, result, classification, count
-                    # find matching jtn, result, classification and increment 'count'
-                    job_name = jtn[job_type_id]
-                    key = (job_name, result, classification)
-                    testgroup_runinfo[man_name][key] += count
+                        # format: job_type_name, result, classification, count
+                        # find matching jtn, result, classification and increment 'count'
+                        key = (job_name, result, classification)
+
+                        # only keep the "parent" manifest
+                        testgroup_runinfo[man_name.split(":")[0]][key] += count
 
         for m in testgroup_runinfo:
             retVal[m] = [
@@ -484,8 +543,7 @@ class TestInfoReport(TestInfo):
             "https://treeherder.mozilla.org/api/failures/?startday=%s&endday=%s&tree=trunk"
             % (start, end)
         )
-        r = requests.get(url, headers={"User-agent": "mach-test-info/1.0"})
-        if_data = r.json()
+        if_data = self.get_url(url)
         buglist = [x["bug_id"] for x in if_data]
 
         # get bug data for summary, 800 bugs at a time
@@ -494,13 +552,16 @@ class TestInfoReport(TestInfo):
         bug_data = []
         fields = ["id", "product", "component", "summary"]
         for bug_index in range(0, len(buglist), max_bugs):
-            bugs = [str(x) for x in buglist[bug_index:max_bugs]]
+            bugs = [str(x) for x in buglist[bug_index : bug_index + max_bugs]]
+            if not bugs:
+                print(f"warning: found no bugs in range {bug_index}, +{max_bugs}")
+                continue
+
             url = "https://bugzilla.mozilla.org/rest/bug?include_fields=%s&id=%s" % (
                 ",".join(fields),
                 ",".join(bugs),
             )
-            r = requests.get(url, headers={"User-agent": "mach-test-info/1.0"})
-            data = r.json()
+            data = self.get_url(url)
             if data and "bugs" in data.keys():
                 bug_data.extend(data["bugs"])
 
@@ -550,6 +611,8 @@ class TestInfoReport(TestInfo):
         start,
         end,
         show_testruns,
+        runcounts_input_file,
+        config_matrix_output_file,
     ):
         def matches_filters(test):
             """
@@ -579,6 +642,15 @@ class TestInfoReport(TestInfo):
             show_manifests = True
             show_summary = True
 
+        trunk = False
+        if os.environ.get("GECKO_HEAD_REPOSITORY", "") in [
+            "https://hg.mozilla.org/mozilla-central",
+            "https://hg.mozilla.org/try",
+        ]:
+            trunk = True
+        else:
+            show_testruns = False
+
         by_component = {}
         if components:
             components = components.split(",")
@@ -593,11 +665,8 @@ class TestInfoReport(TestInfo):
         ifd = self.get_intermittent_failure_data(start, end)
 
         runcount = {}
-        if show_testruns and os.environ.get("GECKO_HEAD_REPOSITORY", "") in [
-            "https://hg.mozilla.org/mozilla-central",
-            "https://hg.mozilla.org/try",
-        ]:
-            runcount = self.get_runcount_data(start, end)
+        if show_testruns and trunk:
+            runcount = self.get_runcount_data(runcounts_input_file, start, end)
 
         print("Finding tests...")
         here = os.path.abspath(os.path.dirname(__file__))
@@ -616,9 +685,27 @@ class TestInfoReport(TestInfo):
                     manifest_path = "%s:%s" % (t["ancestor_manifest"], t["manifest"])
                 manifest_paths.add(manifest_path)
         manifest_count = len(manifest_paths)
-        print(
-            "Resolver found {} tests, {} manifests".format(len(tests), manifest_count)
-        )
+        print(f"Resolver found {len(tests)} tests, {manifest_count} manifests")
+
+        if config_matrix_output_file and trunk:
+            topsrcdir = self.build_obj.topsrcdir
+            config_matrix = {}
+            for manifest in manifest_paths:
+                # we want the first part of the parent:child, as parent shows up in MHTP
+                # TODO: figure out a better solution for child manifests
+                if ".toml" in manifest:
+                    relpath = mozpath.relpath(
+                        f"{manifest.split('.toml')[0]}.toml", topsrcdir
+                    )
+                else:
+                    relpath = mozpath.relpath(manifest, topsrcdir)
+                # hack for wpt manifests
+                if relpath.startswith(".."):
+                    relpath = "/" + relpath.replace("../", "")
+                config_matrix[relpath] = self.create_matrix_from_task_graph(
+                    relpath, runcount
+                )
+            self.write_report(config_matrix, config_matrix_output_file)
 
         if show_manifests:
             topsrcdir = self.build_obj.topsrcdir
@@ -641,9 +728,7 @@ class TestInfoReport(TestInfo):
                 if relpath in files_info:
                     bug_component = files_info[relpath].get("BUG_COMPONENT")
                     if bug_component:
-                        key = "{}::{}".format(
-                            bug_component.product, bug_component.component
-                        )
+                        key = f"{bug_component.product}::{bug_component.component}"
                     else:
                         key = "<unknown bug component>"
                     if (not components) or (key in components):
@@ -723,11 +808,11 @@ class TestInfoReport(TestInfo):
                                 temp.extend(condition.split("\n"))
                             annotation_conditions = temp
 
-                            for condition in annotation_conditions:
+                            for c in annotation_conditions:
                                 condition_count += 1
                                 # Trim reftest fuzzy-if ranges: everything after the first comma
                                 # eg. "Android,0-2,1-3" -> "Android"
-                                condition = condition.split(",")[0]
+                                condition = c.split(",")[0]
                                 if condition not in conditions:
                                     conditions[condition] = 0
                                 conditions[condition] += 1
@@ -736,9 +821,7 @@ class TestInfoReport(TestInfo):
                 if relpath in files_info:
                     bug_component = files_info[relpath].get("BUG_COMPONENT")
                     if bug_component:
-                        key = "{}::{}".format(
-                            bug_component.product, bug_component.component
-                        )
+                        key = f"{bug_component.product}::{bug_component.component}"
                     else:
                         key = "<unknown bug component>"
                     if (not components) or (key in components):
@@ -776,7 +859,7 @@ class TestInfoReport(TestInfo):
                                 total_runs = 0
                                 for m in test_info["manifest"]:
                                     if m in runcount.keys():
-                                        for x in runcount.get("m", []):
+                                        for x in runcount.get(m, []):
                                             if not x:
                                                 break
                                             total_runs += x[3]
@@ -856,7 +939,7 @@ class TestInfoReport(TestInfo):
         )
 
     def write_report(self, by_component, output_file):
-        json_report = json.dumps(by_component, indent=2, sort_keys=True)
+        json_report = json.dumps(by_component, indent=2, sort_keys=True, cls=SetEncoder)
         if output_file:
             output_file = os.path.abspath(output_file)
             output_dir = os.path.dirname(output_file)
@@ -939,3 +1022,166 @@ class TestInfoReport(TestInfo):
             "%s: %d deleted, %d added, %d common"
             % (component, len(deleted), len(added), common)
         )
+
+    ################################################################################
+    ###
+    ###  Below is code for creating a os/version/processor/config/variant matrix
+    ###
+
+    def build_matrix_cache(self):
+        # this is an attempt to cache the .json for the duration of the task
+        filename = "task-graph.json"
+        if os.path.exists(filename):
+            with open(filename) as f:
+                data = json.load(f)
+        else:
+            url = (
+                "https://firefox-ci-tc.services.mozilla.com/api/index/v1/task/gecko.v2.mozilla-central.latest.taskgraph.decision/artifacts/public/"
+                + filename
+            )
+
+            response = requests.get(url, headers={"User-agent": "mach-test-info/1.0"})
+            data = response.json()
+            with open(filename, "w") as f:
+                json.dump(data, f)
+
+        for task in data.values():
+            task_label: str = task["label"]
+            # HACK: we treat shippable and opt the same from mozinfo, runtime counts
+            task_label = task_label.replace("-shippable", "")
+
+            # we only want test tasks
+            if not task_label.startswith("test-"):
+                continue
+            if task_label.endswith("-cf"):
+                continue
+
+            # skip tier-3
+            if (
+                task.get("task", {})
+                .get("extra", {})
+                .get("treeherder", {})
+                .get("tier", 3)
+                == 3
+            ):
+                continue
+
+            try:
+                parts = task_label.split("-")
+                if int(parts[-1]):
+                    task_label = "-".join(parts[:-1])
+            except ValueError:
+                pass
+
+            # TODO: this only works for tasks where we schedule by manifest
+            env = task.get("task", {}).get("payload", {}).get("env", {})
+
+            mhtp = json.loads(env.get("MOZHARNESS_TEST_PATHS", "{}"))
+            if not mhtp:
+                # mock up logic here if matching task
+                suite = self.find_non_test_path_loader(task_label)
+                if not suite:
+                    continue
+                mhtp[suite] = [suite]
+
+            # TODO: figure out a better method for dealing with TEST_TAG
+            # when we have a test_tag, all skipped manifests are added to chunk 1.
+            # we are skipping real manifests, but avoiding many overreported manifests.
+            #
+            # NOTE: some variants only have a single chunk, so no numbers
+            if json.loads(env.get("MOZHARNESS_TEST_TAG", "{}")):
+                if not json.loads(env.get("MOZHARNESS_TEST_PATHS", "{}")):
+                    # mock up logic here if matching task
+                    suite = self.find_non_test_path_loader(task_label)
+                    if not suite:
+                        continue
+                    mhtp[suite] = [suite]
+
+            for suite in mhtp:
+                for manifest in mhtp[suite]:
+                    self.matrix_map[manifest].append(task_label)
+
+            extra = task.get("task", {}).get("extra", {}).get("test-setting", {})
+            platform_info = PlatformInfo(extra)
+
+            self.task_tuples[task_label] = platform_info
+
+    matrix_map = defaultdict(list)
+    task_tuples: dict[str, PlatformInfo] = {}
+
+    def find_non_test_path_loader(self, label):
+        # TODO: how to keep this list synchronized?
+        known_suites = [
+            "mochitest-browser-media",
+            "telemetry-tests-client",
+            "mochitest-webgl2-ext",
+            "mochitest-webgl1-ext",
+            "jittest-1proc",
+            "mochitest-browser-translations",
+            "jsreftest",
+            "mochitest-browser-screenshots",
+            "marionette-unittest",
+        ]
+        match = [x for x in known_suites if x in label]
+        if match:
+            return match[0]
+        return ""
+
+    # find manifest in matrix_map and for all tasks that run this
+    # pull the tuples out and create a definitive list
+    def create_matrix_from_task_graph(self, target_manifest, runcount):
+        results = {}
+
+        if not self.matrix_map:
+            self.build_matrix_cache()
+
+        # for tasks with no MOZHARNESS_TEST_PATHS, provide basic data
+        if target_manifest in runcount and self.find_non_test_path_loader(
+            runcount[target_manifest][0][0]
+        ):
+            suite = self.find_non_test_path_loader(runcount[target_manifest][0][0])
+            self.matrix_map[target_manifest] = self.matrix_map[suite]
+
+        for tl in self.matrix_map.get(target_manifest, []):
+            task_label = tl.replace("-shippable", "")
+            platform_info = self.task_tuples[task_label]
+
+            # add in runcounts, we can find find the index of the given task_label in 'job_type_names',
+            # use that to get specific runs
+            passed = 0
+            failed = 0
+            if target_manifest in runcount:
+                # data = [[job_name, result, classification, count], ...]
+                for data in [
+                    x for x in runcount[target_manifest] if task_label == x[0]
+                ]:
+                    if data[1] == "passed":
+                        passed += data[-1]
+                    else:
+                        failed += data[-1]
+
+            # this helps avoid 'skipped' manifests
+            if passed == 0 and failed == 0:
+                continue
+
+            if platform_info.os not in results:
+                results[platform_info.os] = {}
+            os = results[platform_info.os]
+            if platform_info.os_version not in os:
+                os[platform_info.os_version] = {}
+            os_version = os[platform_info.os_version]
+            if platform_info.arch not in os_version:
+                os_version[platform_info.arch] = {}
+            arch = os_version[platform_info.arch]
+            if platform_info.build_type not in arch:
+                arch[platform_info.build_type] = {}
+
+            if platform_info.test_variant not in arch[platform_info.build_type]:
+                arch[platform_info.build_type][platform_info.test_variant] = {
+                    "pass": 0,
+                    "fail": 0,
+                }
+            arch[platform_info.build_type][platform_info.test_variant]["pass"] += passed
+            arch[platform_info.build_type][platform_info.test_variant]["fail"] += failed
+
+        return results
