@@ -1,17 +1,19 @@
+# mypy: allow-untyped-defs
+
 import errno
+import logging
 import os
 import shutil
 import socket
 import subprocess
 import sys
-import tempfile
 import time
-import urllib2
+from urllib.request import urlopen
+from urllib.error import URLError
 
 import pytest
 
-from tools.wpt import wpt
-
+from tools.wpt import browser, utils, wpt
 
 here = os.path.abspath(os.path.dirname(__file__))
 
@@ -20,7 +22,7 @@ def is_port_8000_in_use():
     s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
     try:
         s.bind(("127.0.0.1", 8000))
-    except socket.error as e:
+    except OSError as e:
         if e.errno == errno.EADDRINUSE:
             return True
         else:
@@ -45,14 +47,23 @@ def init_manifest():
 
 
 @pytest.fixture
-def manifest_dir():
+def manifest_dir(tmp_path):
+    shutil.copyfile(get_persistent_manifest_path(),
+                    os.path.join(tmp_path, "MANIFEST.json"))
+    return str(tmp_path)
+
+
+@pytest.fixture(scope="module")
+def firefox_binary_path(tmp_path_factory):
     try:
-        path = tempfile.mkdtemp()
-        shutil.copyfile(get_persistent_manifest_path(),
-                        os.path.join(path, "MANIFEST.json"))
-        yield path
+        logger = logging.getLogger("firefox_binary_path")
+        path = str(tmp_path_factory.mktemp("firefox_binary"))
+        firefox = browser.Firefox(logger)
+        bin_path = firefox.install(dest=path)
+        assert os.path.exists(bin_path) and os.path.isfile(bin_path)
+        yield bin_path
     finally:
-        shutil.rmtree(path)
+        utils.rmtree(path)
 
 
 @pytest.fixture
@@ -77,7 +88,7 @@ def temp_test():
 
     yield make_test
 
-    shutil.rmtree("../../.tools-tests")
+    utils.rmtree("../../.tools-tests")
 
 
 def test_missing():
@@ -93,23 +104,32 @@ def test_help():
     assert excinfo.value.code == 0
 
 
+def test_load_commands():
+    commands = wpt.load_commands()
+    # The `wpt run` command has conditional requirements.
+    assert "conditional_requirements" in commands["run"]
+
+
 @pytest.mark.slow
-@pytest.mark.xfail(sys.platform == "win32",
-                   reason="https://github.com/web-platform-tests/wpt/issues/12935")
-def test_list_tests(manifest_dir):
+@pytest.mark.skipif(sys.platform == "win32",
+                    reason="https://github.com/web-platform-tests/wpt/issues/28745")
+def test_list_tests(manifest_dir, firefox_binary_path):
     """The `--list-tests` option should not produce an error under normal
     conditions."""
 
     with pytest.raises(SystemExit) as excinfo:
         wpt.main(argv=["run", "--metadata", manifest_dir, "--list-tests",
-                       "--yes", "chrome", "/dom/nodes/Element-tagName.html"])
+                       "--yes",
+                       # WebTransport server is not needed (web-platform-tests/wpt#41675).
+                       "--no-enable-webtransport-h3",
+                       "--binary", firefox_binary_path,
+                       "firefox", "/dom/nodes/Element-tagName.html"])
     assert excinfo.value.code == 0
 
 
 @pytest.mark.slow
-@pytest.mark.xfail(sys.platform == "win32",
-                   reason="https://github.com/web-platform-tests/wpt/issues/12935")
-def test_list_tests_missing_manifest(manifest_dir):
+@pytest.mark.remote_network
+def test_list_tests_missing_manifest(manifest_dir, firefox_binary_path):
     """The `--list-tests` option should not produce an error in the absence of
     a test manifest file."""
 
@@ -124,17 +144,20 @@ def test_list_tests_missing_manifest(manifest_dir):
                        # drastically reduces the time to execute the test.
                        "--tests", here,
                        "--metadata", manifest_dir,
+                       "--log-mach", "-",
                        "--list-tests",
                        "--yes",
+                       # WebTransport server is not needed (web-platform-tests/wpt#41675).
+                       "--no-enable-webtransport-h3",
+                       "--binary", firefox_binary_path,
                        "firefox", "/dom/nodes/Element-tagName.html"])
 
     assert excinfo.value.code == 0
 
 
 @pytest.mark.slow
-@pytest.mark.xfail(sys.platform == "win32",
-                   reason="https://github.com/web-platform-tests/wpt/issues/12935")
-def test_list_tests_invalid_manifest(manifest_dir):
+@pytest.mark.remote_network
+def test_list_tests_invalid_manifest(manifest_dir, firefox_binary_path):
     """The `--list-tests` option should not produce an error in the presence of
     a malformed test manifest file."""
 
@@ -154,8 +177,12 @@ def test_list_tests_invalid_manifest(manifest_dir):
                        # drastically reduces the time to execute the test.
                        "--tests", here,
                        "--metadata", manifest_dir,
+                       "--log-mach", "-",
                        "--list-tests",
                        "--yes",
+                       # WebTransport server is not needed (web-platform-tests/wpt#41675).
+                       "--no-enable-webtransport-h3",
+                       "--binary", firefox_binary_path,
                        "firefox", "/dom/nodes/Element-tagName.html"])
 
     assert excinfo.value.code == 0
@@ -163,30 +190,36 @@ def test_list_tests_invalid_manifest(manifest_dir):
 
 @pytest.mark.slow
 @pytest.mark.remote_network
-@pytest.mark.xfail(sys.platform == "win32",
-                   reason="Tests currently don't work on Windows for path reasons")
-def test_run_zero_tests():
+@pytest.mark.skipif(sys.platform == "win32",
+                    reason="https://github.com/web-platform-tests/wpt/issues/28745")
+def test_run_zero_tests(firefox_binary_path):
     """A test execution describing zero tests should be reported as an error
     even in the presence of the `--no-fail-on-unexpected` option."""
     if is_port_8000_in_use():
         pytest.skip("port 8000 already in use")
 
     with pytest.raises(SystemExit) as excinfo:
-        wpt.main(argv=["run", "--yes", "--no-pause", "--binary-arg", "headless",
-                       "chrome", "/non-existent-dir/non-existent-file.html"])
+        wpt.main(argv=["run", "--yes", "--no-pause",
+                       # WebTransport server is not needed (web-platform-tests/wpt#41675).
+                       "--no-enable-webtransport-h3",
+                       "--binary", firefox_binary_path,
+                       "firefox", "/non-existent-dir/non-existent-file.html"])
     assert excinfo.value.code != 0
 
     with pytest.raises(SystemExit) as excinfo:
-        wpt.main(argv=["run", "--yes", "--no-pause", "--binary-arg", "headless",
-                       "--no-fail-on-unexpected",
-                       "chrome", "/non-existent-dir/non-existent-file.html"])
+        wpt.main(argv=["run", "--yes", "--no-pause", "--no-fail-on-unexpected",
+                       # WebTransport server is not needed (web-platform-tests/wpt#41675).
+                       "--no-enable-webtransport-h3",
+                       "--binary", firefox_binary_path,
+                       "firefox", "/non-existent-dir/non-existent-file.html"])
     assert excinfo.value.code != 0
 
+
 @pytest.mark.slow
 @pytest.mark.remote_network
-@pytest.mark.xfail(sys.platform == "win32",
-                   reason="Tests currently don't work on Windows for path reasons")
-def test_run_failing_test():
+@pytest.mark.skipif(sys.platform == "win32",
+                    reason="https://github.com/web-platform-tests/wpt/issues/28745")
+def test_run_failing_test(firefox_binary_path):
     """Failing tests should be reported with a non-zero exit status unless the
     `--no-fail-on-unexpected` option has been specified."""
     if is_port_8000_in_use():
@@ -196,22 +229,28 @@ def test_run_failing_test():
     assert os.path.isfile("../../%s" % failing_test)
 
     with pytest.raises(SystemExit) as excinfo:
-        wpt.main(argv=["run", "--yes", "--no-pause", "--binary-arg", "headless",
-                       "chrome", failing_test])
+        wpt.main(argv=["run", "--yes", "--no-pause",
+                       # WebTransport server is not needed (web-platform-tests/wpt#41675).
+                       "--no-enable-webtransport-h3",
+                       "--binary", firefox_binary_path,
+                       "firefox", failing_test])
     assert excinfo.value.code != 0
 
     with pytest.raises(SystemExit) as excinfo:
-        wpt.main(argv=["run", "--yes", "--no-pause", "--binary-arg", "headless",
-                       "--no-fail-on-unexpected",
-                       "chrome", failing_test])
+        wpt.main(argv=["run", "--yes", "--no-pause", "--no-fail-on-unexpected",
+                       "--channel", "dev",
+                       # WebTransport server is not needed (web-platform-tests/wpt#41675).
+                       "--no-enable-webtransport-h3",
+                       "--binary", firefox_binary_path,
+                       "firefox", failing_test])
     assert excinfo.value.code == 0
 
 
 @pytest.mark.slow
 @pytest.mark.remote_network
-@pytest.mark.xfail(sys.platform == "win32",
-                   reason="Tests currently don't work on Windows for path reasons")
-def test_run_verify_unstable(temp_test):
+@pytest.mark.skipif(sys.platform == "win32",
+                    reason="https://github.com/web-platform-tests/wpt/issues/28745")
+def test_run_verify_unstable(temp_test, firefox_binary_path):
     """Unstable tests should be reported with a non-zero exit status. Stable
     tests should be reported with a zero exit status."""
     if is_port_8000_in_use():
@@ -227,86 +266,57 @@ def test_run_verify_unstable(temp_test):
     """)
 
     with pytest.raises(SystemExit) as excinfo:
-        wpt.main(argv=["run", "--yes", "--verify", "--binary-arg", "headless",
-                       "chrome", unstable_test])
+        wpt.main(argv=["run", "--yes", "--verify",
+                       # WebTransport server is not needed (web-platform-tests/wpt#41675).
+                       "--no-enable-webtransport-h3",
+                       "--binary", firefox_binary_path,
+                       "firefox", unstable_test])
     assert excinfo.value.code != 0
 
     stable_test = temp_test("test(function() {}, 'my test');")
 
     with pytest.raises(SystemExit) as excinfo:
-        wpt.main(argv=["run", "--yes", "--verify", "--binary-arg", "headless",
-                       "chrome", stable_test])
+        wpt.main(argv=["run", "--yes", "--verify",
+                       # WebTransport server is not needed (web-platform-tests/wpt#41675).
+                       "--no-enable-webtransport-h3",
+                       "--binary", firefox_binary_path,
+                       "firefox", stable_test])
     assert excinfo.value.code == 0
 
 
-@pytest.mark.slow
-@pytest.mark.remote_network
-@pytest.mark.xfail(sys.platform == "win32",
-                   reason="Tests currently don't work on Windows for path reasons")
-def test_install_chromedriver():
-    chromedriver_path = os.path.join(wpt.localpaths.repo_root, "_venv", "bin", "chromedriver")
-    if os.path.exists(chromedriver_path):
-        os.unlink(chromedriver_path)
-    with pytest.raises(SystemExit) as excinfo:
-        wpt.main(argv=["install", "chrome", "webdriver"])
-    assert excinfo.value.code == 0
-    assert os.path.exists(chromedriver_path)
-    os.unlink(chromedriver_path)
-
-
-@pytest.mark.slow
-@pytest.mark.remote_network
-@pytest.mark.xfail(sys.platform == "win32",
-                   reason="Tests currently don't work on Windows for path reasons")
-def test_install_firefox():
-    if sys.platform == "darwin":
-        fx_path = os.path.join(wpt.localpaths.repo_root, "_venv", "browsers", "nightly", "Firefox Nightly.app")
-    else:
-        fx_path = os.path.join(wpt.localpaths.repo_root, "_venv", "browsers", "nightly", "firefox")
-    if os.path.exists(fx_path):
-        shutil.rmtree(fx_path)
-    with pytest.raises(SystemExit) as excinfo:
-        wpt.main(argv=["install", "firefox", "browser", "--channel=nightly"])
-    assert excinfo.value.code == 0
-    assert os.path.exists(fx_path)
-    shutil.rmtree(fx_path)
-
-
-@pytest.mark.xfail(sys.platform == "win32",
-                   reason="Tests currently don't work on Windows for path reasons")
 def test_files_changed(capsys):
     commit = "9047ac1d9f51b1e9faa4f9fad9c47d109609ab09"
     with pytest.raises(SystemExit) as excinfo:
-        wpt.main(argv=["files-changed", "%s~..%s" % (commit, commit)])
+        wpt.main(argv=["files-changed", f"{commit}~..{commit}"])
     assert excinfo.value.code == 0
     out, err = capsys.readouterr()
-    assert out == """html/browsers/offline/appcache/workers/appcache-worker.html
+    expected = """html/browsers/offline/appcache/workers/appcache-worker.html
 html/browsers/offline/appcache/workers/resources/appcache-dedicated-worker-not-in-cache.js
 html/browsers/offline/appcache/workers/resources/appcache-shared-worker-not-in-cache.js
 html/browsers/offline/appcache/workers/resources/appcache-worker-data.py
 html/browsers/offline/appcache/workers/resources/appcache-worker-import.py
 html/browsers/offline/appcache/workers/resources/appcache-worker.manifest
 html/browsers/offline/appcache/workers/resources/appcache-worker.py
-"""
+""".replace("/", os.path.sep)
+    assert out == expected
     assert err == ""
 
 
-@pytest.mark.xfail(sys.platform == "win32",
-                   reason="Tests currently don't work on Windows for path reasons")
 def test_files_changed_null(capsys):
     commit = "9047ac1d9f51b1e9faa4f9fad9c47d109609ab09"
     with pytest.raises(SystemExit) as excinfo:
-        wpt.main(argv=["files-changed", "--null", "%s~..%s" % (commit, commit)])
+        wpt.main(argv=["files-changed", "--null", f"{commit}~..{commit}"])
     assert excinfo.value.code == 0
     out, err = capsys.readouterr()
-    assert out == "\0".join(["html/browsers/offline/appcache/workers/appcache-worker.html",
+    expected = "\0".join(["html/browsers/offline/appcache/workers/appcache-worker.html",
         "html/browsers/offline/appcache/workers/resources/appcache-dedicated-worker-not-in-cache.js",
         "html/browsers/offline/appcache/workers/resources/appcache-shared-worker-not-in-cache.js",
         "html/browsers/offline/appcache/workers/resources/appcache-worker-data.py",
         "html/browsers/offline/appcache/workers/resources/appcache-worker-import.py",
         "html/browsers/offline/appcache/workers/resources/appcache-worker.manifest",
         "html/browsers/offline/appcache/workers/resources/appcache-worker.py",
-        ""])
+        ""]).replace("/", os.path.sep)
+    assert out == expected
     assert err == ""
 
 
@@ -322,8 +332,8 @@ def test_files_changed_ignore():
 
 def test_files_changed_ignore_rules():
     from tools.wpt.testfiles import compile_ignore_rule
-    assert compile_ignore_rule("foo*bar*/baz").pattern == "^foo\*bar[^/]*/baz$"
-    assert compile_ignore_rule("foo**bar**/baz").pattern == "^foo\*\*bar.*/baz$"
+    assert compile_ignore_rule("foo*bar*/baz").pattern == r"^foo\*bar[^/]*/baz$"
+    assert compile_ignore_rule("foo**bar**/baz").pattern == r"^foo\*\*bar.*/baz$"
     assert compile_ignore_rule("foobar/baz/*").pattern == "^foobar/baz/[^/]*$"
     assert compile_ignore_rule("foobar/baz/**").pattern == "^foobar/baz/.*$"
 
@@ -340,7 +350,7 @@ def test_tests_affected(capsys, manifest_dir):
     # The test will fail if the file we assert is renamed, so we choose a stable one.
     commit = "3a055e818218f548db240c316654f3cc1aeeb733"
     with pytest.raises(SystemExit) as excinfo:
-        wpt.main(argv=["tests-affected", "--metadata", manifest_dir, "%s~..%s" % (commit, commit)])
+        wpt.main(argv=["tests-affected", "--metadata", manifest_dir, f"{commit}~..{commit}"])
     assert excinfo.value.code == 0
     out, err = capsys.readouterr()
     assert "infrastructure/reftest-wait.html" in out
@@ -354,10 +364,15 @@ def test_tests_affected(capsys, manifest_dir):
 def test_tests_affected_idlharness(capsys, manifest_dir):
     commit = "47cea8c38b88c0ddd3854e4edec0c5b6f2697e62"
     with pytest.raises(SystemExit) as excinfo:
-        wpt.main(argv=["tests-affected", "--metadata", manifest_dir, "%s~..%s" % (commit, commit)])
+        wpt.main(argv=["tests-affected", "--metadata", manifest_dir, f"{commit}~..{commit}"])
     assert excinfo.value.code == 0
     out, err = capsys.readouterr()
-    assert "webrtc-stats/idlharness.window.js\nwebrtc/idlharness.https.window.js\n" == out
+    assert ("mst-content-hint/idlharness.window.js\n" +
+            "webrtc-encoded-transform/idlharness.https.window.js\n" +
+            "webrtc-identity/idlharness.https.window.js\n" +
+            "webrtc-stats/idlharness.window.js\n" +
+            "webrtc-stats/supported-stats.https.html\n" +
+            "webrtc/idlharness.https.window.js\n") == out
 
 
 @pytest.mark.slow  # this updates the manifest
@@ -370,20 +385,20 @@ def test_tests_affected_null(capsys, manifest_dir):
     # the current working directory for references to the changed files, not the ones at
     # that specific commit. But we can at least test it returns something sensible.
     # The test will fail if the file we assert is renamed, so we choose a stable one.
-    commit = "9bf1daa3d8b4425f2354c3ca92c4cf0398d329dd"
+    commit = "2614e3316f1d3d1a744ed3af088d19516552a5de"
     with pytest.raises(SystemExit) as excinfo:
-        wpt.main(argv=["tests-affected", "--null", "--metadata", manifest_dir, "%s~..%s" % (commit, commit)])
+        wpt.main(argv=["tests-affected", "--null", "--metadata", manifest_dir, f"{commit}~..{commit}"])
     assert excinfo.value.code == 0
     out, err = capsys.readouterr()
 
     tests = out.split("\0")
-    assert "dom/interfaces.html" in tests
-    assert "html/dom/interfaces.https.html" in tests
+    assert "dom/idlharness.any.js" in tests
+    assert "xhr/idlharness.any.js" in tests
 
 
 @pytest.mark.slow
-@pytest.mark.xfail(sys.platform == "win32",
-                   reason="Tests currently don't work on Windows for path reasons")
+@pytest.mark.skipif(sys.platform == "win32",
+                    reason="no os.setsid/killpg to easily cleanup the process tree")
 def test_serve():
     if is_port_8000_in_use():
         pytest.skip("port 8000 already in use")
@@ -399,9 +414,9 @@ def test_serve():
             if time.time() - start > 60:
                 assert False, "server did not start responding within 60s"
             try:
-                resp = urllib2.urlopen("http://web-platform.test:8000")
+                resp = urlopen("http://web-platform.test:8000")
                 print(resp)
-            except urllib2.URLError:
+            except URLError:
                 print("URLError")
                 time.sleep(1)
             else:
@@ -409,6 +424,7 @@ def test_serve():
                 break
     finally:
         os.killpg(p.pid, 15)
+        p.wait(10)
 
 # The following commands are slow running and used implicitly in other CI
 # jobs, so we skip them here:
