@@ -3,24 +3,25 @@
 # Any copyright is dedicated to the Public Domain.
 # http://creativecommons.org/publicdomain/zero/1.0/
 #
+# These tests are run before any xpcshell test is ran when the --self-test flag
+# is passed. To run these tests, run:
+# ./mach xpcshell-test --self-test
+# or use "mach test" with any xpcshell test file or directory:
+# ./mach test testing/modules/tests/xpcshell/test_assert.js --self-test
 
-from __future__ import absolute_import, print_function
-
-import mozinfo
 import os
 import pprint
 import re
-import shutil
-import six
 import sys
 import tempfile
 import unittest
 
+import mozfile
+import mozinfo
+import six
 from mozlog import structured
-
 from runxpcshelltests import XPCShellTests
 
-
 TEST_PASS_STRING = "TEST-PASS"
 TEST_FAIL_STRING = "TEST-UNEXPECTED-FAIL"
 
@@ -28,7 +29,6 @@ SIMPLE_PASSING_TEST = "function run_test() { Assert.ok(true); }"
 SIMPLE_FAILING_TEST = "function run_test() { Assert.ok(false); }"
 SIMPLE_PREFCHECK_TEST = """
 function run_test() {
-  const { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
   Assert.ok(Services.prefs.getBoolPref("fake.pref.to.test"));
 }
 """
@@ -117,7 +117,7 @@ add_test(function test_loop () {
 });
 """
 
-PASSING_TEST_UNICODE = b"""
+PASSING_TEST_UNICODE = rb"""
 function run_test () { run_next_test(); }
 
 add_test(function test_unicode_print () {
@@ -155,6 +155,14 @@ add_task(async function test_failing() {
 });
 """
 
+ADD_TASK_REJECTED_UNDEFINED = """
+function run_test() { run_next_test(); }
+
+add_task(async function test_failing() {
+  await Promise.reject();
+});
+"""
+
 ADD_TASK_FAILURE_INSIDE = """
 function run_test() { run_next_test(); }
 
@@ -285,7 +293,9 @@ no_such_var = "foo"; // assignment to undeclared variable
 # A test that crashes outright.
 TEST_CRASHING = """
 function run_test () {
-  const { ctypes } = ChromeUtils.import("resource://gre/modules/ctypes.jsm");
+  const { ctypes } = ChromeUtils.importESModule(
+    "resource://gre/modules/ctypes.sys.mjs"
+  );
   let zero = new ctypes.intptr_t(8);
   let badptr = ctypes.cast(zero, ctypes.PointerType(ctypes.int32_t));
   badptr.contents;
@@ -295,10 +305,6 @@ function run_test () {
 # A test for asynchronous cleanup functions
 ASYNC_CLEANUP = """
 function run_test() {
-  let { PromiseUtils } = ChromeUtils.import(
-    "resource://gre/modules/PromiseUtils.jsm"
-  );
-
   // The list of checkpoints in the order we encounter them.
   let checkpoints = [];
 
@@ -324,7 +330,7 @@ function run_test() {
   });
 
   registerCleanupFunction(function async_cleanup_2() {
-    let deferred = PromiseUtils.defer();
+    let deferred = Promise.withResolvers();
     executeSoon(deferred.resolve);
     return deferred.promise.then(function() {
       checkpoints.push(3);
@@ -336,7 +342,7 @@ function run_test() {
   });
 
   registerCleanupFunction(function async_cleanup() {
-    let deferred = PromiseUtils.defer();
+    let deferred = Promise.withResolvers();
     executeSoon(deferred.resolve);
     return deferred.promise.then(function() {
       checkpoints.push(1);
@@ -420,21 +426,15 @@ add_test(function test_child_mozinfo () {
 
 HEADLESS_TRUE = """
 add_task(function headless_true() {
-  let env = Cc["@mozilla.org/process/environment;1"].getService(
-    Ci.nsIEnvironment
-  );
-  Assert.equal(env.get("MOZ_HEADLESS"), "1", "Check MOZ_HEADLESS");
-  Assert.equal(env.get("DISPLAY"), "77", "Check DISPLAY");
+  Assert.equal(Services.env.get("MOZ_HEADLESS"), "1", "Check MOZ_HEADLESS");
+  Assert.equal(Services.env.get("DISPLAY"), "77", "Check DISPLAY");
 });
 """
 
 HEADLESS_FALSE = """
 add_task(function headless_false() {
-  let env = Cc["@mozilla.org/process/environment;1"].getService(
-    Ci.nsIEnvironment
-  );
-  Assert.notEqual(env.get("MOZ_HEADLESS"), "1", "Check MOZ_HEADLESS");
-  Assert.notEqual(env.get("DISPLAY"), "77", "Check DISPLAY");
+  Assert.notEqual(Services.env.get("MOZ_HEADLESS"), "1", "Check MOZ_HEADLESS");
+  Assert.notEqual(Services.env.get("DISPLAY"), "77", "Check DISPLAY");
 });
 """
 
@@ -466,6 +466,7 @@ class XPCShellTestsTests(unittest.TestCase):
             )
         else:
             self.xpcshellBin = os.path.join(objdir, "dist", "bin", "xpcshell")
+
         if sys.platform == "win32":
             self.xpcshellBin += ".exe"
         self.utility_path = os.path.join(objdir, "dist", "bin")
@@ -484,7 +485,7 @@ class XPCShellTestsTests(unittest.TestCase):
         self.x.harness_timeout = 30 if not mozinfo.info["ccov"] else 60
 
     def tearDown(self):
-        shutil.rmtree(self.tempdir)
+        mozfile.remove(self.tempdir)
         self.x.shutdownNode()
 
     def writeFile(self, name, contents, mode="w"):
@@ -508,26 +509,24 @@ class XPCShellTestsTests(unittest.TestCase):
         """
         testlines = []
         for t in tests:
-            testlines.append("[%s]" % (t if isinstance(t, six.string_types) else t[0]))
+            testlines.append('["%s"]' % (t if isinstance(t, str) else t[0]))
             if isinstance(t, tuple):
                 testlines.extend(t[1:])
         prefslines = []
         for p in prefs:
             # Append prefs lines as indented inside "prefs=" manifest option.
-            prefslines.append("  %s" % p)
+            prefslines.append('  "%s",' % p)
 
-        self.manifest = self.writeFile(
-            "xpcshell.ini",
-            """
+        val = """
 [DEFAULT]
-head =
-tail =
-prefs =
+head = ""
+tail = ""
+prefs = [
 """
-            + "\n".join(prefslines)
-            + "\n"
-            + "\n".join(testlines),
-        )
+        val += "\n".join(prefslines)
+        val += "]\n"
+        val += "\n".join(testlines)
+        self.manifest = self.writeFile("xpcshell.toml", val)
 
     def assertTestResult(self, expected, shuffle=False, verbose=False, headless=False):
         """
@@ -535,6 +534,7 @@ prefs =
         returns |expected|.
         """
         kwargs = {}
+        kwargs["app_binary"] = self.app_binary
         kwargs["xpcshell"] = self.xpcshellBin
         kwargs["symbolsPath"] = self.symbols_path
         kwargs["manifest"] = self.manifest
@@ -545,16 +545,23 @@ prefs =
         kwargs["sequential"] = True
         kwargs["testingModulesDir"] = self.testing_modules
         kwargs["utility_path"] = self.utility_path
-        self.assertEqual(
-            expected,
-            self.x.runTests(kwargs),
-            msg="""Tests should have %s, log:
+        kwargs["repeat"] = 0
+
+        startup_profiling = os.environ.pop("MOZ_PROFILER_STARTUP", None)
+        try:
+            self.assertEqual(
+                expected,
+                self.x.runTests(kwargs),
+                msg="""Tests should have %s, log:
 ========
 %s
 ========
 """
-            % ("passed" if expected else "failed", self.log.getvalue()),
-        )
+                % ("passed" if expected else "failed", self.log.getvalue()),
+            )
+        finally:
+            if startup_profiling:
+                os.environ["MOZ_PROFILER_STARTUP"] = startup_profiling
 
     def _assertLog(self, s, expected):
         l = self.log.getvalue()
@@ -661,8 +668,8 @@ prefs =
 
         self.assertInLog("###!!! ASSERTION")
         log_lines = self.log.getvalue().splitlines()
-        line_pat = "#\d\d:"
-        unknown_pat = "#\d\d\: \?\?\?\[.* \+0x[a-f0-9]+\]"
+        line_pat = r"#\d\d:"
+        unknown_pat = r"#\d\d\: \?\?\?\[.* \+0x[a-f0-9]+\]"
         self.assertFalse(
             any(re.search(unknown_pat, line) for line in log_lines),
             "An stack frame without symbols was found in\n%s"
@@ -1090,6 +1097,21 @@ add_test({
         self.assertEqual(0, self.x.passCount)
         self.assertEqual(1, self.x.failCount)
 
+    def testAddTaskTestRejectedUndefined(self):
+        """
+        Ensure rejected task with undefined reason reports as failure and does not hang.
+        """
+        self.writeFile(
+            "test_add_task_rejected_undefined.js", ADD_TASK_REJECTED_UNDEFINED
+        )
+        self.writeManifest(["test_add_task_rejected_undefined.js"])
+
+        self.assertTestResult(False)
+        self.assertEqual(1, self.x.testCount)
+        self.assertEqual(0, self.x.passCount)
+        self.assertEqual(1, self.x.failCount)
+        self.assertNotInLog("TEST-UNEXPECTED-TIMEOUT")
+
     def testAddTaskTestFailureInside(self):
         """
         Ensure tests inside task are reported as failures.
@@ -1126,7 +1148,6 @@ add_test({
         self.assertInLog("this_test_will_fail")
         self.assertInLog("run_next_test")
         self.assertInLog("run_test")
-        self.assertNotInLog("Task.jsm")
 
     def testAddTaskSkip(self):
         self.writeFile("test_tasks_skip.js", ADD_TASK_SKIP)
@@ -1148,10 +1169,10 @@ add_test({
 
     def testMissingHeadFile(self):
         """
-        Ensure that missing head file results in fatal error.
+        Ensure that missing head file results in fatal failure.
         """
         self.writeFile("test_basic.js", SIMPLE_PASSING_TEST)
-        self.writeManifest([("test_basic.js", "head = missing.js")])
+        self.writeManifest([("test_basic.js", 'head = "missing.js"')])
 
         raised = False
 
