@@ -2,22 +2,13 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-const {
-  PrintUtils,
-  Services,
-  AppConstants,
-} = window.docShell.chromeEventHandler.ownerGlobal;
+const { PrintUtils, Services, AppConstants } =
+  window.docShell.chromeEventHandler.ownerGlobal;
 
-ChromeUtils.defineModuleGetter(
-  this,
-  "DownloadPaths",
-  "resource://gre/modules/DownloadPaths.jsm"
-);
-ChromeUtils.defineModuleGetter(
-  this,
-  "DeferredTask",
-  "resource://gre/modules/DeferredTask.jsm"
-);
+ChromeUtils.defineESModuleGetters(this, {
+  DeferredTask: "resource://gre/modules/DeferredTask.sys.mjs",
+  DownloadPaths: "resource://gre/modules/DownloadPaths.sys.mjs",
+});
 
 const PDF_JS_URI = "resource://pdf.js/web/viewer.html";
 const INPUT_DELAY_MS = Cu.isInAutomation ? 100 : 500;
@@ -29,11 +20,13 @@ const PSSVC = Cc["@mozilla.org/gfx/printsettings-service;1"].getService(
   Ci.nsIPrintSettingsService
 );
 
-var logger = (function() {
+var logger = (function () {
   const getMaxLogLevel = () =>
     Services.prefs.getBoolPref("print.debug", false) ? "all" : "warn";
 
-  let { ConsoleAPI } = ChromeUtils.import("resource://gre/modules/Console.jsm");
+  let { ConsoleAPI } = ChromeUtils.importESModule(
+    "resource://gre/modules/Console.sys.mjs"
+  );
   // Create a new instance of the ConsoleAPI so we can control the maxLogLevel with a pref.
   let _logger = new ConsoleAPI({
     prefix: "printUI",
@@ -53,7 +46,7 @@ var logger = (function() {
   return _logger;
 })();
 
-function serializeSettings(settings, logPrefix) {
+function serializeSettings(settings) {
   let re = /^(k[A-Z]|resolution)/; // accessing settings.resolution throws an exception?
   let types = new Set(["string", "boolean", "number", "undefined"]);
   let nameValues = {};
@@ -87,13 +80,17 @@ function cancelDeferredTasks() {
 
 document.addEventListener(
   "DOMContentLoaded",
-  e => {
-    window._initialized = PrintEventHandler.init().catch(e =>
-      Cu.reportError(e)
-    );
+  () => {
+    const dialogBox = ourBrowser.closest(".dialogBox");
+    if (!dialogBox) {
+      return;
+    }
+
+    window._initialized = PrintEventHandler.init().catch(e => console.error(e));
     ourBrowser.setAttribute("flex", "0");
+    ourBrowser.setAttribute("constrainpopups", "false");
     ourBrowser.classList.add("printSettingsBrowser");
-    ourBrowser.closest(".dialogBox")?.classList.add("printDialogBox");
+    dialogBox.classList.add("printDialogBox");
   },
   { once: true }
 );
@@ -104,7 +101,7 @@ window.addEventListener("dialogclosing", () => {
 
 window.addEventListener(
   "unload",
-  e => {
+  () => {
     document.textContent = "";
   },
   { once: true }
@@ -166,11 +163,10 @@ var PrintEventHandler = {
   _noPreviewUpdateSettings: new Set(["numCopies", "printDuplex"]),
 
   async init() {
-    Services.telemetry.scalarAdd("printing.preview_opened_tm", 1);
+    Glean.printing.previewOpenedTm.add(1);
 
-    this.printPreviewEl = ourBrowser.parentElement.querySelector(
-      "print-preview"
-    );
+    this.printPreviewEl =
+      ourBrowser.parentElement.querySelector("print-preview");
 
     // Do not keep a reference to source browser, it may mutate after printing
     // is initiated and the print preview clone must be a snapshot from the
@@ -196,14 +192,16 @@ var PrintEventHandler = {
     let topWindowContext = sourceBrowsingContext.top.currentWindowContext;
     this.topContentTitle = topWindowContext.documentTitle;
     this.topCurrentURI = topWindowContext.documentURI.spec;
+    this.isReader = this.topCurrentURI.startsWith("about:reader");
 
-    if (!this.hasSelection && !this.isArticle) {
+    let canSimplify = !this.isReader && this.isArticle;
+    if (!this.hasSelection && !canSimplify) {
       document.getElementById("source-version-section").hidden = true;
     } else {
-      document.getElementById("source-version-selection").hidden = !this
-        .hasSelection;
-      document.getElementById("source-version-simplified").hidden = !this
-        .isArticle;
+      document.getElementById("source-version-selection").hidden =
+        !this.hasSelection;
+      document.getElementById("source-version-simplified").hidden =
+        !canSimplify;
     }
 
     // We don't need the sourceBrowsingContext anymore, get rid of it.
@@ -282,11 +280,10 @@ var PrintEventHandler = {
       // Hide the dialog box before opening system dialog
       // We cannot close the window yet because the browsing context for the
       // print preview browser is needed to print the page.
-      let sourceBrowser = this.printPreviewEl.getSourceBrowsingContext().top
-        .embedderElement;
-      let dialogBoxManager = PrintUtils.getTabDialogBox(
-        sourceBrowser
-      ).getTabDialogManager();
+      let sourceBrowser =
+        this.printPreviewEl.getSourceBrowsingContext().top.embedderElement;
+      let dialogBoxManager =
+        PrintUtils.getTabDialogBox(sourceBrowser).getTabDialogManager();
       dialogBoxManager.hideDialog(sourceBrowser);
 
       // Use our settings to prepopulate the system dialog.
@@ -301,25 +298,49 @@ var PrintEventHandler = {
       // system dialog the title will be used to generate the prepopulated
       // filename in the file picker.
       settings.title = this.activeTitle;
-      Services.telemetry.scalarAdd("printing.dialog_opened_via_preview_tm", 1);
+      Glean.printing.dialogOpenedViaPreviewTm.add(1);
       const doPrint = await this._showPrintDialog(
         window,
         this.hasSelection,
         settings
       );
       if (!doPrint) {
-        Services.telemetry.scalarAdd(
-          "printing.dialog_via_preview_cancelled_tm",
-          1
-        );
+        Glean.printing.dialogViaPreviewCancelledTm.add(1);
         window.close();
         return;
       }
       await this.print(settings);
     });
 
-    let settingsToChange = await this.refreshSettings(selectedPrinter.value);
-    await this.updateSettings(settingsToChange, true);
+    let originalError;
+    const printersByPriority = [
+      selectedPrinter.value,
+      ...Object.getOwnPropertyNames(printersByName).filter(
+        name => name != selectedPrinter.value
+      ),
+    ];
+
+    // Try to update settings, falling back to any available printer
+    for (const printerName of printersByPriority) {
+      try {
+        let settingsToChange = await this.refreshSettings(printerName);
+        await this.updateSettings(settingsToChange, true);
+        originalError = null;
+        break;
+      } catch (e) {
+        if (!originalError) {
+          originalError = e;
+          // Report on how often fetching the last used printer settings fails.
+          this.reportPrintingError("PRINTER_SETTINGS_LAST_USED");
+        }
+      }
+    }
+
+    // Only throw original error if no fallback was possible
+    if (originalError) {
+      this.reportPrintingError("PRINTER_SETTINGS");
+      throw originalError;
+    }
 
     let initialPreviewDone = this._updatePrintPreview();
 
@@ -387,7 +408,7 @@ var PrintEventHandler = {
     await window._initialized;
 
     // This seems like it should be handled automatically but it isn't.
-    Services.prefs.setStringPref("print_printer", settings.printerName);
+    PSSVC.maybeSaveLastUsedPrinterNameToPrefs(settings.printerName);
 
     try {
       // We'll provide our own progress indicator.
@@ -400,7 +421,7 @@ var PrintEventHandler = {
       let bc = this.printPreviewEl.currentBrowsingContext;
       await this._doPrint(bc, settings);
     } catch (e) {
-      Cu.reportError(e);
+      console.error(e);
     }
 
     if (settings.printerName == PrintUtils.SAVE_TO_PDF_PRINTER) {
@@ -426,7 +447,7 @@ var PrintEventHandler = {
   },
 
   cancelPrint() {
-    Services.telemetry.scalarAdd("printing.preview_cancelled_tm", 1);
+    Glean.printing.previewCancelledTm.add(1);
     window.close();
   },
 
@@ -434,9 +455,8 @@ var PrintEventHandler = {
     this.currentPrinterName = printerName;
     let currentPrinter;
     try {
-      currentPrinter = await PrintSettingsViewProxy.resolvePropertiesForPrinter(
-        printerName
-      );
+      currentPrinter =
+        await PrintSettingsViewProxy.resolvePropertiesForPrinter(printerName);
     } catch (e) {
       this.reportPrintingError("PRINTER_PROPERTIES");
       throw e;
@@ -602,11 +622,7 @@ var PrintEventHandler = {
   async onUserSettingsChange(changedSettings = {}) {
     let previewableChange = false;
     for (let [setting, value] of Object.entries(changedSettings)) {
-      Services.telemetry.keyedScalarAdd(
-        "printing.settings_changed",
-        setting,
-        1
-      );
+      Glean.printing.settingsChanged[setting].add(1);
       // Update the list of user-changed settings, which we attempt to maintain
       // across printer changes.
       this._userChangedSettings[setting] = value;
@@ -697,6 +713,7 @@ var PrintEventHandler = {
           );
         }
 
+        let unwriteableMarginsInvalid = false;
         if (
           verticalMarginsInvalid(this.viewSettings.customMargins) ||
           this.viewSettings.customMargins.marginTop < 0 ||
@@ -707,9 +724,12 @@ var PrintEventHandler = {
             let marginsNone = this.getMarginPresets("none");
             marginTop = marginsNone.marginTop;
             marginBottom = marginsNone.marginBottom;
+            unwriteableMarginsInvalid = true;
           }
-          changedSettings.marginTop = changedSettings.customMarginTop = marginTop;
-          changedSettings.marginBottom = changedSettings.customMarginBottom = marginBottom;
+          changedSettings.marginTop = changedSettings.customMarginTop =
+            marginTop;
+          changedSettings.marginBottom = changedSettings.customMarginBottom =
+            marginBottom;
           delete this._userChangedSettings.customMargins;
         }
 
@@ -723,11 +743,18 @@ var PrintEventHandler = {
             let marginsNone = this.getMarginPresets("none");
             marginLeft = marginsNone.marginLeft;
             marginRight = marginsNone.marginRight;
+            unwriteableMarginsInvalid = true;
           }
-          changedSettings.marginLeft = changedSettings.customMarginLeft = marginLeft;
-          changedSettings.marginRight = changedSettings.customMarginRight = marginRight;
+          changedSettings.marginLeft = changedSettings.customMarginLeft =
+            marginLeft;
+          changedSettings.marginRight = changedSettings.customMarginRight =
+            marginRight;
           delete this._userChangedSettings.customMargins;
         }
+
+        if (unwriteableMarginsInvalid) {
+          changedSettings.ignoreUnwriteableMargins = true;
+        }
       } catch (e) {
         this.reportPrintingError("PAPER_MARGINS");
         throw e;
@@ -756,9 +783,8 @@ var PrintEventHandler = {
         ) {
           flags |= this.settingFlags[setting];
         }
-        updatePreviewWithoutFlag |= this._nonFlaggedUpdatePreviewSettings.has(
-          setting
-        );
+        updatePreviewWithoutFlag |=
+          this._nonFlaggedUpdatePreviewSettings.has(setting);
       }
     }
 
@@ -777,7 +803,7 @@ var PrintEventHandler = {
   },
 
   saveSettingsToPrefs(flags) {
-    PSSVC.savePrintSettingsToPrefs(this.settings, true, flags);
+    PSSVC.maybeSavePrintSettingsToPrefs(this.settings, flags);
   },
 
   /**
@@ -798,37 +824,30 @@ var PrintEventHandler = {
   async _updatePrintPreview() {
     let { settings } = this;
 
-    const isFirstCall = !this.printInitiationTime;
-    if (isFirstCall) {
-      let params = new URLSearchParams(location.search);
-      this.printInitiationTime = parseInt(
-        params.get("printInitiationTime"),
-        10
-      );
-      const elapsed = Date.now() - this.printInitiationTime;
-      Services.telemetry
-        .getHistogramById("PRINT_INIT_TO_PLATFORM_SENT_SETTINGS_MS")
-        .add(elapsed);
-    }
-
-    let totalPageCount, sheetCount, isEmpty, orientation;
+    let totalPageCount, sheetCount, isEmpty, orientation, pageWidth, pageHeight;
     try {
       // This resolves with a PrintPreviewSuccessInfo dictionary.
       let { sourceVersion } = this.viewSettings;
       let sourceURI = this.activeURI;
+      // The printing backend can't generate a title for the selection document
+      // since it is only a fragment of the page, give it the active title.
+      settings.title =
+        this.viewSettings.sourceVersion == "selection" ? this.activeTitle : "";
       this._lastPrintPreviewSettings = settings;
       ({
         totalPageCount,
         sheetCount,
         isEmpty,
         orientation,
+        pageWidth,
+        pageHeight,
       } = await this.printPreviewEl.printPreview(settings, {
         sourceVersion,
         sourceURI,
       }));
     } catch (e) {
       this.reportPrintingError("PRINT_PREVIEW");
-      Cu.reportError(e);
+      console.error(e);
       throw e;
     }
 
@@ -844,6 +863,35 @@ var PrintEventHandler = {
       document.dispatchEvent(new CustomEvent("hide-orientation"));
     }
 
+    // If the page size is set, check whether we should use it as our paper size.
+    let isUsingPageRuleSizeAsPaperSize =
+      settings.usePageRuleSizeAsPaperSize &&
+      pageWidth !== null &&
+      pageHeight !== null;
+    if (isUsingPageRuleSizeAsPaperSize) {
+      // We canonically represent paper sizes using the width/height of a portrait-oriented sheet,
+      // with landscape-orientation applied as a supplemental rotation.
+      // If the page-size is landscape oriented, we flip the pageWidth / pageHeight here
+      // in order to pass a canonical representation into the paper-size settings.
+      if (orientation == "landscape") {
+        [pageHeight, pageWidth] = [pageWidth, pageHeight];
+      }
+
+      let matchedPaper = PrintSettingsViewProxy.getBestPaperMatch(
+        pageWidth,
+        pageHeight,
+        settings.kPaperSizeInches
+      );
+      if (matchedPaper) {
+        settings.paperId = matchedPaper.id;
+      }
+
+      settings.paperWidth = pageWidth;
+      settings.paperHeight = pageHeight;
+      settings.paperSizeUnit = settings.kPaperSizeInches;
+      document.dispatchEvent(new CustomEvent("hide-paper-size"));
+    }
+
     this.previewIsEmpty = isEmpty;
     // If the preview is empty, we know our range is greater than the number of pages.
     // We have to send a pageRange update to display a non-empty page.
@@ -857,13 +905,6 @@ var PrintEventHandler = {
         detail: { sheetCount, totalPages: totalPageCount },
       })
     );
-
-    if (isFirstCall) {
-      const elapsed = Date.now() - this.printInitiationTime;
-      Services.telemetry
-        .getHistogramById("PRINT_INIT_TO_PREVIEW_DOC_SHOWN_MS")
-        .add(elapsed);
-    }
   },
 
   async getPrintDestinations() {
@@ -1004,7 +1045,7 @@ var PrintEventHandler = {
 
   reportPrintingError(aMessage) {
     logger.debug("reportPrintingError:", aMessage);
-    Services.telemetry.keyedScalarAdd("printing.error", aMessage, 1);
+    Glean.printing.error[aMessage].add(1);
   },
 
   /**
@@ -1181,6 +1222,11 @@ var PrintSettingsViewProxy = {
         Ci.nsIPrintSettings.kOutputFormatPDF;
       printerInfo.defaultSettings.outputDestination =
         Ci.nsIPrintSettings.kOutputDestinationFile;
+      printerInfo.defaultSettings.usePageRuleSizeAsPaperSize =
+        Services.prefs.getBoolPref(
+          "print.save_as_pdf.use_page_rule_size_as_paper_size.enabled",
+          false
+        );
       printerInfo.paperList = this.fallbackPaperList;
     }
     printerInfo.settings = printerInfo.defaultSettings.clone();
@@ -1265,12 +1311,8 @@ var PrintSettingsViewProxy = {
         let uniqueMargins = new Set();
         let marginsEnabled = {};
         for (let name of ["none", "default", "minimum", "custom"]) {
-          let {
-            marginTop,
-            marginLeft,
-            marginBottom,
-            marginRight,
-          } = allMarginPresets[name];
+          let { marginTop, marginLeft, marginBottom, marginRight } =
+            allMarginPresets[name];
           let key = [marginTop, marginLeft, marginBottom, marginRight].join(
             ","
           );
@@ -1290,15 +1332,31 @@ var PrintSettingsViewProxy = {
         };
         // see if they match the none, minimum, or default margin values
         let allMarginPresets = this.get(target, "marginPresets");
-        for (let presetName of ["none", "minimum", "default"]) {
+        const marginsMatch = function (lhs, rhs) {
+          return Object.keys(marginSettings).every(
+            name => lhs[name].toFixed(2) == rhs[name].toFixed(2)
+          );
+        };
+        const potentialPresets = (function () {
+          let presets = [];
+          const minimumIsNone = marginsMatch(
+            allMarginPresets.none,
+            allMarginPresets.minimum
+          );
+          // We only attempt to match the serialized values against the "none"
+          // preset if the unwriteable margins are being ignored or are zero.
+          if (target.ignoreUnwriteableMargins || minimumIsNone) {
+            presets.push("none");
+          }
+          if (!minimumIsNone) {
+            presets.push("minimum");
+          }
+          presets.push("default");
+          return presets;
+        })();
+        for (let presetName of potentialPresets) {
           let marginPresets = allMarginPresets[presetName];
-          if (
-            Object.keys(marginSettings).every(
-              name =>
-                marginSettings[name].toFixed(2) ==
-                marginPresets[name].toFixed(2)
-            )
-          ) {
+          if (marginsMatch(marginSettings, marginPresets)) {
             return presetName;
           }
         }
@@ -1395,6 +1453,7 @@ var PrintSettingsViewProxy = {
           target[settingName] = presetValue;
         }
         target.honorPageRuleMargins = value == "default";
+        target.ignoreUnwriteableMargins = value == "none";
         break;
 
       case "paperId": {
@@ -1433,7 +1492,7 @@ var PrintSettingsViewProxy = {
         break;
 
       case "printDuplex": {
-        let duplex = (function() {
+        let duplex = (function () {
           switch (value) {
             case "off":
               break;
@@ -1530,7 +1589,7 @@ function PrintUIControlMixin(superClass) {
 
     render() {}
 
-    update(settings) {}
+    update() {}
 
     dispatchSettingsChange(changedSettings) {
       this.dispatchEvent(
@@ -1550,7 +1609,7 @@ function PrintUIControlMixin(superClass) {
       );
     }
 
-    handleEvent(event) {}
+    handleEvent() {}
   };
 }
 
@@ -1563,16 +1622,7 @@ class PrintUIForm extends PrintUIControlMixin(HTMLFormElement) {
     this.addEventListener("revalidate", this);
 
     this._printerDestination = this.querySelector("#destination");
-
     this.printButton = this.querySelector("#print-button");
-    if (AppConstants.platform != "win") {
-      // Move the Print button to the end if this isn't Windows.
-      this.printButton.parentElement.append(this.printButton);
-    }
-    this.querySelector("#pages-per-sheet").hidden = !Services.prefs.getBoolPref(
-      "print.pages_per_sheet.enabled",
-      false
-    );
   }
 
   removeNonPdfSettings() {
@@ -1734,8 +1784,8 @@ customElements.define("setting-select", PrintSettingSelect, {
 class PrintSettingNumber extends PrintUIControlMixin(HTMLInputElement) {
   initialize() {
     super.initialize();
-    this.addEventListener("keypress", e => this.handleKeypress(e));
-    this.addEventListener("paste", e => this.handlePaste(e));
+    this.addEventListener("beforeinput", e => this.preventWhitespaceEntry(e));
+    this.addEventListener("paste", e => this.pasteWithoutWhitespace(e));
   }
 
   connectedCallback() {
@@ -1750,36 +1800,25 @@ class PrintSettingNumber extends PrintUIControlMixin(HTMLInputElement) {
     }
   }
 
-  handleKeypress(e) {
-    let char = String.fromCharCode(e.charCode);
-    let acceptedChar = e.target.step.includes(".")
-      ? char.match(/^[0-9.]$/)
-      : char.match(/^[0-9]$/);
-    if (!acceptedChar && !char.match("\x00") && !e.ctrlKey && !e.metaKey) {
+  preventWhitespaceEntry(e) {
+    if (e.data && !e.data.trim().length) {
       e.preventDefault();
     }
   }
 
-  handlePaste(e) {
+  pasteWithoutWhitespace(e) {
+    // Prevent original value from being pasted
+    e.preventDefault();
+
+    // Manually update input's value with sanitized clipboard data
     let paste = (e.clipboardData || window.clipboardData)
       .getData("text")
       .trim();
-    let acceptedChars = e.target.step.includes(".")
-      ? paste.match(/^[0-9.]*$/)
-      : paste.match(/^[0-9]*$/);
-    if (!acceptedChars) {
-      e.preventDefault();
-    }
+    this.value = paste;
   }
 
   handleEvent(e) {
     switch (e.type) {
-      case "paste":
-        this.handlePaste();
-        break;
-      case "keypress":
-        this.handleKeypress();
-        break;
       case "input":
         if (this.settingName && this.checkValidity()) {
           this.dispatchSettingsChange({
@@ -1805,7 +1844,7 @@ class PrintSettingCheckbox extends PrintUIControlMixin(HTMLInputElement) {
     this.checked = settings[this.settingName];
   }
 
-  handleEvent(e) {
+  handleEvent() {
     this.dispatchSettingsChange({
       [this.settingName]: this.checked,
     });
@@ -1826,7 +1865,7 @@ class PrintSettingRadio extends PrintUIControlMixin(HTMLInputElement) {
     this.checked = settings[this.settingName] == this.value;
   }
 
-  handleEvent(e) {
+  handleEvent() {
     this.dispatchSettingsChange({
       [this.settingName]: this.value,
     });
@@ -1888,6 +1927,8 @@ class PaperSizePicker extends PrintSettingSelect {
   initialize() {
     super.initialize();
     this._printerName = null;
+    this._section = this.closest(".section-block");
+    document.addEventListener("hide-paper-size", this);
   }
 
   update(settings) {
@@ -1896,6 +1937,19 @@ class PaperSizePicker extends PrintSettingSelect {
       this.setOptions(settings.paperSizes);
     }
     this.value = settings.paperId;
+
+    // Unhide the paper-size picker, if we've stopped using the page size as paper-size.
+    if (this._section.hidden && !settings.usePageRuleSizeAsPaperSize) {
+      this._section.hidden = false;
+    }
+  }
+
+  handleEvent(e) {
+    super.handleEvent(e);
+    const { type } = e;
+    if (type == "hide-paper-size") {
+      this._section.hidden = true;
+    }
   }
 }
 customElements.define("paper-size-select", PaperSizePicker, {
@@ -1947,7 +2001,7 @@ class CopiesInput extends PrintUIControlMixin(HTMLElement) {
     this._copiesError.hidden = true;
   }
 
-  handleEvent(e) {
+  handleEvent() {
     this._copiesError.hidden = this._copiesInput.checkValidity();
   }
 }
@@ -1990,7 +2044,7 @@ class ScaleInput extends PrintUIControlMixin(HTMLElement) {
     // we repopulate the scale field with the stored, valid scaling value.
     let isValid = this._percentScale.checkValidity();
     if (
-      !this._percentScale.value ||
+      this._percentScale.value != parseInt(scaling * 100, 10) ||
       (this._shrinkToFitChoice.checked && !isValid) ||
       (this.printerName != printerName && !isValid)
     ) {
@@ -2063,8 +2117,18 @@ class PageRangeInput extends PrintUIControlMixin(HTMLElement) {
 
   updatePageRange() {
     let isCustom = this._rangePicker.value == "custom";
+    let isCurrent = this._rangePicker.value == "current";
+
+    if (!isCurrent) {
+      this._currentPage = null;
+    }
+
     if (isCustom) {
       this.validateRangeInput();
+    } else if (isCurrent) {
+      this._currentPage = this._rangeInput.value =
+        this._currentPage || this.getCurrentVisiblePageNumber();
+      this.validateRangeInput();
     } else {
       this._pagesSet.clear();
 
@@ -2165,12 +2229,14 @@ class PageRangeInput extends PrintUIControlMixin(HTMLElement) {
 
   update(settings) {
     let { pageRanges, printerName } = settings;
+
     this.toggleAttribute("all-pages", !pageRanges.length);
     if (!this.printerName) {
       this.printerName = printerName;
     }
 
     let isValid = this._rangeInput.checkValidity();
+
     if (this.printerName != printerName && !isValid) {
       this.printerName = printerName;
       this._rangeInput.value = "";
@@ -2255,11 +2321,21 @@ class PageRangeInput extends PrintUIControlMixin(HTMLElement) {
   }
 
   validateRangeInput() {
-    let value =
-      this._rangePicker.value == "custom" ? this._rangeInput.value : "";
+    let value = ["custom", "current"].includes(this._rangePicker.value)
+      ? this._rangeInput.value
+      : "";
     this._validateRangeInput(value, this._numPages);
   }
 
+  getCurrentVisiblePageNumber() {
+    let pageNum = parseInt(
+      PrintEventHandler.printPreviewEl.lastPreviewBrowser.getAttribute(
+        "current-page"
+      )
+    );
+    return isNaN(pageNum) ? 1 : pageNum;
+  }
+
   handleEvent(e) {
     if (e.type == "keypress") {
       if (e.target == this._rangeInput) {
@@ -2474,7 +2550,7 @@ class MarginsPicker extends PrintUIControlMixin(HTMLElement) {
         this._sizeUnit == settings.kPaperSizeMillimeters ? "mm" : "inches";
       for (let elem of this.querySelectorAll("[data-unit-prefix-l10n-id]")) {
         let l10nId = elem.getAttribute("data-unit-prefix-l10n-id") + unitStr;
-        elem.setAttribute("data-l10n-id", l10nId);
+        document.l10n.setAttributes(elem, l10nId);
       }
     }
 
@@ -2584,7 +2660,7 @@ class TwistySummary extends PrintUIControlMixin(HTMLElement) {
     this.updateSummary(shouldOpen);
   }
 
-  handleEvent(e) {
+  handleEvent() {
     let willOpen = !this.isOpen;
     Services.prefs.setBoolPref("print.more-settings.open", willOpen);
     this.updateSummary(willOpen);
@@ -2716,7 +2792,7 @@ async function pickFileName(contentTitle, currentURI) {
   filename = DownloadPaths.sanitize(filename);
 
   picker.init(
-    window.docShell.chromeEventHandler.ownerGlobal,
+    window.docShell.chromeEventHandler.ownerGlobal.browsingContext,
     title,
     Ci.nsIFilePicker.modeSave
   );
