@@ -32,7 +32,7 @@
 #include "mozilla/Preferences.h"
 #include "mozilla/ScopeExit.h"
 #include "mozilla/TimeStamp.h"
-#include "mozilla/Telemetry.h"
+#include "mozilla/glean/UrlClassifierMetrics.h"
 #include "mozilla/Unused.h"
 #include "mozilla/Logging.h"
 #include "prnetdb.h"
@@ -59,6 +59,7 @@
 #include "nsIUploadChannel.h"
 #include "nsStringStream.h"
 #include "nsNetUtil.h"
+#include "nsThreadUtils.h"
 #include "nsToolkitCompsCID.h"
 
 namespace mozilla {
@@ -159,19 +160,16 @@ class nsUrlClassifierDBService::FeatureHolder final {
     MOZ_ASSERT(!NS_IsMainThread());
     MOZ_ASSERT(aWorker);
 
-    mozilla::Telemetry::AutoTimer<
-        mozilla::Telemetry::URLCLASSIFIER_CL_CHECK_TIME>
-        timer;
+    auto timer = mozilla::glean::urlclassifier::cl_check_time.Measure();
 
     // Get the set of fragments based on the url. This is necessary because we
     // only look up at most 5 URLs per aSpec, even if aSpec has more than 5
     // components.
     nsTArray<nsCString> fragments;
-    nsresult rv = LookupCache::GetLookupFragments(aSpec, &fragments);
-    NS_ENSURE_SUCCESS(rv, rv);
+    LookupCache::GetLookupFragments(aSpec, &fragments);
 
     for (TableData* tableData : mTableData) {
-      rv = aWorker->DoSingleLocalLookupWithURIFragments(
+      nsresult rv = aWorker->DoSingleLocalLookupWithURIFragments(
           fragments, tableData->mTable, tableData->mResults);
       if (NS_WARN_IF(NS_FAILED(rv))) {
         return rv;
@@ -231,12 +229,52 @@ class nsUrlClassifierDBService::FeatureHolder final {
   }
 
   ~FeatureHolder() {
-    for (FeatureData& featureData : mFeatureData) {
-      NS_ReleaseOnMainThread("FeatureHolder:mFeatureData",
-                             featureData.mFeature.forget());
+    class FeatureHolderRelease final : public mozilla::Runnable {
+     public:
+      explicit FeatureHolderRelease(nsTArray<nsCOMPtr<nsISupports>>&& aDoomed)
+          : Runnable("FeatureHolderRelease"), mDoomed(std::move(aDoomed)) {}
+
+      NS_IMETHOD Run() override {
+        mDoomed.Clear();
+        return NS_OK;
+      }
+
+     private:
+      ~FeatureHolderRelease() {
+        // If we still have some references, let's forget them to avoid crashes.
+        // Probably we are shutdown.
+        for (nsCOMPtr<nsISupports>& doomed : mDoomed) {
+          mozilla::Unused << doomed.forget().take();
+        }
+      }
+
+      nsTArray<nsCOMPtr<nsISupports>> mDoomed;
+    };
+
+    nsTArray<nsCOMPtr<nsISupports>> arrayToRelease;
+
+    if (mURI) {
+      arrayToRelease.AppendElement(mURI.forget());
     }
 
-    NS_ReleaseOnMainThread("FeatureHolder:mURI", mURI.forget());
+    for (FeatureData& featureData : mFeatureData) {
+      if (featureData.mFeature) {
+        arrayToRelease.AppendElement(featureData.mFeature.forget());
+      }
+    }
+
+    if (arrayToRelease.IsEmpty()) {
+      return;
+    }
+
+    if (NS_IsMainThread()) {
+      arrayToRelease.Clear();
+      return;
+    }
+
+    RefPtr<FeatureHolderRelease> runnable(
+        new FeatureHolderRelease(std::move(arrayToRelease)));
+    NS_DispatchToMainThread(runnable.forget());
   }
 
   TableData* GetOrCreateTableData(const nsACString& aTable) {
@@ -286,7 +324,8 @@ static Atomic<bool> gShuttingDownThread(false);
 NS_IMPL_ISUPPORTS(nsUrlClassifierDBServiceWorker, nsIUrlClassifierDBService)
 
 nsUrlClassifierDBServiceWorker::nsUrlClassifierDBServiceWorker()
-    : mInStream(false),
+    : mUpdateObserverLock("nsUrlClassifierDBServerWorker.mUpdateObserverLock"),
+      mInStream(false),
       mGethashNoise(0),
       mPendingLookupLock("nsUrlClassifierDBServerWorker.mPendingLookupLock") {}
 
@@ -435,9 +474,8 @@ nsresult nsUrlClassifierDBServiceWorker::HandlePendingLookups() {
       MutexAutoUnlock unlock(mPendingLookupLock);
       DoLookup(lookup.mKey, lookup.mFeatureHolder, lookup.mCallback);
     }
-    double lookupTime = (TimeStamp::Now() - lookup.mStartTime).ToMilliseconds();
-    Telemetry::Accumulate(Telemetry::URLCLASSIFIER_LOOKUP_TIME_2,
-                          static_cast<uint32_t>(lookupTime));
+    glean::urlclassifier::lookup_time_2.AccumulateRawDuration(
+        TimeStamp::Now() - lookup.mStartTime);
   }
 
   return NS_OK;
@@ -517,7 +555,10 @@ void nsUrlClassifierDBServiceWorker::ResetUpdate() {
   LOG(("ResetUpdate"));
   mUpdateWaitSec = 0;
   mUpdateStatus = NS_OK;
-  mUpdateObserver = nullptr;
+  {
+    MutexAutoLock lock(mUpdateObserverLock);
+    mUpdateObserver = nullptr;
+  }
 }
 
 NS_IMETHODIMP
@@ -536,18 +577,21 @@ nsUrlClassifierDBServiceWorker::BeginUpdate(
     return NS_ERROR_NOT_INITIALIZED;
   }
 
-  NS_ENSURE_STATE(!mUpdateObserver);
+  {
+    MutexAutoLock lock(mUpdateObserverLock);
+    NS_ENSURE_STATE(!mUpdateObserver);
 
-  nsresult rv = OpenDb();
-  if (NS_FAILED(rv)) {
-    NS_ERROR("Unable to open SafeBrowsing database");
-    return NS_ERROR_FAILURE;
+    nsresult rv = OpenDb();
+    if (NS_FAILED(rv)) {
+      NS_ERROR("Unable to open SafeBrowsing database");
+      return NS_ERROR_FAILURE;
+    }
+
+    mUpdateStatus = NS_OK;
+    MOZ_ASSERT(mTableUpdates.IsEmpty(),
+               "mTableUpdates should have been cleared in FinishUpdate()");
+    mUpdateObserver = observer;
   }
-
-  mUpdateStatus = NS_OK;
-  MOZ_ASSERT(mTableUpdates.IsEmpty(),
-             "mTableUpdates should have been cleared in FinishUpdate()");
-  mUpdateObserver = observer;
   Classifier::SplitTables(tables, mUpdateTables);
 
   return NS_OK;
@@ -563,7 +607,10 @@ nsUrlClassifierDBServiceWorker::BeginStream(const nsACString& table) {
     return NS_ERROR_NOT_INITIALIZED;
   }
 
-  NS_ENSURE_STATE(mUpdateObserver);
+  {
+    MutexAutoLock lock(mUpdateObserverLock);
+    NS_ENSURE_STATE(mUpdateObserver);
+  }
   NS_ENSURE_STATE(!mInStream);
 
   mInStream = true;
@@ -656,6 +703,8 @@ nsUrlClassifierDBServiceWorker::FinishStream() {
     return NS_ERROR_NOT_INITIALIZED;
   }
 
+  MutexAutoLock lock(mUpdateObserverLock);
+
   MOZ_ASSERT(mProtocolParser);
 
   NS_ENSURE_STATE(mInStream);
@@ -681,11 +730,6 @@ nsUrlClassifierDBServiceWorker::FinishStream() {
     mTableUpdates.AppendElements(mProtocolParser->GetTableUpdates());
     mProtocolParser->ForgetTableUpdates();
 
-#ifdef MOZ_SAFEBROWSING_DUMP_FAILED_UPDATES
-    // The assignment involves no string copy since the source string is
-    // sharable.
-    mRawTableUpdates = mProtocolParser->GetRawTableUpdates();
-#endif
   } else {
     LOG(
         ("nsUrlClassifierDBService::FinishStream Failed to parse the stream "
@@ -722,7 +766,10 @@ nsUrlClassifierDBServiceWorker::FinishUpdate() {
              "Should have been nulled out in FinishStream() "
              "or never created.");
 
-  NS_ENSURE_STATE(mUpdateObserver);
+  {
+    MutexAutoLock lock(mUpdateObserverLock);
+    NS_ENSURE_STATE(mUpdateObserver);
+  }
 
   if (NS_FAILED(mUpdateStatus)) {
     LOG(
@@ -739,18 +786,8 @@ nsUrlClassifierDBServiceWorker::FinishUpdate() {
 
   RefPtr<nsUrlClassifierDBServiceWorker> self = this;
   nsresult rv = mClassifier->AsyncApplyUpdates(
-      mTableUpdates, [self](nsresult aRv) -> void {
-#ifdef MOZ_SAFEBROWSING_DUMP_FAILED_UPDATES
-        if (NS_FAILED(aRv) && NS_ERROR_OUT_OF_MEMORY != aRv &&
-            NS_ERROR_UC_UPDATE_SHUTDOWNING != aRv) {
-          self->mClassifier->DumpRawTableUpdates(self->mRawTableUpdates);
-        }
-        // Invalidate the raw table updates.
-        self->mRawTableUpdates.Truncate();
-#endif
-
-        self->NotifyUpdateObserver(aRv);
-      });
+      mTableUpdates,
+      [self](nsresult aRv) -> void { self->NotifyUpdateObserver(aRv); });
   mTableUpdates.Clear();  // Classifier is working on its copy.
 
   if (NS_FAILED(rv)) {
@@ -800,10 +837,12 @@ nsresult nsUrlClassifierDBServiceWorker::NotifyUpdateObserver(
 
   // Do not record telemetry for testing tables.
   if (!provider.EqualsLiteral(TESTING_TABLE_PROVIDER_NAME)) {
-    Telemetry::Accumulate(Telemetry::URLCLASSIFIER_UPDATE_ERROR, provider,
-                          NS_ERROR_GET_CODE(updateStatus));
+    glean::urlclassifier::update_error.Get(provider).AccumulateSingleSample(
+        NS_ERROR_GET_CODE(updateStatus));
   }
 
+  MutexAutoLock lock(mUpdateObserverLock);
+
   if (!mUpdateObserver) {
     // In the normal shutdown process, CancelUpdate() would NOT be
     // called prior to NotifyUpdateObserver(). However, CancelUpdate()
@@ -888,25 +927,30 @@ NS_IMETHODIMP
 nsUrlClassifierDBServiceWorker::CancelUpdate() {
   LOG(("nsUrlClassifierDBServiceWorker::CancelUpdate"));
 
-  if (mUpdateObserver) {
+  {
+    MutexAutoLock lock(mUpdateObserverLock);
+    if (!mUpdateObserver) {
+      LOG(("No UpdateObserver, nothing to cancel"));
+
+      return NS_OK;
+    }
+
     LOG(("UpdateObserver exists, cancelling"));
 
     mUpdateStatus = NS_BINDING_ABORTED;
 
     mUpdateObserver->UpdateError(mUpdateStatus);
-
-    /*
-     * mark the tables as spoiled(clear cache in LookupCache), we don't want to
-     * block hosts longer than normal because our update failed
-     */
-    mClassifier->ResetTables(Classifier::Clear_Cache, mUpdateTables);
-
-    ResetStream();
-    ResetUpdate();
-  } else {
-    LOG(("No UpdateObserver, nothing to cancel"));
   }
 
+  /*
+   * mark the tables as spoiled(clear cache in LookupCache), we don't want to
+   * block hosts longer than normal because our update failed
+   */
+  mClassifier->ResetTables(Classifier::Clear_Cache, mUpdateTables);
+
+  ResetStream();
+  ResetUpdate();
+
   return NS_OK;
 }
 
@@ -1454,6 +1498,11 @@ nsresult nsUrlClassifierLookupCallback::CacheMisses() {
   return NS_OK;
 }
 
+struct LiteralProvider {
+  nsLiteralCString name;
+  uint8_t priority;
+};
+
 struct Provider {
   nsCString name;
   uint8_t priority;
@@ -1461,7 +1510,7 @@ struct Provider {
 
 // Order matters
 // Provider which is not included in this table has the lowest priority 0
-static const Provider kBuiltInProviders[] = {
+static constexpr LiteralProvider kBuiltInProviders[] = {
     {"mozilla"_ns, 1},
     {"google4"_ns, 2},
     {"google"_ns, 3},
@@ -1552,9 +1601,9 @@ nsUrlClassifierClassifyCallback::HandleResult(const nsACString& aTable,
 
   matchedInfo->provider.name = NS_SUCCEEDED(rv) ? provider : ""_ns;
   matchedInfo->provider.priority = 0;
-  for (uint8_t i = 0; i < ArrayLength(kBuiltInProviders); i++) {
-    if (kBuiltInProviders[i].name.Equals(matchedInfo->provider.name)) {
-      matchedInfo->provider.priority = kBuiltInProviders[i].priority;
+  for (auto const& BuiltInProvider : kBuiltInProviders) {
+    if (BuiltInProvider.name.Equals(matchedInfo->provider.name)) {
+      matchedInfo->provider.priority = BuiltInProvider.priority;
     }
   }
   matchedInfo->errorCode = TablesToResponse(aTable);
@@ -1707,11 +1756,10 @@ nsUrlClassifierDBService::Classify(nsIPrincipal* aPrincipal,
     MOZ_ASSERT(content);
 
     auto actor = static_cast<URLClassifierChild*>(
-        content->AllocPURLClassifierChild(IPC::Principal(aPrincipal), aResult));
+        content->AllocPURLClassifierChild(aPrincipal, aResult));
     MOZ_ASSERT(actor);
 
-    if (!content->SendPURLClassifierConstructor(
-            actor, IPC::Principal(aPrincipal), aResult)) {
+    if (!content->SendPURLClassifierConstructor(actor, aPrincipal, aResult)) {
       *aResult = false;
       return NS_ERROR_FAILURE;
     }
@@ -1857,14 +1905,13 @@ ThreatHitReportListener::OnStopRequest(nsIRequest* aRequest, nsresult aStatus) {
   uint8_t netErrCode =
       NS_FAILED(aStatus) ? mozilla::safebrowsing::NetworkErrorToBucket(aStatus)
                          : 0;
-  mozilla::Telemetry::Accumulate(
-      mozilla::Telemetry::URLCLASSIFIER_THREATHIT_NETWORK_ERROR, netErrCode);
+  mozilla::glean::urlclassifier::threathit_network_error.AccumulateSingleSample(
+      netErrCode);
 
   uint32_t requestStatus;
   nsresult rv = httpChannel->GetResponseStatus(&requestStatus);
   NS_ENSURE_SUCCESS(rv, aStatus);
-  mozilla::Telemetry::Accumulate(
-      mozilla::Telemetry::URLCLASSIFIER_THREATHIT_REMOTE_STATUS,
+  mozilla::glean::urlclassifier::threathit_remote_status.AccumulateSingleSample(
       mozilla::safebrowsing::HTTPStatusToBucket(requestStatus));
 
   if (LOG_ENABLED()) {
@@ -1945,7 +1992,7 @@ nsUrlClassifierDBService::SendThreatHitReport(nsIChannel* aChannel,
   NS_ENSURE_SUCCESS(rv, rv);
   nsCOMPtr<nsIStringInputStream> sis(
       do_CreateInstance(NS_STRINGINPUTSTREAM_CONTRACTID));
-  rv = sis->SetData(reportBody.get(), reportBody.Length());
+  rv = sis->SetByteStringData(reportBody);
   NS_ENSURE_SUCCESS(rv, rv);
 
   LOG(("Sending the following ThreatHit report to %s about %s: %s",
@@ -2116,6 +2163,7 @@ nsUrlClassifierDBService::BeginUpdate(nsIUrlClassifierUpdateObserver* observer,
     LOG(("Already updating, not available"));
     return NS_ERROR_NOT_AVAILABLE;
   }
+
   if (mWorker->IsBusyUpdating()) {
     // |mInUpdate| used to work well because "notifying update observer"
     // is synchronously done in Worker::FinishUpdate(). Even if the
@@ -2306,7 +2354,7 @@ nsresult nsUrlClassifierDBService::Shutdown() {
     return NS_OK;
   }
 
-  Telemetry::AutoTimer<Telemetry::URLCLASSIFIER_SHUTDOWN_TIME> timer;
+  auto timer = glean::urlclassifier::shutdown_time.Measure();
 
   mCompleters.Clear();
 
@@ -2388,7 +2436,7 @@ NS_IMETHODIMP
 nsUrlClassifierDBService::AsyncClassifyLocalWithFeatures(
     nsIURI* aURI, const nsTArray<RefPtr<nsIUrlClassifierFeature>>& aFeatures,
     nsIUrlClassifierFeature::listType aListType,
-    nsIUrlClassifierFeatureCallback* aCallback) {
+    nsIUrlClassifierFeatureCallback* aCallback, bool aIdlePriority) {
   MOZ_ASSERT(NS_IsMainThread());
 
   if (gShuttingDownThread) {
@@ -2439,16 +2487,7 @@ nsUrlClassifierDBService::AsyncClassifyLocalWithFeatures(
         continue;
       }
 
-      nsAutoCString exceptionHostList;
-      if (aListType == nsIUrlClassifierFeature::blocklist) {
-        rv = feature->GetExceptionHostList(exceptionHostList);
-        if (NS_WARN_IF(NS_FAILED(rv))) {
-          continue;
-        }
-      }
-
-      ipcFeatures.AppendElement(
-          IPCURLClassifierFeature(name, tables, exceptionHostList));
+      ipcFeatures.AppendElement(IPCURLClassifierFeature(name, tables));
     }
 
     if (!content->SendPURLClassifierLocalConstructor(actor, aURI,
@@ -2479,15 +2518,15 @@ nsUrlClassifierDBService::AsyncClassifyLocalWithFeatures(
 
   nsCOMPtr<nsIRunnable> r = NS_NewRunnableFunction(
       "nsUrlClassifierDBService::AsyncClassifyLocalWithFeatures",
-      [worker, key, holder, callback, startTime]() -> void {
+      [worker, key, holder, callback, startTime, aIdlePriority]() -> void {
         holder->DoLocalLookup(key, worker);
 
         nsCOMPtr<nsIRunnable> cbRunnable = NS_NewRunnableFunction(
             "nsUrlClassifierDBService::AsyncClassifyLocalWithFeatures",
             [callback, holder, startTime]() -> void {
               // Measure the time diff between calling and callback.
-              AccumulateTimeDelta(
-                  Telemetry::URLCLASSIFIER_ASYNC_CLASSIFYLOCAL_TIME, startTime);
+              glean::urlclassifier::async_classifylocal_time
+                  .AccumulateRawDuration(TimeStamp::Now() - startTime);
 
               nsTArray<RefPtr<nsIUrlClassifierFeatureResult>> results;
               holder->GetResults(results);
@@ -2497,13 +2536,66 @@ nsUrlClassifierDBService::AsyncClassifyLocalWithFeatures(
                   const_cast<nsIUrlClassifierFeatureCallback*>(callback.get());
               cb->OnClassifyComplete(results);
             });
-
-        NS_DispatchToMainThread(cbRunnable);
+        if (aIdlePriority) {
+          NS_DispatchToMainThreadQueue(cbRunnable.forget(),
+                                       EventQueuePriority::Idle);
+        } else {
+          NS_DispatchToMainThread(cbRunnable);
+        }
       });
 
+  if (aIdlePriority) {
+    return NS_DispatchToThreadQueue(r.forget(), gDbBackgroundThread,
+                                    EventQueuePriority::Idle);
+  }
   return gDbBackgroundThread->Dispatch(r, NS_DISPATCH_NORMAL);
 }
 
+NS_IMETHODIMP
+nsUrlClassifierDBService::AsyncClassifyLocalWithFeatureNames(
+    nsIURI* aURI, const nsTArray<nsCString>& aFeatureNames,
+    nsIUrlClassifierFeature::listType aListType,
+    nsIUrlClassifierFeatureCallback* aCallback) {
+  MOZ_ASSERT(NS_IsMainThread());
+
+  if (gShuttingDownThread) {
+    return NS_ERROR_ABORT;
+  }
+
+  if (!XRE_IsContentProcess()) {
+    nsTArray<RefPtr<nsIUrlClassifierFeature>> features;
+    for (const nsCString& featureName : aFeatureNames) {
+      nsCOMPtr<nsIUrlClassifierFeature> feature =
+          mozilla::net::UrlClassifierFeatureFactory::GetFeatureByName(
+              featureName);
+      if (NS_WARN_IF(!feature)) {
+        continue;
+      }
+      features.AppendElement(feature);
+    }
+    MOZ_ASSERT(!features.IsEmpty(),
+               "At least one URL classifier feature must be present");
+    return AsyncClassifyLocalWithFeatures(aURI, features, aListType, aCallback,
+                                          true);
+  }
+
+  mozilla::dom::ContentChild* content =
+      mozilla::dom::ContentChild::GetSingleton();
+  if (NS_WARN_IF(!content || content->IsShuttingDown())) {
+    return NS_ERROR_FAILURE;
+  }
+
+  auto actor = new mozilla::dom::URLClassifierLocalByNameChild();
+
+  if (!content->SendPURLClassifierLocalByNameConstructor(
+          actor, aURI, aFeatureNames, aListType)) {
+    return NS_ERROR_FAILURE;
+  }
+
+  actor->SetFeaturesAndCallback(aFeatureNames, aCallback);
+  return NS_OK;
+}
+
 bool nsUrlClassifierDBService::AsyncClassifyLocalWithFeaturesUsingPreferences(
     nsIURI* aURI, const nsTArray<RefPtr<nsIUrlClassifierFeature>>& aFeatures,
     nsIUrlClassifierFeature::listType aListType,
