@@ -14,7 +14,6 @@
 #include "prenv.h"
 
 #include "nsIAppShell.h"
-#include "nsAppStartupNotifier.h"
 #include "nsIToolkitProfile.h"
 
 #ifdef XP_WIN
@@ -26,15 +25,13 @@
 #  endif
 #  include "mozilla/ScopeExit.h"
 #  include "mozilla/WinDllServices.h"
+#  include "mozilla/WindowsBCryptInitialization.h"
 #  include "WinUtils.h"
-#  ifdef ACCESSIBILITY
-#    include "mozilla/GeckoArgs.h"
-#    include "mozilla/mscom/ActCtxResource.h"
-#  endif
 #endif
 
 #include "nsAppRunner.h"
 #include "nsExceptionHandler.h"
+#include "mozilla/RuntimeExceptionModule.h"
 #include "nsThreadUtils.h"
 #include "nsJSUtils.h"
 #include "nsWidgetsCID.h"
@@ -66,12 +63,13 @@
 #include "mozilla/AbstractThread.h"
 #include "mozilla/FilePreferences.h"
 #include "mozilla/IOInterposer.h"
+#include "mozilla/NeverDestroyed.h"
+#include "mozilla/ProcessType.h"
 #include "mozilla/RDDProcessImpl.h"
 #include "mozilla/ipc/UtilityProcessImpl.h"
 #include "mozilla/UniquePtr.h"
 
-#include "mozilla/ipc/BrowserProcessSubThread.h"
-#include "mozilla/ipc/IOThreadChild.h"
+#include "mozilla/ipc/IOThread.h"
 #include "mozilla/ipc/ProcessChild.h"
 
 #include "mozilla/dom/ContentProcess.h"
@@ -92,7 +90,6 @@
 #if defined(MOZ_SANDBOX) && defined(XP_WIN)
 #  include "mozilla/sandboxTarget.h"
 #  include "mozilla/sandboxing/loggingCallbacks.h"
-#  include "mozilla/RemoteSandboxBrokerProcessChild.h"
 #endif
 
 #if defined(MOZ_SANDBOX)
@@ -114,10 +111,6 @@
 #  endif
 #endif
 
-#ifdef MOZ_JPROF
-#  include "jprof.h"
-#endif
-
 #if defined(XP_WIN) && defined(MOZ_SANDBOX)
 #  include "mozilla/sandboxing/SandboxInitialization.h"
 #  include "mozilla/sandboxing/sandboxLogging.h"
@@ -135,96 +128,27 @@
 
 using namespace mozilla;
 
-using mozilla::ipc::BrowserProcessSubThread;
 using mozilla::ipc::GeckoChildProcessHost;
-using mozilla::ipc::IOThreadChild;
+using mozilla::ipc::IOThread;
 using mozilla::ipc::ProcessChild;
-using mozilla::ipc::ScopedXREEmbed;
 
 using mozilla::dom::ContentParent;
 using mozilla::dom::ContentProcess;
+using mozilla::dom::UniqueContentParentKeepAlive;
 
 using mozilla::gmp::GMPProcessChild;
 
 using mozilla::ipc::TestShellCommandParent;
 using mozilla::ipc::TestShellParent;
 
-using mozilla::startup::sChildProcessType;
-
 namespace mozilla::_ipdltest {
 // Set in IPDLUnitTest.cpp when running gtests.
 UniquePtr<mozilla::ipc::ProcessChild> (*gMakeIPDLUnitTestProcessChild)(
-    base::ProcessId) = nullptr;
+    IPC::Channel::ChannelHandle, base::ProcessId, const nsID&) = nullptr;
 }  // namespace mozilla::_ipdltest
 
 static NS_DEFINE_CID(kAppShellCID, NS_APPSHELL_CID);
 
-nsresult XRE_LockProfileDirectory(nsIFile* aDirectory,
-                                  nsISupports** aLockObject) {
-  nsCOMPtr<nsIProfileLock> lock;
-
-  nsresult rv =
-      NS_LockProfilePath(aDirectory, nullptr, nullptr, getter_AddRefs(lock));
-  if (NS_SUCCEEDED(rv)) NS_ADDREF(*aLockObject = lock);
-
-  return rv;
-}
-
-static int32_t sInitCounter;
-
-nsresult XRE_InitEmbedding2(nsIFile* aLibXULDirectory, nsIFile* aAppDirectory,
-                            nsIDirectoryServiceProvider* aAppDirProvider) {
-  // Initialize some globals to make nsXREDirProvider happy
-  static char* kNullCommandLine[] = {nullptr};
-  gArgv = kNullCommandLine;
-  gArgc = 0;
-
-  NS_ENSURE_ARG(aLibXULDirectory);
-
-  if (++sInitCounter > 1)  // XXXbsmedberg is this really the right solution?
-    return NS_OK;
-
-  if (!aAppDirectory) aAppDirectory = aLibXULDirectory;
-
-  nsresult rv;
-
-  new nsXREDirProvider;  // This sets gDirServiceProvider
-  if (!gDirServiceProvider) return NS_ERROR_OUT_OF_MEMORY;
-
-  rv = gDirServiceProvider->Initialize(aAppDirectory, aLibXULDirectory,
-                                       aAppDirProvider);
-  if (NS_FAILED(rv)) return rv;
-
-  rv = NS_InitXPCOM(nullptr, aAppDirectory, gDirServiceProvider);
-  if (NS_FAILED(rv)) return rv;
-
-  // We do not need to autoregister components here. The CheckCompatibility()
-  // bits in nsAppRunner.cpp check for an invalidation flag in
-  // compatibility.ini.
-  // If the app wants to autoregister every time (for instance, if it's debug),
-  // it can do so after we return from this function.
-
-  nsAppStartupNotifier::NotifyObservers(APPSTARTUP_CATEGORY);
-
-  return NS_OK;
-}
-
-void XRE_NotifyProfile() {
-  NS_ASSERTION(gDirServiceProvider, "XRE_InitEmbedding was not called!");
-  gDirServiceProvider->DoStartup();
-}
-
-void XRE_TermEmbedding() {
-  if (--sInitCounter != 0) return;
-
-  NS_ASSERTION(gDirServiceProvider,
-               "XRE_TermEmbedding without XRE_InitEmbedding");
-
-  gDirServiceProvider->DoShutdown();
-  NS_ShutdownXPCOM(nullptr);
-  delete gDirServiceProvider;
-}
-
 const char* XRE_GeckoProcessTypeToString(GeckoProcessType aProcessType) {
   switch (aProcessType) {
 #define GECKO_PROCESS_TYPE(enum_value, enum_name, string_name, proc_typename, \
@@ -252,38 +176,18 @@ const char* XRE_ChildProcessTypeToAnnotation(GeckoProcessType aProcessType) {
   }
 }
 
-namespace mozilla::startup {
-GeckoProcessType sChildProcessType = GeckoProcessType_Default;
-}  // namespace mozilla::startup
-
 #if defined(MOZ_WIDGET_ANDROID)
-void XRE_SetAndroidChildFds(JNIEnv* env, const XRE_AndroidChildFds& fds) {
+void XRE_SetAndroidChildFds(JNIEnv* env, jintArray jfds) {
   mozilla::jni::SetGeckoThreadEnv(env);
-  mozilla::ipc::SetPrefsFd(fds.mPrefsFd);
-  mozilla::ipc::SetPrefMapFd(fds.mPrefMapFd);
-  IPC::Channel::SetClientChannelFd(fds.mIpcFd);
-  CrashReporter::SetNotificationPipeForChild(fds.mCrashFd);
-  CrashReporter::SetCrashAnnotationPipeForChild(fds.mCrashAnnotationFd);
+
+  // Copy passed file handles from the JNI environment into geckoargs.
+  jsize size = env->GetArrayLength(jfds);
+  jint* fds = env->GetIntArrayElements(jfds, nullptr);
+  geckoargs::SetPassedFileHandles(Span(fds, size));
+  env->ReleaseIntArrayElements(jfds, fds, JNI_ABORT);
 }
 #endif  // defined(MOZ_WIDGET_ANDROID)
 
-void XRE_SetProcessType(const char* aProcessTypeString) {
-  static bool called = false;
-  if (called && sChildProcessType != GeckoProcessType_ForkServer) {
-    MOZ_CRASH();
-  }
-  called = true;
-
-  sChildProcessType = GeckoProcessType_Invalid;
-  for (GeckoProcessType t :
-       MakeEnumeratedRange(GeckoProcessType::GeckoProcessType_End)) {
-    if (!strcmp(XRE_GeckoProcessTypeToString(t), aProcessTypeString)) {
-      sChildProcessType = t;
-      return;
-    }
-  }
-}
-
 #if defined(XP_WIN)
 void SetTaskbarGroupId(const nsString& aId) {
   if (FAILED(SetCurrentProcessExplicitAppUserModelID(aId.get()))) {
@@ -296,9 +200,13 @@ void SetTaskbarGroupId(const nsString& aId) {
 #if defined(MOZ_SANDBOX)
 void AddContentSandboxLevelAnnotation() {
   if (XRE_GetProcessType() == GeckoProcessType_Content) {
-    int level = GetEffectiveContentSandboxLevel();
-    CrashReporter::AnnotateCrashReport(
-        CrashReporter::Annotation::ContentSandboxLevel, level);
+    uint32_t contentSandboxLevel = GetEffectiveContentSandboxLevel();
+    CrashReporter::RecordAnnotationU32(
+        CrashReporter::Annotation::ContentSandboxLevel, contentSandboxLevel);
+  } else if (XRE_GetProcessType() == GeckoProcessType_GPU) {
+    uint32_t gpuSandboxLevel = GetEffectiveGpuSandboxLevel();
+    CrashReporter::RecordAnnotationU32(
+        CrashReporter::Annotation::GpuSandboxLevel, gpuSandboxLevel);
   }
 }
 #endif /* MOZ_SANDBOX */
@@ -310,32 +218,21 @@ int GetDebugChildPauseTime() {
   if (pauseStr && *pauseStr) {
     int pause = atoi(pauseStr);
     if (pause != 1) {  // must be !=1 since =1 enables the default pause time
-#if defined(OS_WIN)
+#if defined(XP_WIN)
       pause *= 1000;  // convert to ms
 #endif
       return pause;
     }
   }
-#ifdef OS_POSIX
+#ifdef XP_UNIX
   return 30;  // seconds
-#elif defined(OS_WIN)
+#elif defined(XP_WIN)
   return 10000;  // milliseconds
 #else
   return 0;
 #endif
 }
 
-static bool IsCrashReporterEnabled(const char* aArg) {
-  // on windows and mac, |aArg| is the named pipe on which the server is
-  // listening for requests, or "-" if crash reporting is disabled.
-#if defined(XP_MACOSX) || defined(XP_WIN)
-  return 0 != strcmp("-", aArg);
-#else
-  // on POSIX, |aArg| is "true" if crash reporting is enabled, false otherwise
-  return 0 != strcmp("false", aArg);
-#endif
-}
-
 }  // namespace
 
 nsresult XRE_InitChildProcess(int aArgc, char* aArgv[],
@@ -372,17 +269,13 @@ nsresult XRE_InitChildProcess(int aArgc, char* aArgv[],
 
 #if defined(XP_LINUX) && defined(MOZ_SANDBOX)
   // This has to happen before glib thread pools are started.
-  mozilla::SandboxEarlyInit();
+  mozilla::SandboxEarlyInit(geckoargs::sSandboxReporter.Get(aArgc, aArgv),
+                            geckoargs::sChrootClient.Get(aArgc, aArgv));
   // This just needs to happen before sandboxing, to initialize the
   // cached value, but libmozsandbox can't see this symbol.
   mozilla::GetNumberOfProcessors();
 #endif
 
-#ifdef MOZ_JPROF
-  // Call the code to install our handler
-  setupProfilingStuff();
-#endif
-
 #if defined(XP_WIN)
   // From the --attach-console support in nsNativeAppSupportWin.cpp, but
   // here we are a content child process, so we always attempt to attach
@@ -435,24 +328,13 @@ nsresult XRE_InitChildProcess(int aArgc, char* aArgv[],
 
   const int kTimeoutMs = 1000;
 
-  UniqueMachSendRight task_sender;
-  kern_return_t kr = bootstrap_look_up(bootstrap_port, mach_port_name,
-                                       getter_Transfers(task_sender));
-  if (kr != KERN_SUCCESS) {
-    NS_WARNING(nsPrintfCString("child bootstrap_look_up failed: %s",
-                               mach_error_string(kr))
-                   .get());
+  std::vector<mozilla::UniqueMachSendRight> sendRights;
+  if (NS_WARN_IF(
+          !MachChildProcessCheckIn(mach_port_name, kTimeoutMs, sendRights))) {
     return NS_ERROR_FAILURE;
   }
 
-  kr = MachSendPortSendRight(task_sender.get(), mach_task_self(),
-                             Some(kTimeoutMs));
-  if (kr != KERN_SUCCESS) {
-    NS_WARNING(nsPrintfCString("child MachSendPortSendRight failed: %s",
-                               mach_error_string(kr))
-                   .get());
-    return NS_ERROR_FAILURE;
-  }
+  geckoargs::SetPassedMachSendRights(std::move(sendRights));
 
 #  if defined(MOZ_SANDBOX)
   std::string sandboxError;
@@ -468,38 +350,27 @@ nsresult XRE_InitChildProcess(int aArgc, char* aArgv[],
 
   bool exceptionHandlerIsSet = false;
   if (!CrashReporter::IsDummy()) {
-    CrashReporter::FileHandle crashTimeAnnotationFile =
-        CrashReporter::kInvalidFileHandle;
-#if defined(XP_WIN)
-    if (aArgc < 1) {
-      return NS_ERROR_FAILURE;
-    }
-    // Pop the first argument, this is used by the WER runtime exception module
-    // which reads it from the command-line so we can just discard it here.
-    --aArgc;
-
-    const char* const crashTimeAnnotationArg = aArgv[--aArgc];
-    crashTimeAnnotationFile = reinterpret_cast<CrashReporter::FileHandle>(
-        std::stoul(std::string(crashTimeAnnotationArg)));
-#endif
-
-    if (aArgc < 1) return NS_ERROR_FAILURE;
-    const char* const crashReporterArg = aArgv[--aArgc];
-
-    if (IsCrashReporterEnabled(crashReporterArg)) {
+    auto crashReporterArg = geckoargs::sCrashReporter.Get(aArgc, aArgv);
+    auto crashHelperArg = geckoargs::sCrashHelper.Get(aArgc, aArgv);
+    if (crashReporterArg && crashHelperArg) {
       exceptionHandlerIsSet = CrashReporter::SetRemoteExceptionHandler(
-          crashReporterArg, crashTimeAnnotationFile);
+          std::move(*crashReporterArg), std::move(*crashHelperArg));
+      MOZ_ASSERT(exceptionHandlerIsSet,
+                 "Should have been able to set remote exception handler");
 
       if (!exceptionHandlerIsSet) {
         // Bug 684322 will add better visibility into this condition
         NS_WARNING("Could not setup crash reporting\n");
       }
+    } else {
+      // We might have registered a runtime exception module very early in
+      // process startup to catch early crashes. This is before we process the
+      // crash reporter arg, so unregister here if it turns out the crash
+      // reporter is disabled.
+      CrashReporter::UnregisterRuntimeExceptionModule();
     }
   }
 
-  gArgv = aArgv;
-  gArgc = aArgc;
-
 #ifdef MOZ_X11
   XInitThreads();
 #endif
@@ -508,7 +379,7 @@ nsresult XRE_InitChildProcess(int aArgc, char* aArgv[],
   g_set_prgname(aArgv[0]);
 #endif
 
-#ifdef OS_POSIX
+#ifdef XP_UNIX
   if (PR_GetEnv("MOZ_DEBUG_CHILD_PROCESS") ||
       PR_GetEnv("MOZ_DEBUG_CHILD_PAUSE")) {
 #  if defined(XP_LINUX) && defined(DEBUG)
@@ -521,7 +392,7 @@ nsresult XRE_InitChildProcess(int aArgc, char* aArgv[],
         XRE_GetProcessTypeString(), base::GetCurrentProcId());
     sleep(GetDebugChildPauseTime());
   }
-#elif defined(OS_WIN)
+#elif defined(XP_WIN)
   if (PR_GetEnv("MOZ_DEBUG_CHILD_PROCESS")) {
     NS_DebugBreak(NS_DEBUG_BREAK,
                   "Invoking NS_DebugBreak() to debug child process", nullptr,
@@ -541,16 +412,6 @@ nsresult XRE_InitChildProcess(int aArgc, char* aArgv[],
   mozilla::startup::IncreaseDescriptorLimits();
 #endif
 
-  // child processes launched by GeckoChildProcessHost get this magic
-  // argument appended to their command lines
-  const char* const parentPIDString = aArgv[aArgc - 1];
-  MOZ_ASSERT(parentPIDString, "NULL parent PID");
-  --aArgc;
-
-  char* end = 0;
-  base::ProcessId parentPID = strtol(parentPIDString, &end, 10);
-  MOZ_ASSERT(!*end, "invalid parent PID");
-
 #if defined(XP_WIN)
   // On Win7+, when not running as an MSIX package, register the application
   // user model id passed in by parent. This ensures windows created by the
@@ -571,6 +432,30 @@ nsresult XRE_InitChildProcess(int aArgc, char* aArgv[],
   }
 #endif
 
+  Maybe<base::ProcessId> parentPID = geckoargs::sParentPid.Get(aArgc, aArgv);
+  Maybe<const char*> initialChannelIdString =
+      geckoargs::sInitialChannelID.Get(aArgc, aArgv);
+  if (NS_WARN_IF(!parentPID || !initialChannelIdString)) {
+    return NS_ERROR_FAILURE;
+  }
+
+  Maybe<IPC::Channel::ChannelHandle> clientChannel =
+      geckoargs::sIPCHandle.Get(aArgc, aArgv);
+#ifdef XP_DARWIN
+  MOZ_ASSERT_IF(clientChannel, !geckoargs::sIPCPort.IsPresent(aArgc, aArgv));
+  if (!clientChannel) {
+    clientChannel = geckoargs::sIPCPort.Get(aArgc, aArgv);
+  }
+#endif
+  if (NS_WARN_IF(!clientChannel)) {
+    return NS_ERROR_FAILURE;
+  }
+
+  nsID messageChannelId{};
+  if (NS_WARN_IF(!messageChannelId.Parse(*initialChannelIdString))) {
+    return NS_ERROR_FAILURE;
+  }
+
   base::AtExitManager exitManager;
 
   nsresult rv = XRE_InitCommandLine(aArgc, aArgv);
@@ -591,32 +476,22 @@ nsresult XRE_InitChildProcess(int aArgc, char* aArgv[],
       uiLoopType = MessageLoop::TYPE_MOZILLA_CHILD;
       break;
     case GeckoProcessType_GMPlugin:
-    case GeckoProcessType_RemoteSandboxBroker:
-      uiLoopType = MessageLoop::TYPE_DEFAULT;
+      gmp::GMPProcessChild::InitStatics(aArgc, aArgv);
+      uiLoopType = gmp::GMPProcessChild::UseXPCOM()
+                       ? MessageLoop::TYPE_MOZILLA_CHILD
+                       : MessageLoop::TYPE_DEFAULT;
       break;
     default:
       uiLoopType = MessageLoop::TYPE_UI;
       break;
   }
 
-#if defined(MOZ_SANDBOX) && defined(XP_WIN)
-  if (aChildData->sandboxBrokerServices) {
-    SandboxBroker::Initialize(aChildData->sandboxBrokerServices);
-    SandboxBroker::GeckoDependentInitialize();
+#if defined(XP_WIN)
+  {
+    DebugOnly<bool> result = mozilla::WindowsBCryptInitialization();
+    MOZ_ASSERT(result);
   }
-
-  // Call BCryptGenRandom() to pre-load bcryptPrimitives.dll while the current
-  // thread still has an unrestricted impersonation token. We need to perform
-  // that operation to warmup the BCryptGenRandom() call that is used by
-  // others, especially rust.  See bug 1746524, bug 1751094, bug 1751177
-  UCHAR buffer[32];
-  NTSTATUS status = BCryptGenRandom(NULL,            // hAlgorithm
-                                    buffer,          // pbBuffer
-                                    sizeof(buffer),  // cbBuffer
-                                    BCRYPT_USE_SYSTEM_PREFERRED_RNG  // dwFlags
-  );
-  MOZ_RELEASE_ASSERT(status == STATUS_SUCCESS);
-#endif  // defined(MOZ_SANDBOX) && defined(XP_WIN)
+#endif  // defined(XP_WIN)
 
   {
     // This is a lexical scope for the MessageLoop below.  We want it
@@ -624,21 +499,11 @@ nsresult XRE_InitChildProcess(int aArgc, char* aArgv[],
     // spurious warnings about XPCOM objects being destroyed from a
     // static context.
 
-    Maybe<IOInterposerInit> ioInterposerGuard;
+    AutoIOInterposer ioInterposerGuard;
 
     // Associate this thread with a UI MessageLoop
     MessageLoop uiMessageLoop(uiLoopType);
     {
-#if defined(XP_WIN) && defined(ACCESSIBILITY)
-      // The accessibility resource ID is passed down on the command line
-      // because its retrieval causes issues with the sandbox. When it is set,
-      // it is required for ProcessRuntime construction within ProcessChild.
-      auto a11yResourceId = geckoargs::sA11yResourceId.Get(aArgc, aArgv);
-      if (a11yResourceId.isSome()) {
-        mscom::ActCtxResource::SetAccessibilityResourceId(*a11yResourceId);
-      }
-#endif
-
       UniquePtr<ProcessChild> process;
       switch (XRE_GetProcessType()) {
         case GeckoProcessType_Default:
@@ -646,48 +511,49 @@ nsresult XRE_InitChildProcess(int aArgc, char* aArgv[],
           break;
 
         case GeckoProcessType_Content:
-          ioInterposerGuard.emplace();
-          process = MakeUnique<ContentProcess>(parentPID);
+          ioInterposerGuard.Init();
+          process = MakeUnique<ContentProcess>(std::move(*clientChannel),
+                                               *parentPID, messageChannelId);
           break;
 
         case GeckoProcessType_IPDLUnitTest:
           MOZ_RELEASE_ASSERT(mozilla::_ipdltest::gMakeIPDLUnitTestProcessChild,
                              "xul-gtest not loaded!");
-          process =
-              mozilla::_ipdltest::gMakeIPDLUnitTestProcessChild(parentPID);
+          process = mozilla::_ipdltest::gMakeIPDLUnitTestProcessChild(
+              std::move(*clientChannel), *parentPID, messageChannelId);
           break;
 
         case GeckoProcessType_GMPlugin:
-          process = MakeUnique<gmp::GMPProcessChild>(parentPID);
+          process = MakeUnique<gmp::GMPProcessChild>(
+              std::move(*clientChannel), *parentPID, messageChannelId);
           break;
 
         case GeckoProcessType_GPU:
-          process = MakeUnique<gfx::GPUProcessImpl>(parentPID);
+          process = MakeUnique<gfx::GPUProcessImpl>(
+              std::move(*clientChannel), *parentPID, messageChannelId);
           break;
 
         case GeckoProcessType_VR:
-          process = MakeUnique<gfx::VRProcessChild>(parentPID);
+          process = MakeUnique<gfx::VRProcessChild>(
+              std::move(*clientChannel), *parentPID, messageChannelId);
           break;
 
         case GeckoProcessType_RDD:
-          process = MakeUnique<RDDProcessImpl>(parentPID);
+          process = MakeUnique<RDDProcessImpl>(std::move(*clientChannel),
+                                               *parentPID, messageChannelId);
           break;
 
         case GeckoProcessType_Socket:
-          ioInterposerGuard.emplace();
-          process = MakeUnique<net::SocketProcessImpl>(parentPID);
+          ioInterposerGuard.Init();
+          process = MakeUnique<net::SocketProcessImpl>(
+              std::move(*clientChannel), *parentPID, messageChannelId);
           break;
 
         case GeckoProcessType_Utility:
-          process = MakeUnique<ipc::UtilityProcessImpl>(parentPID);
+          process = MakeUnique<ipc::UtilityProcessImpl>(
+              std::move(*clientChannel), *parentPID, messageChannelId);
           break;
 
-#if defined(MOZ_SANDBOX) && defined(XP_WIN)
-        case GeckoProcessType_RemoteSandboxBroker:
-          process = MakeUnique<RemoteSandboxBrokerProcessChild>(parentPID);
-          break;
-#endif
-
 #if defined(MOZ_ENABLE_FORKSERVER)
         case GeckoProcessType_ForkServer:
           MOZ_CRASH("Fork server should not go here");
@@ -718,13 +584,8 @@ nsresult XRE_InitChildProcess(int aArgc, char* aArgv[],
       mozilla::sandboxing::InitLoggingIfRequired(
           aChildData->ProvideLogFunction);
 #endif
-      if (XRE_GetProcessType() != GeckoProcessType_RemoteSandboxBroker) {
-        // Remote sandbox launcher process doesn't have prerequisites for
-        // these...
-        mozilla::FilePreferences::InitDirectoriesAllowlist();
-        mozilla::FilePreferences::InitPrefs();
-        OverrideDefaultLocaleIfNeeded();
-      }
+      mozilla::FilePreferences::InitDirectoriesAllowlist();
+      mozilla::FilePreferences::InitPrefs();
 
 #if defined(MOZ_SANDBOX)
       AddContentSandboxLevelAnnotation();
@@ -740,92 +601,13 @@ nsresult XRE_InitChildProcess(int aArgc, char* aArgv[],
     }
   }
 
-  if (exceptionHandlerIsSet) {
-    CrashReporter::UnsetRemoteExceptionHandler();
-  }
+  CrashReporter::UnsetRemoteExceptionHandler(exceptionHandlerIsSet);
 
   return XRE_DeinitCommandLine();
 }
 
-MessageLoop* XRE_GetIOMessageLoop() {
-  if (sChildProcessType == GeckoProcessType_Default) {
-    return BrowserProcessSubThread::GetMessageLoop(BrowserProcessSubThread::IO);
-  }
-  return IOThreadChild::message_loop();
-}
-
-namespace {
-
-class MainFunctionRunnable : public Runnable {
- public:
-  NS_DECL_NSIRUNNABLE
-
-  MainFunctionRunnable(MainFunction aFunction, void* aData)
-      : mozilla::Runnable("MainFunctionRunnable"),
-        mFunction(aFunction),
-        mData(aData) {
-    NS_ASSERTION(aFunction, "Don't give me a null pointer!");
-  }
-
- private:
-  MainFunction mFunction;
-  void* mData;
-};
-
-} /* anonymous namespace */
-
-NS_IMETHODIMP
-MainFunctionRunnable::Run() {
-  mFunction(mData);
-  return NS_OK;
-}
-
-nsresult XRE_InitParentProcess(int aArgc, char* aArgv[],
-                               MainFunction aMainFunction,
-                               void* aMainFunctionData) {
-  NS_ENSURE_ARG_MIN(aArgc, 1);
-  NS_ENSURE_ARG_POINTER(aArgv);
-  NS_ENSURE_ARG_POINTER(aArgv[0]);
-
-  // Set main thread before we initialize the profiler
-  NS_SetMainThread();
-
-  mozilla::LogModule::Init(aArgc, aArgv);
-
-  AUTO_BASE_PROFILER_LABEL("XRE_InitParentProcess (around Gecko Profiler)",
-                           OTHER);
-  AUTO_PROFILER_INIT;
-  AUTO_PROFILER_LABEL("XRE_InitParentProcess", OTHER);
-
-  ScopedXREEmbed embed;
-
-  gArgc = aArgc;
-  gArgv = aArgv;
-  nsresult rv = XRE_InitCommandLine(gArgc, gArgv);
-  if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
-
-  {
-    embed.Start();
-
-    nsCOMPtr<nsIAppShell> appShell(do_GetService(kAppShellCID));
-    NS_ENSURE_TRUE(appShell, NS_ERROR_FAILURE);
-
-    if (aMainFunction) {
-      nsCOMPtr<nsIRunnable> runnable =
-          new MainFunctionRunnable(aMainFunction, aMainFunctionData);
-
-      nsresult rv = NS_DispatchToCurrentThread(runnable);
-      NS_ENSURE_SUCCESS(rv, rv);
-    }
-
-    // Do event loop
-    if (NS_FAILED(appShell->Run())) {
-      NS_WARNING("Failed to run appshell");
-      return NS_ERROR_FAILURE;
-    }
-  }
-
-  return XRE_DeinitCommandLine();
+nsISerialEventTarget* XRE_GetAsyncIOEventTarget() {
+  return IOThread::Get()->GetEventTarget();
 }
 
 nsresult XRE_RunAppShell() {
@@ -878,8 +660,9 @@ nsresult XRE_RunAppShell() {
 void XRE_ShutdownChildProcess() {
   MOZ_ASSERT(NS_IsMainThread(), "Wrong thread!");
 
-  mozilla::DebugOnly<MessageLoop*> ioLoop = XRE_GetIOMessageLoop();
-  MOZ_ASSERT(!!ioLoop, "Bad shutdown order");
+  mozilla::DebugOnly<nsISerialEventTarget*> ioTarget =
+      XRE_GetAsyncIOEventTarget();
+  MOZ_ASSERT(!!ioTarget, "Bad shutdown order");
 
   // Quit() sets off the following chain of events
   //  (1) UI loop starts quitting
@@ -902,22 +685,25 @@ void XRE_ShutdownChildProcess() {
 }
 
 namespace {
-ContentParent* gContentParent;  // long-lived, manually refcounted
+UniqueContentParentKeepAlive& TestShellContentParent() {
+  static NeverDestroyed<UniqueContentParentKeepAlive> sContentParent;
+  return *sContentParent;
+}
+
 TestShellParent* GetOrCreateTestShellParent() {
-  if (!gContentParent) {
+  if (!TestShellContentParent()) {
     // Use a "web" child process by default.  File a bug if you don't like
     // this and you're sure you wouldn't be better off writing a "browser"
     // chrome mochitest where you can have multiple types of content
     // processes.
-    RefPtr<ContentParent> parent =
+    TestShellContentParent() =
         ContentParent::GetNewOrUsedBrowserProcess(DEFAULT_REMOTE_TYPE);
-    parent.forget(&gContentParent);
-  } else if (!gContentParent->IsAlive()) {
+  } else if (TestShellContentParent()->IsShuttingDown()) {
     return nullptr;
   }
-  TestShellParent* tsp = gContentParent->GetTestShellSingleton();
+  TestShellParent* tsp = TestShellContentParent()->GetTestShellSingleton();
   if (!tsp) {
-    tsp = gContentParent->CreateTestShell();
+    tsp = TestShellContentParent()->CreateTestShell();
   }
   return tsp;
 }
@@ -926,7 +712,7 @@ TestShellParent* GetOrCreateTestShellParent() {
 
 bool XRE_SendTestShellCommand(JSContext* aCx, JSString* aCommand,
                               JS::Value* aCallback) {
-  JS::RootedString cmd(aCx, aCommand);
+  JS::Rooted<JSString*> cmd(aCx, aCommand);
   TestShellParent* tsp = GetOrCreateTestShellParent();
   NS_ENSURE_TRUE(tsp, false);
 
@@ -947,15 +733,15 @@ bool XRE_SendTestShellCommand(JSContext* aCx, JSString* aCommand,
 }
 
 bool XRE_ShutdownTestShell() {
-  if (!gContentParent) {
+  if (!TestShellContentParent()) {
     return true;
   }
   bool ret = true;
-  if (gContentParent->IsAlive()) {
-    ret = gContentParent->DestroyTestShell(
-        gContentParent->GetTestShellSingleton());
+  if (TestShellContentParent()->IsAlive()) {
+    ret = TestShellContentParent()->DestroyTestShell(
+        TestShellContentParent()->GetTestShellSingleton());
   }
-  NS_RELEASE(gContentParent);
+  TestShellContentParent().reset();
   return ret;
 }
 
