@@ -1,20 +1,25 @@
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-from __future__ import absolute_import
-
-import os
+import pathlib
 import re
 import shutil
 import tempfile
 
 from perfdocs.logger import PerfDocLogger
-from perfdocs.utils import are_dirs_equal, read_file, read_yaml, save_file
+from perfdocs.utils import (
+    ON_TRY,
+    are_dirs_equal,
+    get_changed_files,
+    read_file,
+    read_yaml,
+    save_file,
+)
 
 logger = PerfDocLogger()
 
 
-class Generator(object):
+class Generator:
     """
     After each perfdocs directory was validated, the generator uses the templates
     for each framework, fills them with the test descriptions in config and saves
@@ -35,10 +40,10 @@ class Generator(object):
         if not self._workspace:
             raise Exception("PerfDocs Generator requires a workspace directory.")
         # Template documents without added information reside here
-        self.templates_path = os.path.join(
+        self.templates_path = pathlib.Path(
             self._workspace, "tools", "lint", "perfdocs", "templates"
         )
-        self.perfdocs_path = os.path.join(
+        self.perfdocs_path = pathlib.Path(
             self._workspace, "testing", "perfdocs", "generated"
         )
 
@@ -61,10 +66,21 @@ class Generator(object):
         # Using the verified `perfdocs_tree`, build up the documentation.
         frameworks_info = {}
         for framework in self._perfdocs_tree:
-            yaml_content = read_yaml(os.path.join(framework["path"], framework["yml"]))
+            yaml_content = read_yaml(pathlib.Path(framework["path"], framework["yml"]))
             rst_content = read_file(
-                os.path.join(framework["path"], framework["rst"]), stringify=True
+                pathlib.Path(framework["path"], framework["rst"]), stringify=True
             )
+            gatherer = self._verifier._gatherer.framework_gatherers[
+                yaml_content["name"]
+            ]
+
+            metrics_rst_content = None
+            metrics_info = self._verifier.metrics_info[yaml_content["name"]]
+            if framework.get("metrics"):
+                metrics_rst_content = read_file(
+                    pathlib.Path(framework["path"], framework["metrics"]),
+                    stringify=True,
+                )
 
             # Gather all tests and descriptions and format them into
             # documentation content
@@ -82,34 +98,63 @@ class Generator(object):
 
                 tests = suite_info.get("tests", {})
                 for test_name in sorted(tests.keys()):
-                    gatherer = self._verifier._gatherer.framework_gatherers[
-                        yaml_content["name"]
-                    ]
                     test_description = gatherer.build_test_description(
-                        test_name, tests[test_name], suite_name
+                        test_name,
+                        test_description=tests[test_name],
+                        suite_name=suite_name,
+                        metrics_info=metrics_info,
                     )
                     documentation.extend(test_description)
                 documentation.append("")
 
+            # For each framework, we may want to organize the metrics differently
+            # so delegate the complete setup of the metric documentation to the
+            # framework-specific gatherers
+            metrics_rst = ""
+            if metrics_rst_content:
+                metrics_documentation = gatherer.build_metrics_documentation(
+                    metrics_info
+                )
+                metrics_rst = re.sub(
+                    r"{metrics_documentation}",
+                    "\n".join(metrics_documentation),
+                    metrics_rst_content,
+                )
+                rst_content = re.sub(
+                    r"{metrics_rst_name}",
+                    f"{yaml_content['name']}-metrics",
+                    rst_content,
+                )
+
             # Insert documentation into `.rst` file
             framework_rst = re.sub(
                 r"{documentation}", "\n".join(documentation), rst_content
             )
             frameworks_info[yaml_content["name"]] = {
                 "dynamic": framework_rst,
+                "metrics": metrics_rst,
                 "static": [],
             }
 
             # For static `.rst` file
             for static_file in framework["static"]:
-                frameworks_info[yaml_content["name"]]["static"].append(
-                    {
-                        "file": static_file,
-                        "content": read_file(
-                            os.path.join(framework["path"], static_file), stringify=True
-                        ),
-                    }
-                )
+                if static_file.endswith("rst"):
+                    frameworks_info[yaml_content["name"]]["static"].append(
+                        {
+                            "file": static_file,
+                            "content": read_file(
+                                pathlib.Path(framework["path"], static_file),
+                                stringify=True,
+                            ),
+                        }
+                    )
+                else:
+                    frameworks_info[yaml_content["name"]]["static"].append(
+                        {
+                            "file": static_file,
+                            "content": pathlib.Path(framework["path"], static_file),
+                        }
+                    )
 
         return frameworks_info
 
@@ -121,17 +166,16 @@ class Generator(object):
         # Build the directory that will contain the final result (a tmp dir
         # that will be moved to the final location afterwards)
         try:
-            tmpdir = tempfile.mkdtemp()
-            perfdocs_tmpdir = os.path.join(tmpdir, "generated")
-            os.mkdir(perfdocs_tmpdir)
+            tmpdir = pathlib.Path(tempfile.mkdtemp())
+            perfdocs_tmpdir = pathlib.Path(tmpdir, "generated")
+            perfdocs_tmpdir.mkdir(parents=True, exist_ok=True)
+            perfdocs_tmpdir.chmod(0o766)
         except OSError as e:
-            logger.critical("Error creating temp file: {}".format(e))
+            logger.critical(f"Error creating temp file: {e}")
 
-        success = False or os.path.isdir(perfdocs_tmpdir)
-        if success:
+        if perfdocs_tmpdir.is_dir():
             return perfdocs_tmpdir
-        else:
-            return success
+        return False
 
     def _create_perfdocs(self):
         """
@@ -145,22 +189,37 @@ class Generator(object):
 
         # Save the documentation files
         frameworks = []
-        for framework_name in sorted(framework_docs.keys()):
+        for framework_name in sorted(framework_docs.keys(), key=str.lower):
             frameworks.append(framework_name)
             save_file(
                 framework_docs[framework_name]["dynamic"],
-                os.path.join(perfdocs_tmpdir, framework_name),
+                pathlib.Path(perfdocs_tmpdir, framework_name),
             )
 
-            for static_name in framework_docs[framework_name]["static"]:
+            if framework_docs[framework_name]["metrics"]:
                 save_file(
-                    static_name["content"],
-                    os.path.join(perfdocs_tmpdir, static_name["file"].split(".")[0]),
+                    framework_docs[framework_name]["metrics"],
+                    pathlib.Path(perfdocs_tmpdir, f"{framework_name}-metrics"),
                 )
 
+            for static_name in framework_docs[framework_name]["static"]:
+                if static_name["file"].endswith(".rst"):
+                    # XXX Replace this with a shutil.copy call (like below)
+                    save_file(
+                        static_name["content"],
+                        pathlib.Path(
+                            perfdocs_tmpdir, static_name["file"].split(".")[0]
+                        ),
+                    )
+                else:
+                    shutil.copy(
+                        static_name["content"],
+                        pathlib.Path(perfdocs_tmpdir, static_name["file"]),
+                    )
+
         # Get the main page and add the framework links to it
         mainpage = read_file(
-            os.path.join(self.templates_path, "index.rst"), stringify=True
+            pathlib.Path(self.templates_path, "index.rst"), stringify=True
         )
 
         fmt_frameworks = "\n".join(["  * :doc:`%s`" % name for name in frameworks])
@@ -169,7 +228,7 @@ class Generator(object):
         fmt_mainpage = re.sub(r"{toctree_documentation}", fmt_toctree, mainpage)
         fmt_mainpage = re.sub(r"{test_documentation}", fmt_frameworks, fmt_mainpage)
 
-        save_file(fmt_mainpage, os.path.join(perfdocs_tmpdir, "index"))
+        save_file(fmt_mainpage, pathlib.Path(perfdocs_tmpdir, "index"))
 
         return perfdocs_tmpdir
 
@@ -183,21 +242,19 @@ class Generator(object):
         # checking if they need to be regenerated.
         logger.log("Regenerating perfdocs...")
 
-        if os.path.exists(self.perfdocs_path):
-            shutil.rmtree(self.perfdocs_path)
+        if self.perfdocs_path.exists():
+            shutil.rmtree(str(self.perfdocs_path))
 
         try:
-            saved = shutil.copytree(perfdocs_tmpdir, self.perfdocs_path)
+            saved = shutil.copytree(str(perfdocs_tmpdir), str(self.perfdocs_path))
             if saved:
                 logger.log(
                     "Documentation saved to {}/".format(
-                        re.sub(".*testing", "testing", self.perfdocs_path)
+                        re.sub(".*testing", "testing", str(self.perfdocs_path))
                     )
                 )
         except Exception as e:
-            logger.critical(
-                "There was an error while saving the documentation: {}".format(e)
-            )
+            logger.critical(f"There was an error while saving the documentation: {e}")
 
     def generate_perfdocs(self):
         """
@@ -226,14 +283,14 @@ class Generator(object):
             for entry in self._perfdocs_tree:
                 files.extend(
                     [
-                        os.path.join(entry["path"], entry["yml"]),
-                        os.path.join(entry["path"], entry["rst"]),
+                        pathlib.Path(entry["path"], entry["yml"]),
+                        pathlib.Path(entry["path"], entry["rst"]),
                     ]
                 )
             return files
 
         # Throw a warning if there's no need for generating
-        if not os.path.exists(self.perfdocs_path) and not self._generate:
+        if not self.perfdocs_path.exists() and not self._generate:
             # If they don't exist and we are not generating, then throw
             # a linting error and exit.
             logger.warning(
@@ -244,11 +301,15 @@ class Generator(object):
         perfdocs_tmpdir = self._create_perfdocs()
         if self._generate:
             self._save_perfdocs(perfdocs_tmpdir)
-        else:
+        elif not are_dirs_equal(perfdocs_tmpdir, self.perfdocs_path):
             # If we are not generating, then at least check if they
             # should be regenerated by comparing the directories.
-            if not are_dirs_equal(perfdocs_tmpdir, self.perfdocs_path):
-                logger.warning(
-                    "PerfDocs are outdated, run ./mach lint -l perfdocs --fix .` to update them.",
-                    files=get_possibly_changed_files(),
-                )
+            logger.warning(
+                "PerfDocs are outdated, run `./mach lint -l perfdocs --outgoing --fix` "
+                + "to update them. You can also apply the "
+                + f"{'perfdocs.diff' if ON_TRY else 'diff.txt'} patch file "
+                + f"{'produced from this reviewbot test ' if ON_TRY else ''}"
+                + "to fix the issue.",
+                files=get_changed_files(self._workspace),
+                restricted=False,
+            )
