@@ -7,94 +7,37 @@
 #ifndef ProfileBufferEntry_h
 #define ProfileBufferEntry_h
 
-#include "ProfileJSONWriter.h"
-
+#include <cstdint>
+#include <cstdlib>
+#include <functional>
+#include <utility>
+#include <type_traits>
 #include "gtest/MozGtestFriend.h"
 #include "js/ProfilingCategory.h"
-#include "js/ProfilingFrameIterator.h"
+#include "mozilla/Attributes.h"
 #include "mozilla/HashFunctions.h"
 #include "mozilla/HashTable.h"
 #include "mozilla/Maybe.h"
-#include "mozilla/UniquePtr.h"
+#include "mozilla/ProfileBufferEntryKinds.h"
+#include "mozilla/ProfileJSONWriter.h"
+#include "mozilla/ProfilerUtils.h"
+#include "mozilla/UniquePtrExtensions.h"
 #include "mozilla/Variant.h"
 #include "mozilla/Vector.h"
 #include "nsString.h"
 
 class ProfilerCodeAddressService;
-
-// NOTE!  If you add entries, you need to verify if they need to be added to the
-// switch statement in DuplicateLastSample!
-// This will evaluate the MACRO with (KIND, TYPE, SIZE)
-#define FOR_EACH_PROFILE_BUFFER_ENTRY_KIND(MACRO)                    \
-  MACRO(CategoryPair, int, sizeof(int))                              \
-  MACRO(CollectionStart, double, sizeof(double))                     \
-  MACRO(CollectionEnd, double, sizeof(double))                       \
-  MACRO(Label, const char*, sizeof(const char*))                     \
-  MACRO(FrameFlags, uint64_t, sizeof(uint64_t))                      \
-  MACRO(DynamicStringFragment, char*, ProfileBufferEntry::kNumChars) \
-  MACRO(JitReturnAddr, void*, sizeof(void*))                         \
-  MACRO(InnerWindowID, uint64_t, sizeof(uint64_t))                   \
-  MACRO(LineNumber, int, sizeof(int))                                \
-  MACRO(ColumnNumber, int, sizeof(int))                              \
-  MACRO(NativeLeafAddr, void*, sizeof(void*))                        \
-  MACRO(Pause, double, sizeof(double))                               \
-  MACRO(Resume, double, sizeof(double))                              \
-  MACRO(ThreadId, int, sizeof(int))                                  \
-  MACRO(Time, double, sizeof(double))                                \
-  MACRO(TimeBeforeCompactStack, double, sizeof(double))              \
-  MACRO(CounterId, void*, sizeof(void*))                             \
-  MACRO(CounterKey, uint64_t, sizeof(uint64_t))                      \
-  MACRO(Number, uint64_t, sizeof(uint64_t))                          \
-  MACRO(Count, int64_t, sizeof(int64_t))                             \
-  MACRO(ProfilerOverheadTime, double, sizeof(double))                \
-  MACRO(ProfilerOverheadDuration, double, sizeof(double))
+struct JSContext;
 
 class ProfileBufferEntry {
  public:
-  // The `Kind` is a single byte identifying the type of data that is actually
-  // stored in a `ProfileBufferEntry`, as per the list in
-  // `FOR_EACH_PROFILE_BUFFER_ENTRY_KIND`.
-  //
-  // This byte is also used to identify entries in ProfileChunkedBuffer blocks,
-  // for both "legacy" entries that do contain a `ProfileBufferEntry`, and for
-  // new types of entries that may carry more data of different types.
-  // TODO: Eventually each type of "legacy" entry should be replaced with newer,
-  // more efficient kinds of entries (e.g., stack frames could be stored in one
-  // bigger entry, instead of multiple `ProfileBufferEntry`s); then we could
-  // discard `ProfileBufferEntry` and move this enum to a more appropriate spot.
-  using KindUnderlyingType = uint8_t;
-  enum class Kind : KindUnderlyingType {
-    INVALID = 0,
-#define KIND(KIND, TYPE, SIZE) KIND,
-    FOR_EACH_PROFILE_BUFFER_ENTRY_KIND(KIND)
-#undef KIND
-
-    // Any value under `LEGACY_LIMIT` represents a `ProfileBufferEntry`.
-    LEGACY_LIMIT,
-
-    // Any value starting here does *not* represent a `ProfileBufferEntry` and
-    // requires separate decoding and handling.
-
-    // Marker data, including payload.
-    MarkerData = LEGACY_LIMIT,
-
-    // Optional between TimeBeforeCompactStack and CompactStack.
-    UnresponsiveDurationMs,
-
-    // Collection of legacy stack entries, must follow a ThreadId and
-    // TimeBeforeCompactStack (which are not included in the CompactStack;
-    // TimeBeforeCompactStack is equivalent to Time, but indicates that a
-    // CompactStack follows shortly afterwards).
-    CompactStack,
-
-    MODERN_LIMIT
-  };
+  using KindUnderlyingType =
+      std::underlying_type_t<::mozilla::ProfileBufferEntryKind>;
+  using Kind = mozilla::ProfileBufferEntryKind;
 
   ProfileBufferEntry();
 
-  // This is equal to sizeof(double), which is the largest non-char variant in
-  // |u|.
-  static const size_t kNumChars = 8;
+  static constexpr size_t kNumChars = mozilla::ProfileBufferEntryNumChars;
 
  private:
   // aString must be a static string.
@@ -105,6 +48,7 @@ class ProfileBufferEntry {
   ProfileBufferEntry(Kind aKind, int64_t aInt64);
   ProfileBufferEntry(Kind aKind, uint64_t aUint64);
   ProfileBufferEntry(Kind aKind, int aInt);
+  ProfileBufferEntry(Kind aKind, ProfilerThreadId aThreadId);
 
  public:
 #define CTOR(KIND, TYPE, SIZE)                   \
@@ -138,37 +82,13 @@ class ProfileBufferEntry {
   int GetInt() const;
   int64_t GetInt64() const;
   uint64_t GetUint64() const;
+  ProfilerThreadId GetThreadId() const;
   void CopyCharsInto(char (&aOutArray)[kNumChars]) const;
 };
 
 // Packed layout: 1 byte for the tag + 8 bytes for the value.
 static_assert(sizeof(ProfileBufferEntry) == 9, "bad ProfileBufferEntry size");
 
-class UniqueJSONStrings {
- public:
-  UniqueJSONStrings();
-  explicit UniqueJSONStrings(const UniqueJSONStrings& aOther);
-
-  void SpliceStringTableElements(SpliceableJSONWriter& aWriter) {
-    aWriter.TakeAndSplice(mStringTableWriter.WriteFunc());
-  }
-
-  void WriteProperty(mozilla::JSONWriter& aWriter, const char* aName,
-                     const char* aStr) {
-    aWriter.IntProperty(aName, GetOrAddIndex(aStr));
-  }
-
-  void WriteElement(mozilla::JSONWriter& aWriter, const char* aStr) {
-    aWriter.IntElement(GetOrAddIndex(aStr));
-  }
-
-  uint32_t GetOrAddIndex(const char* aStr);
-
- private:
-  SpliceableChunkedJSONWriter mStringTableWriter;
-  mozilla::HashMap<mozilla::HashNumber, uint32_t> mStringHashToIndexMap;
-};
-
 // Contains all the information about JIT frames that is needed to stream stack
 // frames for JitReturnAddr entries in the profiler buffer.
 // Every return address (void*) is mapped to one or more JITFrameKeys, and
@@ -223,10 +143,19 @@ struct JITFrameInfoForBufferRange final {
 
 // Contains JITFrameInfoForBufferRange objects for multiple profiler buffer
 // ranges.
-struct JITFrameInfo final {
-  JITFrameInfo() : mUniqueStrings(mozilla::MakeUnique<UniqueJSONStrings>()) {}
+class JITFrameInfo final {
+ public:
+  JITFrameInfo()
+      : mUniqueStrings(mozilla::MakeUniqueFallible<UniqueJSONStrings>(
+            mLocalFailureLatchSource)) {
+    if (!mUniqueStrings) {
+      mLocalFailureLatchSource.SetFailure(
+          "OOM in JITFrameInfo allocating mUniqueStrings");
+    }
+  }
 
-  MOZ_IMPLICIT JITFrameInfo(const JITFrameInfo& aOther);
+  MOZ_IMPLICIT JITFrameInfo(const JITFrameInfo& aOther,
+                            mozilla::ProgressLogger aProgressLogger);
 
   // Creates a new JITFrameInfoForBufferRange object in mRanges by looking up
   // information about the provided JIT return addresses using aCx.
@@ -252,6 +181,23 @@ struct JITFrameInfo final {
     return mRanges.back().mRangeEnd <= aCurrentBufferRangeStart;
   }
 
+  mozilla::FailureLatch& LocalFailureLatchSource() {
+    return mLocalFailureLatchSource;
+  }
+
+  // The encapsulated data points at the local FailureLatch, so on the way out
+  // they must be given a new external FailureLatch to start using instead.
+  mozilla::Vector<JITFrameInfoForBufferRange>&& MoveRangesWithNewFailureLatch(
+      mozilla::FailureLatch& aFailureLatch) &&;
+  mozilla::UniquePtr<UniqueJSONStrings>&& MoveUniqueStringsWithNewFailureLatch(
+      mozilla::FailureLatch& aFailureLatch) &&;
+
+ private:
+  // JITFrameInfo's may exist during profiling, so it carries its own fallible
+  // FailureLatch. If&when the data below is finally extracted, any error is
+  // forwarded to the caller.
+  mozilla::FailureLatchSource mLocalFailureLatchSource;
+
   // The array of ranges of JIT frame information, sorted by buffer position.
   // Ranges are non-overlapping.
   // The JSON of the cached frames can contain string indexes, which refer
@@ -263,19 +209,20 @@ struct JITFrameInfo final {
   mozilla::UniquePtr<UniqueJSONStrings> mUniqueStrings;
 };
 
-class UniqueStacks {
+class UniqueStacks final : public mozilla::FailureLatch {
  public:
   struct FrameKey {
     explicit FrameKey(const char* aLocation)
-        : mData(NormalFrameData{nsCString(aLocation), false, 0,
+        : mData(NormalFrameData{nsCString(aLocation), false, false, 0,
                                 mozilla::Nothing(), mozilla::Nothing()}) {}
 
-    FrameKey(nsCString&& aLocation, bool aRelevantForJS,
+    FrameKey(nsCString&& aLocation, bool aRelevantForJS, bool aBaselineInterp,
              uint64_t aInnerWindowID, const mozilla::Maybe<unsigned>& aLine,
              const mozilla::Maybe<unsigned>& aColumn,
              const mozilla::Maybe<JS::ProfilingCategoryPair>& aCategoryPair)
-        : mData(NormalFrameData{aLocation, aRelevantForJS, aInnerWindowID,
-                                aLine, aColumn, aCategoryPair}) {}
+        : mData(NormalFrameData{aLocation, aRelevantForJS, aBaselineInterp,
+                                aInnerWindowID, aLine, aColumn,
+                                aCategoryPair}) {}
 
     FrameKey(void* aJITAddress, uint32_t aJITDepth, uint32_t aRangeIndex)
         : mData(JITFrameData{aJITAddress, aJITDepth, aRangeIndex}) {}
@@ -292,6 +239,7 @@ class UniqueStacks {
 
       nsCString mLocation;
       bool mRelevantForJS;
+      bool mBaselineInterp;
       uint64_t mInnerWindowID;
       mozilla::Maybe<unsigned> mLine;
       mozilla::Maybe<unsigned> mColumn;
@@ -320,6 +268,7 @@ class UniqueStacks {
                                     mozilla::HashString(data.mLocation.get()));
         }
         hash = mozilla::AddToHash(hash, data.mRelevantForJS);
+        hash = mozilla::AddToHash(hash, data.mBaselineInterp);
         hash = mozilla::AddToHash(hash, data.mInnerWindowID);
         if (data.mLine.isSome()) {
           hash = mozilla::AddToHash(hash, *data.mLine);
@@ -390,14 +339,16 @@ class UniqueStacks {
     }
   };
 
-  explicit UniqueStacks(JITFrameInfo&& aJITFrameInfo);
+  UniqueStacks(mozilla::FailureLatch& aFailureLatch,
+               JITFrameInfo&& aJITFrameInfo,
+               ProfilerCodeAddressService* aCodeAddressService = nullptr);
 
   // Return a StackKey for aFrame as the stack's root frame (no prefix).
-  [[nodiscard]] StackKey BeginStack(const FrameKey& aFrame);
+  [[nodiscard]] mozilla::Maybe<StackKey> BeginStack(const FrameKey& aFrame);
 
   // Return a new StackKey that is obtained by appending aFrame to aStack.
-  [[nodiscard]] StackKey AppendFrame(const StackKey& aStack,
-                                     const FrameKey& aFrame);
+  [[nodiscard]] mozilla::Maybe<StackKey> AppendFrame(const StackKey& aStack,
+                                                     const FrameKey& aFrame);
 
   // Look up frame keys for the given JIT address, and ensure that our frame
   // table has entries for the returned frame keys. The JSON for these frames
@@ -408,22 +359,36 @@ class UniqueStacks {
   LookupFramesForJITAddressFromBufferPos(void* aJITAddress,
                                          uint64_t aBufferPosition);
 
-  [[nodiscard]] uint32_t GetOrAddFrameIndex(const FrameKey& aFrame);
-  [[nodiscard]] uint32_t GetOrAddStackIndex(const StackKey& aStack);
+  [[nodiscard]] mozilla::Maybe<uint32_t> GetOrAddFrameIndex(
+      const FrameKey& aFrame);
+  [[nodiscard]] mozilla::Maybe<uint32_t> GetOrAddStackIndex(
+      const StackKey& aStack);
 
   void SpliceFrameTableElements(SpliceableJSONWriter& aWriter);
   void SpliceStackTableElements(SpliceableJSONWriter& aWriter);
 
+  [[nodiscard]] UniqueJSONStrings& UniqueStrings() {
+    MOZ_RELEASE_ASSERT(mUniqueStrings.get());
+    return *mUniqueStrings;
+  }
+
+  [[nodiscard]] nsCString& TracedValues() { return mTracedValues; }
+
+  // Find the function name at the given PC (if a ProfilerCodeAddressService was
+  // provided), otherwise just stringify that PC.
+  [[nodiscard]] nsAutoCString FunctionNameOrAddress(void* aPC);
+
+  FAILURELATCH_IMPL_PROXY(mFrameTableWriter)
+
  private:
   void StreamNonJITFrame(const FrameKey& aFrame);
   void StreamStack(const StackKey& aStack);
 
- public:
   mozilla::UniquePtr<UniqueJSONStrings> mUniqueStrings;
+  nsCString mTracedValues;
 
   ProfilerCodeAddressService* mCodeAddressService = nullptr;
 
- private:
   SpliceableChunkedJSONWriter mFrameTableWriter;
   mozilla::HashMap<FrameKey, uint32_t, FrameKeyHasher> mFrameToIndexMap;
 
@@ -460,6 +425,7 @@ class UniqueStacks {
 //       "stack": 0,          /* index into stackTable */
 //       "time": 1,           /* number */
 //       "eventDelay": 2,     /* number */
+//       "ThreadCPUDelta": 3, /* optional number */
 //     },
 //     "data":
 //     [
@@ -503,17 +469,16 @@ class UniqueStacks {
 //       "relevantForJS": 1,  /* bool */
 //       "innerWindowID": 2,  /* inner window ID of global JS `window` object */
 //       "implementation": 3, /* index into stringTable */
-//       "optimizations": 4,  /* arbitrary JSON */
-//       "line": 5,           /* number */
-//       "column": 6,         /* number */
-//       "category": 7,       /* index into profile.meta.categories */
-//       "subcategory": 8     /* index into
+//       "line": 4,           /* number */
+//       "column": 5,         /* number */
+//       "category": 6,       /* index into profile.meta.categories */
+//       "subcategory": 7     /* index into
 //       profile.meta.categories[category].subcategories */
 //     },
 //     "data":
 //     [
 //       [ 0 ],               /* { location: '(root)' } */
-//       [ 1, 2 ]             /* { location: 'foo.js',
+//       [ 1, null, null, 2 ] /* { location: 'foo.js',
 //                                 implementation: 'baseline' } */
 //     ]
 //   },
