@@ -1,9 +1,9 @@
-function run_test() {
+add_task(async function test() {
   Assert.ok(!Services.profiler.IsActive());
 
-  Services.profiler.StartProfiler(100, 10, ["js"]);
+  await Services.profiler.StartProfiler(100, 10, ["js"]);
   // The function is entered with the profiler enabled
-  (function() {
+  (function () {
     Services.profiler.StopProfiler();
     let n = 10000;
     // eslint-disable-next-line no-empty
@@ -11,4 +11,4 @@ function run_test() {
     // An assertion will fail when this function returns, if the
     // profiler stack was misbalanced.
   })();
-}
+});
