@@ -10,7 +10,6 @@ from argparse import ArgumentParser
 
 from .task_config import all_task_configs
 
-
 COMMON_ARGUMENT_GROUPS = {
     "push": [
         [
@@ -23,16 +22,6 @@ COMMON_ARGUMENT_GROUPS = {
                 "$EDITOR if blank. Defaults to computed message.",
             },
         ],
-        [
-            ["--no-push"],
-            {
-                "dest": "push",
-                "action": "store_false",
-                "help": "Do not push to try as a result of running this command (if "
-                "specified this command will only print calculated try "
-                "syntax and selection info).",
-            },
-        ],
         [
             ["--closed-tree"],
             {
@@ -41,6 +30,17 @@ COMMON_ARGUMENT_GROUPS = {
                 "help": "Push despite a closed try tree",
             },
         ],
+        [
+            ["--push-to-vcs"],
+            {
+                "action": "store_true",
+                "default": False,
+                "help": (
+                    "Submit changes directly to VCS instead of Lando. "
+                    "Set `MACH_TRY_PUSH_TO_VCS=1` in the environment to force this."
+                ),
+            },
+        ],
     ],
     "preset": [
         [
@@ -99,6 +99,28 @@ COMMON_ARGUMENT_GROUPS = {
     ],
 }
 
+NO_PUSH_ARGUMENT_GROUP = [
+    [
+        ["--stage-changes"],
+        {
+            "dest": "stage_changes",
+            "action": "store_true",
+            "help": "Locally stage changes created by this command but do not "
+            "push to try.",
+        },
+    ],
+    [
+        ["--no-push"],
+        {
+            "dest": "dry_run",
+            "action": "store_true",
+            "help": "Do not push to try as a result of running this command (if "
+            "specified this command will only print calculated task "
+            "selection info and not change files).",
+        },
+    ],
+]
+
 
 class BaseTryParser(ArgumentParser):
     name = "try"
@@ -109,12 +131,12 @@ class BaseTryParser(ArgumentParser):
     def __init__(self, *args, **kwargs):
         ArgumentParser.__init__(self, *args, **kwargs)
 
-        group = self.add_argument_group("{} arguments".format(self.name))
+        group = self.add_argument_group(f"{self.name} arguments")
         for cli, kwargs in self.arguments:
             group.add_argument(*cli, **kwargs)
 
         for name in self.common_groups:
-            group = self.add_argument_group("{} arguments".format(name))
+            group = self.add_argument_group(f"{name} arguments")
             arguments = COMMON_ARGUMENT_GROUPS[name]
 
             # Preset arguments are all mutually exclusive.
@@ -124,6 +146,12 @@ class BaseTryParser(ArgumentParser):
             for cli, kwargs in arguments:
                 group.add_argument(*cli, **kwargs)
 
+            if name == "push":
+                group_no_push = group.add_mutually_exclusive_group()
+                arguments = NO_PUSH_ARGUMENT_GROUP
+                for cli, kwargs in arguments:
+                    group_no_push.add_argument(*cli, **kwargs)
+
         group = self.add_argument_group("task configuration arguments")
         self.task_configs = {c: all_task_configs[c]() for c in self.task_configs}
         for cfg in self.task_configs.values():
