@@ -8,9 +8,8 @@ import importlib
 import os
 import sys
 
-from mach.decorators import Command, SettingsProvider, SubCommand
+from mach.decorators import Command, SubCommand
 from mach.util import get_state_dir
-from mozbuild.base import BuildEnvironmentNotFoundException
 from mozbuild.util import memoize
 
 CONFIG_ENVIRONMENT_NOT_FOUND = """
@@ -28,8 +27,8 @@ class get_parser:
         self.selector = selector
 
     def __call__(self):
-        mod = importlib.import_module("tryselect.selectors.{}".format(self.selector))
-        return getattr(mod, "{}Parser".format(self.selector.capitalize()))()
+        mod = importlib.import_module(f"tryselect.selectors.{self.selector}")
+        return getattr(mod, f"{self.selector.capitalize()}Parser")()
 
 
 def generic_parser():
@@ -40,32 +39,12 @@ def generic_parser():
     return parser
 
 
-@SettingsProvider
-class TryConfig:
-    @classmethod
-    def config_settings(cls):
-        from mach.registrar import Registrar
-
-        desc = (
-            "The default selector to use when running `mach try` without a subcommand."
-        )
-        choices = Registrar.command_handlers["try"].subcommand_handlers.keys()
-
-        return [
-            ("try.default", "string", desc, "auto", {"choices": choices}),
-            (
-                "try.maxhistory",
-                "int",
-                "Maximum number of pushes to save in history.",
-                10,
-            ),
-        ]
-
-
 def init(command_context):
-    from tryselect import push
+    from tryselect import lando, push
 
-    push.MAX_HISTORY = command_context._mach_context.settings["try"]["maxhistory"]
+    mach_context = command_context._mach_context
+    lando.LAUNCH_BROWSER = not mach_context.settings["try"]["nobrowser"]
+    push.MAX_HISTORY = mach_context.settings["try"]["maxhistory"]
 
 
 @memoize
@@ -123,13 +102,13 @@ def handle_presets(
         # Only save non-default values for simplicity.
         kwargs = {k: v for k, v in kwargs.items() if v != default(k)}
         user_presets.save(save, selector=selector, **kwargs)
-        print("preset saved, run with: --preset={}".format(save))
+        print(f"preset saved, run with: --preset={save}")
         sys.exit()
 
     if preset:
         if preset not in presets(command_context):
             command_context._mach_context.parser.error(
-                "preset '{}' does not exist".format(preset)
+                f"preset '{preset}' does not exist"
             )
 
         name = preset
@@ -141,8 +120,8 @@ def handle_presets(
             subcommand = selector
         elif subcommand != selector:
             print(
-                "error: preset '{}' exists for a different selector "
-                "(did you mean to run 'mach try {}' instead?)".format(name, selector)
+                f"error: preset '{name}' exists for a different selector "
+                f"(did you mean to run 'mach try {selector}' instead?)"
             )
             sys.exit(1)
 
@@ -161,16 +140,16 @@ def handle_presets(
     return kwargs
 
 
-def handle_try_config(command_context, **kwargs):
+def handle_try_params(command_context, **kwargs):
     from tryselect.util.dicttools import merge
 
     to_validate = []
-    kwargs.setdefault("try_config", {})
+    kwargs.setdefault("try_config_params", {})
     for cls in command_context._mach_context.handler.parser.task_configs.values():
-        try_config = cls.try_config(**kwargs)
-        if try_config is not None:
+        params = cls.get_parameters(**kwargs)
+        if params is not None:
             to_validate.append(cls)
-            kwargs["try_config"] = merge(kwargs["try_config"], try_config)
+            kwargs["try_config_params"] = merge(kwargs["try_config_params"], params)
 
         for name in cls.dests:
             del kwargs[name]
@@ -186,12 +165,10 @@ def run(command_context, **kwargs):
     kwargs = handle_presets(command_context, **kwargs)
 
     if command_context._mach_context.handler.parser.task_configs:
-        kwargs = handle_try_config(command_context, **kwargs)
+        kwargs = handle_try_params(command_context, **kwargs)
 
     mod = importlib.import_module(
-        "tryselect.selectors.{}".format(
-            command_context._mach_context.handler.subcommand
-        )
+        f"tryselect.selectors.{command_context._mach_context.handler.subcommand}"
     )
     return mod.run(**kwargs)
 
@@ -201,6 +178,7 @@ def run(command_context, **kwargs):
     category="ci",
     description="Push selected tasks to the try server",
     parser=generic_parser,
+    virtualenv_name="try",
 )
 def try_default(command_context, argv=None, **kwargs):
     """Push selected tests to the try server.
@@ -222,14 +200,19 @@ def try_default(command_context, argv=None, **kwargs):
     if preset:
         if preset not in presets(command_context):
             command_context._mach_context.handler.parser.error(
-                "preset '{}' does not exist".format(preset)
+                f"preset '{preset}' does not exist"
             )
 
         subcommand = presets(command_context)[preset]["selector"]
 
     sub = subcommand or command_context._mach_context.settings["try"]["default"]
     return command_context._mach_context.commands.dispatch(
-        "try", command_context._mach_context, subcommand=sub, argv=argv, **kwargs
+        "try",
+        command_context._mach_context,
+        command_site_manager=command_context.virtualenv_manager,
+        subcommand=sub,
+        argv=argv,
+        **kwargs,
     )
 
 
@@ -238,6 +221,7 @@ def try_default(command_context, argv=None, **kwargs):
     "fuzzy",
     description="Select tasks on try using a fuzzy finder",
     parser=get_parser("fuzzy"),
+    virtualenv_name="try",
 )
 def try_fuzzy(command_context, **kwargs):
     """Select which tasks to run with a fuzzy finding interface (fzf).
@@ -332,9 +316,18 @@ def try_fuzzy(command_context, **kwargs):
         if not kwargs["query"]:
             return
 
+    if kwargs.get("tag") and kwargs.get("paths"):
+        command_context._mach_context.handler.parser.error(
+            "ERROR: only --tag or <path> can be used, not both."
+        )
+        return
+
     if kwargs.get("paths"):
         kwargs["test_paths"] = kwargs["paths"]
 
+    if kwargs.get("tag"):
+        kwargs["test_tag"] = kwargs["tag"]
+
     return run(command_context, **kwargs)
 
 
@@ -343,6 +336,7 @@ def try_fuzzy(command_context, **kwargs):
     "chooser",
     description="Schedule tasks by selecting them from a web interface.",
     parser=get_parser("chooser"),
+    virtualenv_name="try",
 )
 def try_chooser(command_context, **kwargs):
     """Push tasks selected from a web interface to try.
@@ -354,10 +348,6 @@ def try_chooser(command_context, **kwargs):
     """
     init(command_context)
     command_context.activate_virtualenv()
-    path = os.path.join(
-        "tools", "tryselect", "selectors", "chooser", "requirements.txt"
-    )
-    command_context.virtualenv_manager.install_pip_requirements(path, quiet=True)
 
     return run(command_context, **kwargs)
 
@@ -369,6 +359,7 @@ def try_chooser(command_context, **kwargs):
     "set of tasks that would be run on autoland. This "
     "selector is EXPERIMENTAL.",
     parser=get_parser("auto"),
+    virtualenv_name="try",
 )
 def try_auto(command_context, **kwargs):
     init(command_context)
@@ -378,8 +369,9 @@ def try_auto(command_context, **kwargs):
 @SubCommand(
     "try",
     "again",
-    description="Schedule a previously generated (non try syntax) push again.",
+    description="Schedule a previously generated push again.",
     parser=get_parser("again"),
+    virtualenv_name="try",
 )
 def try_again(command_context, **kwargs):
     init(command_context)
@@ -391,6 +383,7 @@ def try_again(command_context, **kwargs):
     "empty",
     description="Push to try without scheduling any tasks.",
     parser=get_parser("empty"),
+    virtualenv_name="try",
 )
 def try_empty(command_context, **kwargs):
     """Push to try, running no builds or tests
@@ -405,73 +398,12 @@ def try_empty(command_context, **kwargs):
     return run(command_context, **kwargs)
 
 
-@SubCommand(
-    "try",
-    "syntax",
-    description="Select tasks on try using try syntax",
-    parser=get_parser("syntax"),
-)
-def try_syntax(command_context, **kwargs):
-    """Push the current tree to try, with the specified syntax.
-
-    Build options, platforms and regression tests may be selected
-    using the usual try options (-b, -p and -u respectively). In
-    addition, tests in a given directory may be automatically
-    selected by passing that directory as a positional argument to the
-    command. For example:
-
-    mach try -b d -p linux64 dom testing/web-platform/tests/dom
-
-    would schedule a try run for linux64 debug consisting of all
-    tests under dom/ and testing/web-platform/tests/dom.
-
-    Test selection using positional arguments is available for
-    mochitests, reftests, xpcshell tests and web-platform-tests.
-
-    Tests may be also filtered by passing --tag to the command,
-    which will run only tests marked as having the specified
-    tags e.g.
-
-    mach try -b d -p win64 --tag media
-
-    would run all tests tagged 'media' on Windows 64.
-
-    If both positional arguments or tags and -u are supplied, the
-    suites in -u will be run in full. Where tests are selected by
-    positional argument they will be run in a single chunk.
-
-    If no build option is selected, both debug and opt will be
-    scheduled. If no platform is selected a default is taken from
-    the AUTOTRY_PLATFORM_HINT environment variable, if set.
-
-    The command requires either its own mercurial extension ("push-to-try",
-    installable from mach vcs-setup) or a git repo using git-cinnabar
-    (installable from mach vcs-setup).
-
-    """
-    init(command_context)
-    try:
-        if command_context.substs.get("MOZ_ARTIFACT_BUILDS"):
-            kwargs["local_artifact_build"] = True
-    except BuildEnvironmentNotFoundException:
-        # If we don't have a build locally, we can't tell whether
-        # an artifact build is desired, but we still want the
-        # command to succeed, if possible.
-        pass
-
-    config_status = os.path.join(command_context.topobjdir, "config.status")
-    if (kwargs["paths"] or kwargs["tags"]) and not config_status:
-        print(CONFIG_ENVIRONMENT_NOT_FOUND)
-        sys.exit(1)
-
-    return run(command_context, **kwargs)
-
-
 @SubCommand(
     "try",
     "coverage",
     description="Select tasks on try using coverage data",
     parser=get_parser("coverage"),
+    virtualenv_name="try",
 )
 def try_coverage(command_context, **kwargs):
     """Select which tasks to use using coverage data."""
@@ -484,6 +416,7 @@ def try_coverage(command_context, **kwargs):
     "release",
     description="Push the current tree to try, configured for a staging release.",
     parser=get_parser("release"),
+    virtualenv_name="try",
 )
 def try_release(command_context, **kwargs):
     """Push the current tree to try, configured for a staging release."""
@@ -496,6 +429,7 @@ def try_release(command_context, **kwargs):
     "scriptworker",
     description="Run scriptworker tasks against a recent release.",
     parser=get_parser("scriptworker"),
+    virtualenv_name="try",
 )
 def try_scriptworker(command_context, **kwargs):
     """Run scriptworker tasks against a recent release.
@@ -511,6 +445,7 @@ def try_scriptworker(command_context, **kwargs):
     "compare",
     description="Push two try jobs, one on your current commit and another on the one you specify",
     parser=get_parser("compare"),
+    virtualenv_name="try",
 )
 def try_compare(command_context, **kwargs):
     init(command_context)
@@ -522,6 +457,7 @@ def try_compare(command_context, **kwargs):
     "perf",
     description="Try selector for running performance tests.",
     parser=get_parser("perf"),
+    virtualenv_name="try",
 )
 def try_perf(command_context, **kwargs):
     init(command_context)
