@@ -7,11 +7,12 @@ from taskgraph.util.python_path import find_object
 
 from ..cli import BaseTryParser
 from ..push import push_to_try
+from ..util.dicttools import merge
 
 TRY_AUTO_PARAMETERS = {
+    "filters": ["try_auto"],
     "optimize_strategies": "gecko_taskgraph.optimize:tryselect.bugbug_reduced_manifests_config_selection_medium",  # noqa
     "optimize_target_tasks": True,
-    "target_tasks_method": "try_auto",
     "test_manifest_loader": "bugbug",
     "try_mode": "try_auto",
     "try_task_config": {},
@@ -63,17 +64,15 @@ class AutoParser(BaseTryParser):
 
         if args.strategy:
             if ":" not in args.strategy:
-                args.strategy = "gecko_taskgraph.optimize:tryselect.{}".format(
-                    args.strategy
-                )
+                args.strategy = f"gecko_taskgraph.optimize:tryselect.{args.strategy}"
 
             try:
                 obj = find_object(args.strategy)
             except (ImportError, AttributeError):
-                self.error("invalid module path '{}'".format(args.strategy))
+                self.error(f"invalid module path '{args.strategy}'")
 
             if not isinstance(obj, dict):
-                self.error("object at '{}' must be a dict".format(args.strategy))
+                self.error(f"object at '{args.strategy}' must be a dict")
 
 
 def run(
@@ -84,14 +83,15 @@ def run(
     strategy=None,
     tasks_regex=None,
     tasks_regex_exclude=None,
-    try_config=None,
-    **ignored
+    try_config_params=None,
+    push_to_vcs=False,
+    **ignored,
 ):
     msg = message.format(msg="Tasks automatically selected.")
 
     params = TRY_AUTO_PARAMETERS.copy()
-    if try_config:
-        params["try_task_config"] = try_config
+    if try_config_params:
+        params = merge(params, try_config_params)
 
     if strategy:
         params["optimize_strategies"] = strategy
@@ -112,4 +112,5 @@ def run(
         stage_changes=stage_changes,
         dry_run=dry_run,
         closed_tree=closed_tree,
+        push_to_vcs=push_to_vcs,
     )
