@@ -7,26 +7,28 @@ Templates provide a way of modifying the task definition of selected tasks.
 They are added to 'try_task_config.json' and processed by the transforms.
 """
 
-
 import json
 import os
+import pathlib
 import subprocess
 import sys
-from abc import ABCMeta, abstractmethod, abstractproperty
+from abc import ABCMeta, abstractmethod
 from argparse import SUPPRESS, Action
 from textwrap import dedent
 
 import mozpack.path as mozpath
-import six
+import requests
 from mozbuild.base import BuildEnvironmentNotFoundException, MozbuildObject
+from taskgraph.util import taskcluster
 
 from .tasks import resolve_tests_by_suite
+from .util.ssh import get_ssh_user
 
-here = os.path.abspath(os.path.dirname(__file__))
-build = MozbuildObject.from_environment(cwd=here)
+here = pathlib.Path(__file__).parent
+build = MozbuildObject.from_environment(cwd=str(here))
 
 
-class TryConfig:
+class ParameterConfig:
     __metaclass__ = ABCMeta
 
     def __init__(self):
@@ -37,20 +39,45 @@ class TryConfig:
             action = parser.add_argument(*cli, **kwargs)
             self.dests.add(action.dest)
 
-    @abstractproperty
+    @property
+    @abstractmethod
     def arguments(self):
         pass
 
     @abstractmethod
-    def try_config(self, **kwargs):
+    def get_parameters(self, **kwargs) -> dict:
         pass
 
     def validate(self, **kwargs):
         pass
 
 
-class Artifact(TryConfig):
+class TargetTasksMethod(ParameterConfig):
+    arguments = [
+        [
+            ["--target-tasks-method"],
+            {"help": "Custom target tasks method to use."},
+        ],
+    ]
 
+    def get_parameters(self, target_tasks_method: str, **kwargs):
+        if target_tasks_method:
+            return {"target_tasks_method": target_tasks_method}
+
+
+class TryConfig(ParameterConfig):
+    @abstractmethod
+    def try_config(self, **kwargs) -> dict:
+        pass
+
+    def get_parameters(self, **kwargs):
+        result = self.try_config(**kwargs)
+        if not result:
+            return None
+        return {"try_task_config": result}
+
+
+class Artifact(TryConfig):
     arguments = [
         [
             ["--artifact"],
@@ -104,7 +131,7 @@ class Pernosco(TryConfig):
                 "dest": "pernosco",
                 "action": "store_false",
                 "default": None,
-                "help": "Opt-out of the Pernosco debugging service (if you are on the whitelist).",
+                "help": "Opt-out of the Pernosco debugging service (if you are on the include list).",
             },
         ],
     ]
@@ -124,24 +151,16 @@ class Pernosco(TryConfig):
                 # log in. Prevent people with non-Mozilla addresses from using this
                 # flag so they don't end up consuming time and resources only to
                 # realize they can't actually log in and see the reports.
-                cmd = ["ssh", "-G", "hg.mozilla.org"]
-                output = subprocess.check_output(
-                    cmd, universal_newlines=True
-                ).splitlines()
-                address = [
-                    l.rsplit(" ", 1)[-1] for l in output if l.startswith("user")
-                ][0]
+                address = get_ssh_user()
                 if not address.endswith("@mozilla.com"):
                     print(
                         dedent(
-                            """\
+                            f"""\
                         Pernosco requires a Mozilla e-mail address to view its reports. Please
                         push to try with an @mozilla.com address to use --pernosco.
 
-                            Current user: {}
-                    """.format(
-                                address
-                            )
+                            Current user: {address}
+                    """
                         )
                     )
                     sys.exit(1)
@@ -168,7 +187,8 @@ class Pernosco(TryConfig):
         }
 
     def validate(self, **kwargs):
-        if kwargs["try_config"].get("use-artifact-builds"):
+        try_config = kwargs["try_config_params"].get("try_task_config") or {}
+        if try_config.get("use-artifact-builds"):
             print(
                 "Pernosco does not support artifact builds at this time. "
                 "Please try again with '--no-artifact'."
@@ -177,7 +197,6 @@ class Pernosco(TryConfig):
 
 
 class Path(TryConfig):
-
     arguments = [
         [
             ["paths"],
@@ -193,26 +212,56 @@ class Path(TryConfig):
         if not paths:
             return
 
-        for p in paths:
+        for i, p in enumerate(paths):
             if not os.path.exists(p):
-                print("error: '{}' is not a valid path.".format(p), file=sys.stderr)
+                print(f"error: '{p}' is not a valid path.", file=sys.stderr)
                 sys.exit(1)
 
-        paths = [
-            mozpath.relpath(mozpath.join(os.getcwd(), p), build.topsrcdir)
-            for p in paths
-        ]
+            # Passing paths to specific tests doesn't work with the Treeherder
+            # test path filter or test-verify. Re-write it to the containing
+            # directory to avoid confusion.
+            if os.path.isfile(p):
+                parent = os.path.dirname(p)
+                print(
+                    f"warning: paths to individual tests don't work, re-writing to {parent}"
+                )
+                paths[i] = parent
+
+            paths[i] = mozpath.relpath(
+                mozpath.join(os.getcwd(), paths[i]), build.topsrcdir
+            )
+
         return {
             "env": {
-                "MOZHARNESS_TEST_PATHS": six.ensure_text(
-                    json.dumps(resolve_tests_by_suite(paths))
-                ),
+                "MOZHARNESS_TEST_PATHS": json.dumps(resolve_tests_by_suite(paths)),
+            }
+        }
+
+
+class Tag(TryConfig):
+    arguments = [
+        [
+            ["--tag"],
+            {
+                "action": "append",
+                "default": [],
+                "help": "Run tests matching the specified tag.",
+            },
+        ],
+    ]
+
+    def try_config(self, tag, **kwargs):
+        if not tag:
+            return
+
+        return {
+            "env": {
+                "MOZHARNESS_TEST_TAG": json.dumps(tag),
             }
         }
 
 
 class Environment(TryConfig):
-
     arguments = [
         [
             ["--env"],
@@ -223,9 +272,20 @@ class Environment(TryConfig):
                 "Can be passed in multiple times.",
             },
         ],
+        [
+            ["--record"],
+            {
+                "action": "store_true",
+                "help": "Get a screen recording of the tests where possible.",
+            },
+        ],
     ]
 
-    def try_config(self, env, **kwargs):
+    def try_config(self, env, record, **kwargs):
+        if env is None:
+            env = []
+        if record:
+            env.append("MOZ_RECORD_TEST=1")
         if not env:
             return
         return {
@@ -233,24 +293,87 @@ class Environment(TryConfig):
         }
 
 
+class ExistingTasks(ParameterConfig):
+    TREEHERDER_PUSH_ENDPOINT = (
+        "https://treeherder.mozilla.org/api/project/try/push/?count=1&author={user}"
+    )
+    TREEHERDER_PUSH_URL = (
+        "https://treeherder.mozilla.org/jobs?repo={branch}&revision={revision}"
+    )
+
+    arguments = [
+        [
+            ["-E", "--use-existing-tasks"],
+            {
+                "const": "last_try_push",
+                "default": None,
+                "nargs": "?",
+                "help": """
+                    Use existing tasks from a previous push. Without args this
+                uses your most recent try push. You may also specify
+                `rev=<revision>` where <revision> is the head revision of the
+                try push or `task-id=<task id>` where <task id> is the Decision
+                task id of the push. This last method even works for non-try
+                branches.
+                """,
+            },
+        ]
+    ]
+
+    def find_decision_task(self, use_existing_tasks):
+        branch = "try"
+        if use_existing_tasks == "last_try_push":
+            # Use existing tasks from user's previous try push.
+            user = get_ssh_user()
+            url = self.TREEHERDER_PUSH_ENDPOINT.format(user=user)
+            res = requests.get(url, headers={"User-Agent": "gecko-mach-try/1.0"})
+            res.raise_for_status()
+            data = res.json()
+            if data["meta"]["count"] == 0:
+                raise Exception(f"Could not find a try push for '{user}'!")
+            revision = data["results"][0]["revision"]
+
+        elif use_existing_tasks.startswith("rev="):
+            revision = use_existing_tasks[len("rev=") :]
+
+        else:
+            raise Exception("Unable to parse '{use_existing_tasks}'!")
+
+        url = self.TREEHERDER_PUSH_URL.format(branch=branch, revision=revision)
+        print(f"Using existing tasks from: {url}")
+        index_path = f"gecko.v2.{branch}.revision.{revision}.taskgraph.decision"
+        return taskcluster.find_task_id(index_path)
+
+    def get_parameters(self, use_existing_tasks, **kwargs):
+        if not use_existing_tasks:
+            return
+
+        if use_existing_tasks.startswith("task-id="):
+            tid = use_existing_tasks[len("task-id=") :]
+        else:
+            tid = self.find_decision_task(use_existing_tasks)
+
+        label_to_task_id = taskcluster.get_artifact(tid, "public/label-to-taskid.json")
+        return {"existing_tasks": label_to_task_id}
+
+
 class RangeAction(Action):
     def __init__(self, min, max, *args, **kwargs):
         self.min = min
         self.max = max
-        kwargs["metavar"] = "[{}-{}]".format(self.min, self.max)
+        kwargs["metavar"] = f"[{self.min}-{self.max}]"
         super().__init__(*args, **kwargs)
 
     def __call__(self, parser, namespace, values, option_string=None):
         name = option_string or self.dest
         if values < self.min:
-            parser.error("{} can not be less than {}".format(name, self.min))
+            parser.error(f"{name} can not be less than {self.min}")
         if values > self.max:
-            parser.error("{} can not be more than {}".format(name, self.max))
+            parser.error(f"{name} can not be more than {self.max}")
         setattr(namespace, self.dest, values)
 
 
 class Rebuild(TryConfig):
-
     arguments = [
         [
             ["--rebuild"],
@@ -269,7 +392,11 @@ class Rebuild(TryConfig):
         if not rebuild:
             return
 
-        if kwargs.get("full") and rebuild > 3:
+        if (
+            not kwargs.get("new_test_config", False)
+            and kwargs.get("full")
+            and rebuild > 3
+        ):
             print(
                 "warning: limiting --rebuild to 3 when using --full. "
                 "Use custom push actions to add more."
@@ -304,7 +431,6 @@ class Routes(TryConfig):
 
 
 class ChemspillPrio(TryConfig):
-
     arguments = [
         [
             ["--chemspill-prio"],
@@ -317,7 +443,7 @@ class ChemspillPrio(TryConfig):
 
     def try_config(self, chemspill_prio, **kwargs):
         if chemspill_prio:
-            return {"chemspill-prio": {}}
+            return {"chemspill-prio": True}
 
 
 class GeckoProfile(TryConfig):
@@ -328,7 +454,10 @@ class GeckoProfile(TryConfig):
                 "dest": "profile",
                 "action": "store_true",
                 "default": False,
-                "help": "Create and upload a gecko profile during talos/raptor tasks.",
+                "help": (
+                    "Create and upload a gecko profile during talos/raptor tasks. "
+                    "Copy paste the parameters used in this profiling run directly from about:profiling in Nightly."
+                ),
             },
         ],
         [
@@ -374,7 +503,7 @@ class GeckoProfile(TryConfig):
                 "help": SUPPRESS,
             },
         ],
-        # This is added for consistency with the 'syntax' selector
+        # This is added for consistency with the old 'syntax' selector
         [
             ["--geckoProfile"],
             {
@@ -393,7 +522,7 @@ class GeckoProfile(TryConfig):
         gecko_profile_entries,
         gecko_profile_features,
         gecko_profile_threads,
-        **kwargs
+        **kwargs,
     ):
         if profile or not all(
             s is None for s in (gecko_profile_features, gecko_profile_threads)
@@ -427,7 +556,6 @@ class Browsertime(TryConfig):
 
 
 class DisablePgo(TryConfig):
-
     arguments = [
         [
             ["--disable-pgo"],
@@ -445,8 +573,25 @@ class DisablePgo(TryConfig):
             }
 
 
-class WorkerOverrides(TryConfig):
+class NewConfig(TryConfig):
+    arguments = [
+        [
+            ["--new-test-config"],
+            {
+                "action": "store_true",
+                "help": "When a test fails (mochitest only) restart the browser and start from the next test",
+            },
+        ],
+    ]
 
+    def try_config(self, new_test_config, **kwargs):
+        if new_test_config:
+            return {
+                "new-test-config": True,
+            }
+
+
+class WorkerOverrides(TryConfig):
     arguments = [
         [
             ["--worker-override"],
@@ -473,50 +618,88 @@ class WorkerOverrides(TryConfig):
                 ),
             },
         ],
+        [
+            ["--worker-type"],
+            {
+                "action": "append",
+                "dest": "worker_types",
+                "default": [],
+                "help": "Select tasks that only run on the specified worker.",
+            },
+        ],
     ]
 
-    def try_config(self, worker_overrides, worker_suffixes, **kwargs):
+    def try_config(self, worker_overrides, worker_suffixes, worker_types, **kwargs):
         from gecko_taskgraph.util.workertypes import get_worker_type
         from taskgraph.config import load_graph_config
 
+        root = build.topsrcdir
+        root = os.path.join(root, "taskcluster")
+        graph_config = load_graph_config(root)
+
         overrides = {}
         if worker_overrides:
             for override in worker_overrides:
                 alias, worker_pool = override.split("=", 1)
                 if alias in overrides:
                     print(
-                        "Can't override worker alias {alias} more than once. "
-                        "Already set to use {previous}, but also asked to use {new}.".format(
-                            alias=alias, previous=overrides[alias], new=worker_pool
-                        )
+                        f"Can't override worker alias {alias} more than once. "
+                        f"Already set to use {overrides[alias]}, but also asked to use {worker_pool}."
                     )
                     sys.exit(1)
                 overrides[alias] = worker_pool
 
+                try:
+                    provisioner, worker_type = get_worker_type(
+                        graph_config, worker_type=alias, parameters={"level": "1"}
+                    )
+                except KeyError:
+                    print(
+                        f"Invalid worker type {alias}, use a value that matches below (limited to b-*, t-*, win*):"
+                    )
+                    root_alias = alias.strip("gecko-")
+                    root_alias = root_alias.strip("t-")
+                    root_alias = root_alias.split("-")[0]
+                    possible_matches = []
+                    for item in graph_config["workers"]["aliases"]:
+                        if root_alias in item:
+                            possible_matches.append(item)
+                        if (
+                            item.startswith("t-")
+                            or item.startswith("b-")
+                            or item.startswith("win")
+                        ):
+                            print(f"{item}")
+
+                    print("")
+                    if len(possible_matches) == 1:
+                        print(f"did you mean: {possible_matches[0]}")
+                    elif len(possible_matches) > 1:
+                        print(f"did you mean one of these {possible_matches}")
+                    sys.exit(1)
+
         if worker_suffixes:
-            root = build.topsrcdir
-            root = os.path.join(root, "taskcluster", "ci")
-            graph_config = load_graph_config(root)
             for worker_suffix in worker_suffixes:
                 alias, suffix = worker_suffix.split("=", 1)
                 if alias in overrides:
                     print(
-                        "Can't override worker alias {alias} more than once. "
-                        "Already set to use {previous}, but also asked "
-                        "to add suffix {suffix}.".format(
-                            alias=alias, previous=overrides[alias], suffix=suffix
-                        )
+                        f"Can't override worker alias {alias} more than once. "
+                        f"Already set to use {overrides[alias]}, but also asked "
+                        f"to add suffix {suffix}."
                     )
                     sys.exit(1)
                 provisioner, worker_type = get_worker_type(
                     graph_config, worker_type=alias, parameters={"level": "1"}
                 )
-                overrides[alias] = "{provisioner}/{worker_type}{suffix}".format(
-                    provisioner=provisioner, worker_type=worker_type, suffix=suffix
-                )
+                overrides[alias] = f"{provisioner}/{worker_type}{suffix}"
+
+        retVal = {}
+        if worker_types:
+            retVal["worker-types"] = list(overrides.keys()) + worker_types
 
         if overrides:
-            return {"worker-overrides": overrides}
+            retVal["worker-overrides"] = overrides
+        return retVal
 
 
 all_task_configs = {
@@ -525,10 +708,14 @@ all_task_configs = {
     "chemspill-prio": ChemspillPrio,
     "disable-pgo": DisablePgo,
     "env": Environment,
+    "existing-tasks": ExistingTasks,
     "gecko-profile": GeckoProfile,
+    "new-test-config": NewConfig,
     "path": Path,
+    "test-tag": Tag,
     "pernosco": Pernosco,
     "rebuild": Rebuild,
     "routes": Routes,
+    "target-tasks-method": TargetTasksMethod,
     "worker-overrides": WorkerOverrides,
 }
